/**
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 **/
package org.objectweb.orchestra.demo.college;

import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;


/**
 * CollegeSessionBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: CollegeSessionBean.java,v 1.1 2006/10/04 15:33:24 lejeuneg Exp $
 *
 * @ejb:bean name="CollegeSessionBean"
 *           display-name="CollegeSessionBean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="org/objectweb/orchestra/demo/college/CollegeSessionHome"
 *           view-type="remote"
 *
 * @ejb.home remote-class="org.objectweb.orchestra.demo.college.CollegeSessionHome"
 * @ejb.interface remote-class="org.objectweb.orchestra.demo.college.CollegeSession"
 *
 * @ejb:transaction type="NotSupported"
 *
 *
 */
public class CollegeSessionBean implements SessionBean {

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------
    private SessionContext mContext;
    private CollegeUserHomeLocal cuhl = null;
    private CollegeUserLocal cul = null;

    // -------------------------------------------------------------------------
    // Methods - SessionBean implementation
    // -------------------------------------------------------------------------

    /**
     * Create the All Projects Session Bean
     *
     * @throws CreateException
     *
     * @ejb:create-method
     */
    public void ejbCreate() throws CreateException, Exception {
        try {
            cuhl = CollegeUserBeanUtil.getLocalHome();
        } catch (NamingException e) {
            throw new Exception("NamingException CollegeDemo CollegeSessionBean constructor :\n", e);
        }
    }

    public void ejbPostCreate() throws CreateException {
    }

    public void setSessionContext(final SessionContext context) {
        mContext = context;
    }

    public void ejbRemove() {
    }

    public void ejbActivate() {
    }

    public void ejbPassivate() {
    }

    // -------------------------------------------------------------------------
    // Methods - public
    // -------------------------------------------------------------------------

    /**
     * @ejb:interface-method
     */
    public void addUser(String name, String firstName, String number, String street, String cp, String city, String classe, String station) throws Exception {
	    cuhl.create(name, firstName, number, street, cp, city, classe, station);
    }

    /**
     * @ejb:interface-method
     */
	public boolean archiveExist(String collegeRecordNb, String name)throws Exception{
	    boolean result = true;
	    try {
	        cuhl.findByPrimaryKey(new Integer(collegeRecordNb));
	    } catch (FinderException e) {
	        result = false;
	    }
	    return result;
	}

    /**
     * @ejb:interface-method
     */
    public Vector archiveFinish(String collegeRecordNb)throws Exception{
	    boolean finished = false;
	    String name = null;
	    String firstName = null;
	    String number = null;
	    String street = null;
	    String pc = null;
	    String city = null;
	    String classe = null;
	    String station = null;
	    String validationAgreement = null;
	    String subsidyAgreement = null;
	    String schoolBusArrival = null;
	    String schoolBusCompanyResponse = null;
	    String schoolBusCompanyRecordNb = null;

	    Vector result = new Vector();
	    try {
	        cul = cuhl.findByPrimaryKey(new Integer(collegeRecordNb));
	        name = cul.getName();
	        firstName = cul.getFirstName();
	        number = cul.getNb();
		    street = cul.getStreet();
		    pc = cul.getPc();
		    city = cul.getCity();
		    classe = cul.getClasse();
		    station = cul.getStation();
		    validationAgreement = cul.getValidationAgreement();
		    subsidyAgreement = cul.getSubsidyAgreement();
		    schoolBusArrival = cul.getSchoolBusArrival();
		    schoolBusCompanyResponse = cul.getSchoolBusCompanyResponse();
		    schoolBusCompanyRecordNb = cul.getSchoolBusCompanyRecordNb();
	        if (validationAgreement!=null) {
		    	finished = true;
			} else {
				finished = false;
			}
	    } catch (FinderException e) {
	        finished = false;
	    }
	    result.addElement(new Boolean(finished));
	    result.addElement(name);
	    result.addElement(firstName);
	    result.addElement(number);
	    result.addElement(street);
	    result.addElement(pc);
	    result.addElement(city);
	    result.addElement(classe);
	    result.addElement(station);
	    result.addElement(validationAgreement);
	    result.addElement(subsidyAgreement);
	    result.addElement(schoolBusArrival);
	    result.addElement(schoolBusCompanyResponse);
	    result.addElement(schoolBusCompanyRecordNb);
	    return result;
	}

    /**
     * @ejb:interface-method
     * @ejb:transaction type="NotSupported"
     */
	public long getCollegeRecordNb(String name, String firstName, String street) throws Exception{
	    long resultat = 0;
	    try {
	        cul = cuhl.findBy_N_FN_S(name, firstName,street);
	        resultat = cul.getId().longValue();
	    } catch (FinderException e) {}
		return resultat;
	}

    /**
     * @ejb:interface-method
     * @ejb:transaction type="NotSupported"
     */
	public void modifyArchive(String collegeRecordNb,String name,String firstName,String validationAgreement,String subsidyAgreement,String schoolBusCompanyResponse,String schoolBusArrival,String schoolBusCompanyNb) throws Exception{
	    try {
	        cul = cuhl.findByPrimaryKey(new Integer(collegeRecordNb));
	        cul.setValidationAgreement(validationAgreement);
	        cul.setSubsidyAgreement(subsidyAgreement);
	        cul.setSchoolBusCompanyResponse(schoolBusCompanyResponse);
	        cul.setSchoolBusArrival(schoolBusArrival);
	        cul.setSchoolBusCompanyRecordNb(schoolBusCompanyNb);
	    } catch (FinderException e) {
	        throw new Exception("College user with Id : "+collegeRecordNb+" NOT FOUND !");
	    }
	}

}