/**
 * Copyright (C) 2005 Bull S.A.
 * Bull 1 rue de Provence 38130 ECHIROLLES
 **/
package org.objectweb.orchestra.demo.college;

import java.util.Collection;

import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.naming.NamingException;
import org.objectweb.jonas.service.ServiceException;

/**
 * CollegeUserBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: CollegeUserBean.java,v 1.2 2006/10/09 09:23:38 souillac Exp $
 *
 * @ejb.bean name="CollegeUserBean"
 *           description="Deployment descriptor for the CollegeUserBean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/demo/college/CollegeUserHome"
 *           local-jndi-name="org/objectweb/orchestra/demo/college/CollegeUserHome_L"
 *           lock-policy="container-serialized-transacted"
 *
 * @ejb.home local-class="org.objectweb.orchestra.demo.college.CollegeUserHomeLocal"
 * @ejb.interface local-class="org.objectweb.orchestra.demo.college.CollegeUserLocal"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "collegeUser"
 *                     automatic-pk="true"
 *
 *
 * @ejb:finder signature="org.objectweb.orchestra.demo.college.CollegeUserLocal findById(java.lang.Integer bpiId)"
 *             query="SELECT OBJECT(o) From CollegeUserBean as o Where o.id = ?1"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.demo.college.CollegeUserLocal findBy_N_FN_S(java.lang.String name, java.lang.String firstName, java.lang.String street)"
 *             query="SELECT OBJECT(o) From CollegeUserBean as o Where o.name = ?1 And o.firstName = ?2 And o.street = ?3"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 */

public abstract class CollegeUserBean implements javax.ejb.EntityBean {

    /**
     * EJBCREATE
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(String name, String firstName, String number, String street, String pc, String city, String classe, String station) throws javax.ejb.CreateException {
        setName(name);
        setFirstName(firstName);
        setNb(number);
        setStreet(street);
        setPc(pc);
        setCity(city);
        setClasse(classe);
        setStation(station);
        return null;
    }

    public void ejbPostCreate(String name, String firstName, String number, String street, String pc, String city, String classe, String station) {

    }

    // persistent fields

    /**
     * @return Integer : The BPI Id
     *
     * @ejb.pk-field
     * @ejb.interface-method
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="Supports"
     */
    public abstract Integer getId();

    /**
     * @param id :
     *            Id of the BPI
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setId(Integer id);

    /**
     * @return Name of the collegeCustomer
     *
     * @ejb:persistence column-name="name"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getName();

    /**
     * @param name : name of the collegeCustomer
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setName(String name);

    /**
     * @return FirstName of the collegeCustomer
     *
     * @ejb:persistence column-name="firstName"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getFirstName();

    /**
     * @param firstName : firstName of the collegeCustomer
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setFirstName(String firstName);

    /**
     * @return Nb of the street
     *
     * @ejb:persistence column-name="nb"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getNb();

    /**
     * @param nb : nb
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setNb(String nb);

    /**
     * @return Street
     *
     * @ejb:persistence column-name="street"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getStreet();

    /**
     * @param street : street
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setStreet(String street);

    /**
     * @return pc postal code
     *
     * @ejb:persistence column-name="pc"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getPc();

    /**
     * @param pc : pc
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setPc(String pc);

    /**
     * @return city
     *
     * @ejb:persistence column-name="city"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getCity();

    /**
     * @param city : city
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setCity(String city);

    /**
     * @return classe
     *
     * @ejb:persistence column-name="classe"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getClasse();

    /**
     * @param classe : classe
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setClasse(String classe);

    /**
     * @return station
     *
     * @ejb:persistence column-name="station"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getStation();

    /**
     * @param station
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setStation(String station);

    /**
     * @return validationAgreement
     *
     * @ejb:persistence column-name="validationAgreement"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getValidationAgreement();

    /**
     * @param validationAgreement
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setValidationAgreement(String validationAgreement);

    /**
     * @return subsidyAgreement
     *
     * @ejb:persistence column-name="subsidyAgreement"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getSubsidyAgreement();

    /**
     * @param subsidyAgreement
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setSubsidyAgreement(String subsidyAgreement);

    /**
     * @return schoolBusCompanyResponse
     *
     * @ejb:persistence column-name="schoolBusCompanyResponse"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getSchoolBusCompanyResponse();

    /**
     * @param schoolBusCompanyResponse
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setSchoolBusCompanyResponse(String schoolBusCompanyResponse);

    /**
     * @return schoolBusArrival
     *
     * @ejb:persistence column-name="schoolBusArrival"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getSchoolBusArrival();

    /**
     * @param schoolBusArrival
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setSchoolBusArrival(String schoolBusArrival);

    /**
     * @return schoolBusCompanyRecordNb
     *
     * @ejb:persistence column-name="schoolBusCompanyRecordNb"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getSchoolBusCompanyRecordNb();

    /**
     * @param schoolBusCompanyRecordNb
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setSchoolBusCompanyRecordNb(String schoolBusCompanyRecordNb);

    public void setEntityContext(EntityContext ec) {
    }

    public void unsetEntityContext() {
    }

    public void ejbLoad() {
    }

    public void ejbStore() {
    }

    public void ejbActivate() {
    }

    public void ejbPassivate() {
    }

    public void ejbRemove() throws javax.ejb.RemoveException {
    }

}