/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.admin;

import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;

/**
 * BpelAuthRoleBean.java
 *
 * The Entity bean represents a BpelAuthRole (for authorization of ejb method)
 *
 * @author Steve Krupanszky, Goulven Le Jeune & Charles Souillard
 * @version $Id: BpelAuthRoleBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb:bean name="BpelAuthRole"
 *           display-name="BpelAuthRole Entity"
 *           type="CMP"
 *           jndi-name="org/objectweb/orchestra/admin/BpelAuthRole"
 *           local-jndi-name="org/objectweb/orchestra/admin/BpelAuthRole_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *           primkey-field="id"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.admin.BpelAuthRoleHomeLocal"
 *           remote-class="org.objectweb.orchestra.admin.BpelAuthRoleHome"
 *
 * @ejb.interface local-class="org.objectweb.orchestra.admin.BpelAuthRoleLocal"
 *                 remote-class="org.objectweb.orchestra.admin.BpelAuthRole"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @ejb:transaction type="Required"
 *
 * @ejb.permission role-name="admin"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name="BpelAuthRole"
 *                     automatic-pk="true"
 *
 * @jonas.session-timeout session-timeout="60"
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *             query="SELECT object(a) FROM BpelAuthRole a" transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.admin.BpelAuthRole findByName(java.lang.String pName)"
 *             result-type-mapping="Remote"
 *             query="SELECT OBJECT(n) FROM BpelAuthRole n WHERE n.name = ?1"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.admin.BpelAuthRoleLocal findByName(java.lang.String pName)"
 *             result-type-mapping="Local"
 *             query="SELECT OBJECT(n) FROM BpelAuthRole n WHERE n.name = ?1"
 *             transaction-type="Supports"
 *
 * @jonas.ejb-relation ejb-relation-name="AuthRoles"
 *                     jdbc-table-name="BpelAuthRole_bpeluser"
 *                     ejb-relationship-role-name1="is-authrole-of"
 *                     foreign-key-jdbc-name1="BpelUser_id"
 *                     key-jdbc-name1="id"
 *
 * @jonas.max-cache-size @BpelAuthRoleBean.max-cache-size@
 */
public abstract class BpelAuthRoleBean implements EntityBean {

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------

    public EntityContext mContext;

    // -------------------------------------------------------------------------
    // Properties (Getters/Setters)
    // -------------------------------------------------------------------------

    /**
     * @return Integer : The BpelAuthRole Id
     *
     * @ejb.pk-field
     * @ejb.interface-method
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */

    public abstract Integer getId();

    /**
     * Set the BpelAuthRole's id.
     *
     * @param pId
     *            The id of this BpelAuthRole. Is set at creation time.
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer pId);

    /**
     * Retrieve the BpelAuthRole's Name.
     *
     * @return Returns a String representing the Name of this BpelAuthRole.
     *
     * @ejb:interface-method
     * @ejb:persistence column-name="name"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getName();

    /**
     * Set the BpelAuthRole's Name.
     *
     * @param pName
     *            The Name of this BpelAuthRole. Is set at creation time.
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb.interface-method
     */
    public abstract void setName(String pName);

    /**
     * Retrieve the BpelAuthRole's Group.
     *
     * @return Returns an int representing the Group of this BpelAuthRole.
     *
     * @ejb:interface-method
     * @ejb:persistence column-name="bpelRoleGroup"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getBpelRoleGroup();

    /**
     * Set the BpelAuthRole's Group.
     *
     * @param pGroup
     *            the Group of this BpelAuthRole
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBpelRoleGroup(String pGroup);

    /**
     * @returns the users of the role
     *
     * @ejb:interface-method
     * @ejb:relation name="AuthRoles"
     *               role-name="is-authrole-of"
     *               target-multiple="yes"
     *
     * @jonas.ejb-relation
     *   jdbc-table-name="UserAuthRoleJoin"
     *
     * @jonas.field-mapping
     *   foreign-key-jdbc-name="userId"
     *   key-jdbc-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getBpelUsers();

    /**
     * Set the AuthRoles of this project
     *
     * @param pUsers users of the role
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setBpelUsers(Collection pUsers);

    // -------------------------------------------------------------------------
    // Framework Callbacks
    // -------------------------------------------------------------------------
    /**
     * Create a BpelAuthRole based on its fields
     *
     * @param pAuthRole
     *            The data used to create the BpelAuthRole.
     *
     * @ejb:create-method
     */
    public Integer ejbCreate(String name, String roleGroup) throws CreateException {
        setName(name);
        setBpelRoleGroup(roleGroup);
        return null;
    }

}