/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.admin;

import java.sql.Date;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 *
 * BpelAuthRoleBean.java
 *
 * The Entity bean represents a BpelUser
 *
 * @author Steve Krupanszky, Goulven Le Jeune & Charles Souillard
 * @version $Id: BpelUserBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb:bean name="BpelUser"
 *           display-name="BpelUser Entity"
 *           type="CMP"
 *           jndi-name="org/objectweb/orchestra/admin/BpelUser"
 *           local-jndi-name="org/objectweb/orchestra/admin/BpelUser_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *           primkey-field="id"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.admin.BpelUserHomeLocal"
 *           remote-class="org.objectweb.orchestra.admin.BpelUserHome"
 *
 * @ejb.interface local-class="org.objectweb.orchestra.admin.BpelUserLocal"
 *                remote-class="org.objectweb.orchestra.admin.BpelUser"
 * @ejb:pk class="java.lang.Integer"
 *
 * @ejb:transaction type="Required"
 *
 * @ejb.permission role-name="admin"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name="BpelUser"
 *                     automatic-pk="true"
 *
 * @jonas.session-timeout session-timeout="60"
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *             query="SELECT object(u) FROM BpelUser u"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.admin.BpelUser findByName(java.lang.String pName )"
 *             result-type-mapping="Remote"
 *             query="SELECT Object(u) From BpelUser as u Where u.name = ?1"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.admin.BpelUserLocal findByName(java.lang.String pName )"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From BpelUser as u Where u.name = ?1"
 *             transaction-type="Supports"
 *
 * @jonas.max-cache-size @BpelUserBean.max-cache-size@
 */

public abstract class BpelUserBean implements EntityBean {

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------

    public EntityContext mContext;

    // -------------------------------------------------------------------------
    // Properties (Getters/Setters)
    // -------------------------------------------------------------------------

    /**
     * @return Integer : The BpelUser Id
     *
     * @ejb.pk-field
     * @ejb.interface-method
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */

    public abstract Integer getId();

    /**
     * Set the BpelUser's id.
     *
     * @param pId The id of this BpelUser. Is set at creation time.
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer pId);

    /**
     * Retrieve the BpelUser's Name.
     *
     * @return Returns a String representing the Name of this BpelUser.
     *
     * @ejb:interface-method
     * @ejb:persistence column-name="name"
     * @ejb:transaction type="@tx.entity.getMethod@"
     *
     */
    public abstract String getName();

    /**
     * Set the BpelUser's Name.
     *
     * @param pName
     *            The Name of this BpelUser. Is set at creation time.
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb.interface-method
     */
    public abstract void setName(String pName);

    /**
     * Retrieve the BpelUser's Password.
     *
     * @return Returns an int representing the Password of this BpelUser.
     *
     * @ejb:interface-method
     * @ejb:persistence column-name="password"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getPassword();

    /**
     * Set the BpelUser's Password.
     *
     * @param pPassword
     *            The Password of this BpelUser
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb.interface-method
     */
    public abstract void setPassword(String pPassword);

    /**
     * Retrieve the BpelUser's Email.
     *
     * @return Returns an String representing the Email of this BpelUser.
     *
     * @ejb:interface-method
     * @ejb:persistence column-name="email"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getEmail();

    /**
     * Set the BpelUser's Email.
     *
     * @param pEmail
     *            The Email of this BpelUser. Is set at creation time.
     *
     * @ejb:persistence column-name="email"
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setEmail(String pEmail);

    /**
     * @return Returns the creation date of this BpelUser
     *
     * @ejb:interface-method
     * @ejb:persistence column-name="creationdate"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Date getCreationDate();

    /**
     * Specify the creation date of this BpelUser
     *
     * @param pCreationDate
     *            Date of the creation of this BpelUser
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setCreationDate(Date pCreationDate);

    /**
     * @return Returns the modification date of this BpelUser
     *
     * @ejb:persistence column-name="modificationdate"
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Date getModificationDate();

    /**
     * Specify the modification date of this BpelUser
     *
     * @param pModificationDate
     *            Date of the modification of this BpelUser
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setModificationDate(Date pModificationDate);


    /**
     * @ejb:interface-method
     * @ejb:relation name="AuthRoles"
     *               role-name="has-authrole"
     *               multiple="yes"
     * @jonas.field-mapping
     *   foreign-key-jdbc-name="authRoleId"
     *   key-jdbc-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getBpelAuthRoles();

    /**
     * Set the authorisation roles of the user (for ejb security)
     *
     * @param pRoles roles of the user
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setBpelAuthRoles(Collection pRoles);

    // -------------------------------------------------------------------------
    // Framework Callbacks
    // -------------------------------------------------------------------------

    /**
     * Create a BpelUser based on its fields
     *
     * @param pUser The data used to create the BpelUser.
     *
     * @ejb:create-method view-type="both"
     */
    public Integer ejbCreate(String name, String passwd, String email) throws CreateException {
        setName(name);
        setPassword(passwd);
        setEmail(email);
        setCreationDate(new java.sql.Date(new java.util.Date().getTime()));
        setModificationDate(new java.sql.Date(new java.util.Date().getTime()));
        return null;
    }

    public void setEntityContext(EntityContext lContext) {
        this.mContext = lContext;
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbRemove() throws RemoveException {
        // Bean Method
    }
}