/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.admin;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;

import org.objectweb.orchestra.exception.BPELAdminException;



/**
 * InsertAuthRoleUserBean.java.
 *
 * Encapsulates the retrival of DB data
 *
 * @author Steve Krupanszky, Goulven Le Jeune & Charles Souillard
 * @version $Id: InsertAuthRoleUserBean.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 * @ejb:bean name="InsertAuthRoleUser"
 *           display-name="Makes inserts in the AuthRole_User table"
 *           type="Stateless"
 *           jndi-name="org/objectweb/orchestra/admin/InsertAuthRoleUserHome"
 *           local-jndi-name="org/objectweb/orchestra/admin/InsertAuthRoleUserHome_L"
 *           transaction-type="Container" view-type="both"
 *
 * @ejb.home local-class="org.objectweb.orchestra.admin.InsertAuthRoleUserHomeLocal"
 *           remote-class="org.objectweb.orchestra.admin.InsertAuthRoleUserHome"
 * @ejb.interface local-class="org.objectweb.orchestra.admin.InsertAuthRoleUserLocal"
 *                remote-class="org.objectweb.orchestra.admin.InsertAuthRoleUser"
 *
 *
 *
 * @ejb:transaction type="Required"
 *
 * @ejb.security-identity run-as="admin"
 *
 * @jonas.max-cache-size @InsertAuthRoleUserBean.max-cache-size@
 */

public class InsertAuthRoleUserBean implements SessionBean {

    private SessionContext mContext;

    /**
     * @return : Session Context
     */
    public SessionContext getContext() {
        return this.mContext;
    }

    /**
     * Makes inserts in User table.
     *
     * @param name : name of the user
     * @param password : password of the user
     * @param email : email of the user
     * @throws BPELAdminException
     *
     * @ejb:interface-method
     */
    public void initializeUser(final String name, final String password, final String email) throws BPELAdminException {
        BpelUserHomeLocal uhome = null;
        try {
            uhome = BpelUserUtil.getLocalHome();
            try {
                uhome.findByName(name);
                throw new org.objectweb.orchestra.exception.BPELAdminException("BpelUser "
                        + name + " already exist in the system ", null);
            } catch (javax.ejb.FinderException nn) {
                uhome.create(name, password, email);
            }
        } catch (javax.ejb.CreateException fe) {
            throw new BPELAdminException("CreateException InsertAuthRoleUserBean initializeUser : \n" , fe);
        } catch (NamingException e) {
            throw new BPELAdminException("NamingException InsertAuthRoleUserBean initializeUser :\n" , e);
        }
    }

    /**
     * Makes inserts in User table.
     *
     * @param name : Name
     * @param rolegroup : role
     * @throws BPELAdminException
     *
     * @ejb:interface-method
     */
    public void initializeAuthRole(final String name, final String rolegroup) throws BPELAdminException {
        BpelAuthRoleHomeLocal arhome = null;
        try {
            arhome = BpelAuthRoleUtil.getLocalHome();
            try {
                arhome.findByName(name);
                throw new org.objectweb.orchestra.exception.BPELAdminException("InsertAuthRoleUserBean "
                        + "initializeAuthRole BpelAuthRole " + name + " already exist in the system ", null);
            } catch (javax.ejb.FinderException nn) {
                arhome.create(name, rolegroup);
            }
        } catch (javax.ejb.CreateException fe) {
            throw new BPELAdminException("CreateException InsertAuthRoleUserBean initializeAuthRole : \n" , fe);
        } catch (NamingException e) {
            throw new BPELAdminException("NamingException InsertAuthRoleUserBean initializeAuthRole :\n" , e);
        }
    }

    /**
     * Makes inserts in AuthRole_User table.
     *
     * @param user : name of the user
     * @param authRole : role of the user
     * @throws BPELAdminException
     *
     * @ejb:interface-method view-type="remote"
     */
    public void initialize(final String user, final String authRole) throws BPELAdminException {
        BpelUserHomeLocal uhome = null;
        BpelAuthRoleHomeLocal arhome = null;
        BpelUserLocal userLocal;
        BpelAuthRoleLocal authRoleLocal;
        try {
            uhome = BpelUserUtil.getLocalHome();
            arhome = BpelAuthRoleUtil.getLocalHome();
            userLocal = uhome.findByName(user);
            authRoleLocal = arhome.findByName(authRole);

            if (!authRoleLocal.getBpelUsers().contains(userLocal)) {
                authRoleLocal.getBpelUsers().add(userLocal);
            }
            if (!userLocal.getBpelAuthRoles().contains(authRoleLocal)) {
                userLocal.getBpelAuthRoles().add(authRoleLocal);
            }

        } catch (javax.ejb.FinderException fe) {
            throw new BPELAdminException("FinderException InsertAuthRoleUserBean initialize : \n" , fe);
        } catch (NamingException fe) {
            throw new BPELAdminException("NamingException InsertAuthRoleUserBean initialize : \n" , fe);
        }
    }

    /**
     * Create the Session Bean.
     *
     * @throws CreateException
     *
     * @ejb:create-method
     */
    public void ejbCreate() throws CreateException {
        // Bean Method
    }

    // -------------------------------------------------------------------------
    // Framework Callbacks
    // -------------------------------------------------------------------------

    public void setSessionContext(final SessionContext aContext) throws EJBException {
        this.mContext = aContext;
    }

    public void ejbActivate() throws EJBException {
        // Bean Method
    }

    public void ejbPassivate() throws EJBException {
        // Bean Method
    }

    public void ejbRemove() throws EJBException {
        // Bean Method
    }

}
