/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.admin;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;

import org.objectweb.orchestra.exception.BPELAdminException;

import org.objectweb.orchestra.data.BpelDfltSetupBeanUtil;
import org.objectweb.orchestra.data.BpelDfltSetupHomeLocal;

/**
 * InsertDfltSetupBean.java.
 *
 * Encapsulates the retrival of DB data
 *
 * @version $Id: InsertDfltSetupBean.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 * @ejb:bean name="InsertDfltSetup"
 *           display-name="Makes inserts in the BpelDfltSetup table"
 *           type="Stateless"
 *           jndi-name="org/objectweb/orchestra/admin/InsertDfltSetupHome"
 *           local-jndi-name="org/objectweb/orchestra/admin/InsertDfltSetupHome_L"
 *           transaction-type="Container" view-type="both"
 *
 * @ejb.home local-class="org.objectweb.orchestra.admin.InsertDfltSetupHomeLocal"
 *           remote-class="org.objectweb.orchestra.admin.InsertDfltSetupHome"
 * @ejb.interface local-class="org.objectweb.orchestra.admin.InsertDfltSetupLocal"
 *                remote-class="org.objectweb.orchestra.admin.InsertDfltSetup"
 *
 *
 *
 * @ejb:transaction type="Required"
 *
 * @ejb.security-identity run-as="admin"
 *
 * @jonas.max-cache-size @InsertDfltSetupBean.max-cache-size@
 *
 */

public class InsertDfltSetupBean implements SessionBean {

    private SessionContext mContext;

    public SessionContext getContext() {
        return this.mContext;
    }

    /**
     * Makes inserts in BpelDfltSetup table.
     *
     *
     * @throws RemoteException
     *
     * @ejb:interface-method
     */

    public void initializeDfltSetup(final boolean log, final boolean trace, final String loglevel,
            final String tracelevel, final String historic) throws BPELAdminException {
        BpelDfltSetupHomeLocal dshome = null;
        try {
            dshome = BpelDfltSetupBeanUtil.getLocalHome();
            try {
                dshome.findRow();
                throw new org.objectweb.orchestra.exception.BPELAdminException("InsertDfltSetupBean "
                        + "initializeDfltSetup BpelDfltSetup already exist in the system ", null);
            } catch (javax.ejb.FinderException nn) {
                dshome.create(log, trace, loglevel, tracelevel, historic);
            }
        } catch (javax.ejb.CreateException fe) {
            throw new BPELAdminException("CreateException InsertDfltSetupBean initialize : \n" , fe);
        } catch (NamingException fe) {
            throw new BPELAdminException("NamingException InsertDfltSetupBean initialize : \n" , fe);
        }
    }

    /**
     * Create the Session Bean.
     *
     * @throws CreateException
     *
     * @ejb:create-method
     */
    public void ejbCreate() throws CreateException {
        // Bean Method
    }

    // -------------------------------------------------------------------------
    // Framework Callbacks
    // -------------------------------------------------------------------------

    public void setSessionContext(final SessionContext aContext) throws EJBException {
        this.mContext = aContext;
    }

    public void ejbActivate() throws EJBException {
        // Bean Method
    }

    public void ejbPassivate() throws EJBException {
        // Bean Method
    }

    public void ejbRemove() throws EJBException {
        // Bean Method
    }

}
