/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bindingframework;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Vector;

import org.objectweb.orchestra.engine.BPELMessage;
import org.objectweb.orchestra.engine.MessageStructure;
import org.objectweb.orchestra.engine.PartStructure;
import org.objectweb.orchestra.exception.BFUtilException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.ClassLoaderMap;
import org.objectweb.orchestra.util.WSDLJavaMatching;



/**
 * BFUtil.java
 *
 * @author Goulven Le Jeune & Charles Souillard & Eric Hardesty
 * @version $Id: BFUtil.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class BFUtil {

    /**
     * Build a holder object
     * @param type String type of holder
     * @param value Object of value
     * @param processName String of process name
     * @return Object holder object
     * @throws BFUtilException if an error occurs building the holder
     */
    public static Object buildHolder(String type, Object value, String processName) throws BFUtilException {
        Object obj = null;
        try {
            ClassLoader classL = ClassLoaderMap.getCL(processName);
            Class cl = Class.forName(WSDLJavaMatching.getHolderType(type),true,classL);
            Class cl2 = WSDLJavaMatching.getPrimitiveClass(value.getClass());
            Constructor constr = cl.getDeclaredConstructor(new Class[] { cl2});
            obj = constr.newInstance(new Object[] { value});
        } catch (InvocationTargetException ex) {
            throw new BFUtilException("InvocationTargetException EngineBean buildHolder :\n" , ex);
        } catch (NoSuchMethodException ex) {
            throw new BFUtilException("NoSuchMethodException EngineBean buildHolder :\n" , ex);
        } catch (InstantiationException ex) {
            throw new BFUtilException("InstantiationException EngineBean buildHolder :\n" , ex);
        } catch (IllegalArgumentException ex) {
            throw new BFUtilException("IllegalArgumentException EngineBean buildHolder :\n" , ex);
        } catch (IllegalAccessException ex) {
            throw new BFUtilException("IllegalAccessException EngineBean buildHolder :\n" , ex);
        } catch (ClassNotFoundException ex) {
            throw new BFUtilException("ClassNotFoundException EngineBean buildHolder :\n" , ex);
        } catch (BPELUtilException ex) {
            throw new BFUtilException("BPELUtilException EngineBean buildHolder : " , ex);
        }
        return obj;
    }

    /**
     * Get input values
     * @param jmsgin BPELMessage input message
     * @param jmsgout BPELMessage output message
     * @param met Method to determine parameter types
     * @param processName String process name
     * @return Vector of values
     * @throws BFUtilException
     */
    public static Vector getInputValues(BPELMessage jmsgin, BPELMessage jmsgout, Method met, String processName) throws BFUtilException {
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("BFUtil getInputValues: processName = " + processName + ", met = " + met + ", jmsgin = " + jmsgin + ", jmsgout = " + jmsgout);
        Vector values = new Vector();
        if (jmsgin != null) {
            try {
                MessageStructure msin = jmsgin.getStructure();
                PartStructure[] psin = msin.getParts();
                MessageStructure msout = null;
                PartStructure[] psout = null;
                Vector outElements = new Vector();
                if (jmsgout != null) {
                    msout = jmsgout.getStructure();
                    psout = msout.getParts();
                    for (int k = 0; k < psout.length; k++) {
                        outElements.addElement(psout[k].getType());
                    }
                }
                Class[] paramTypes = met.getParameterTypes();
                Class cl = null;
                Constructor constr = null;
                Object obj = null;
                Object o = null;

                if (jmsgout == null) {
                    for (int k = 0; k < psin.length; k++) {
                        values.addElement(jmsgin.getPart(psin[k].getName()));
                    }
                } else {
                    Vector inParts = new Vector();
                    for (int k = 0; k < psin.length; k++) {
                        inParts.addElement(psin[k].getType());
                        inParts.addElement(jmsgin.getPart(psin[k].getName()));
                    }
                    for (int k = 0; k < paramTypes.length; k++) {
                        String type = paramTypes[k].getName();
                        boolean isHolder = type.indexOf("holders") != -1;
                        if (isHolder) {
                            o = getCommonHolder(inParts,outElements,type,processName);
                            //there are some input parts still not used, so there is a common part
                            //between in and out
                            //if the part is empty return null

                        } else {
                            o = getCorrespondingPart(inParts,type);
                        }
                        if (o != null || !isHolder) {
                            values.addElement(o);
                        }
                    }

                    if (outElements.size() > 1) {
                        ClassLoader classL = ClassLoaderMap.getCL(processName);
                        for (int l = 0; l < outElements.size(); l++) {
                            cl = Class.forName(WSDLJavaMatching.getHolderType((String) outElements.get(l)),true,classL);
                            constr = cl.getConstructor((Class[])null);
                            obj = constr.newInstance((Object[])null);
                            values.addElement(obj);
                        }
                    }
                }
            } catch (ClassNotFoundException ex) {
                throw new BFUtilException("ClassNotFoundException Util getInputValues :\n" , ex);
            } catch (NoSuchMethodException ex) {
                throw new BFUtilException("NoSuchMethodException Util getInputValues :\n" , ex);
            } catch (InstantiationException ex) {
                throw new BFUtilException("InstantiationException Util getInputValues :\n" , ex);
            } catch (IllegalArgumentException ex) {
                throw new BFUtilException("IllegalArgumentException Util getInputValues :\n" , ex);
            } catch (IllegalAccessException ex) {
                throw new BFUtilException("IllegalAccessException Util getInputValues :\n" , ex);
            } catch (InvocationTargetException ex) {
                throw new BFUtilException("InvocationTargetException Util getInputValues :\n" , ex);
            } catch (BPELUtilException ex) {
                throw new BFUtilException("BPELUtilException Util getInputValues :\n" , ex);
            }
        }
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("BFUtil getInputValues: values= " + values + " size " + values.size());
        return values;
    }
//  Private methods

    /** Get a common holder
     *
     * @param inParts
     * @param outElements
     * @param searchedType
     * @param processName
     * @return
     * @throws BFUtilException
     */
    private static Object getCommonHolder(Vector inParts, Vector outElements, String searchedType, String processName) throws BFUtilException {
        Object result = null;
        root:
            for (int i = 0; i < inParts.size(); i = i+2) {
                String inType = (String) inParts.get(i);
                if (WSDLJavaMatching.getHolderType(inType).equals(searchedType)) {
                    for (int l = 0; l < outElements.size(); l++) {
                        String outType = (String) outElements.get(l);
                        if (WSDLJavaMatching.getHolderType(outType).equals(searchedType)) {
                            result = buildHolder((String) outElements.get(l), inParts.get(i+1),processName);
                            outElements.removeElementAt(l);
                            inParts.removeElementAt(i);
                            inParts.removeElementAt(i);
                            break root;
                        }
                    }
                }
            }
        return result;
    }

    /**
     * Get a corresponding part
     * @param inParts Vector of input parts
     * @param searchedType String type searching for
     * @return Object input par found
     */
    private static Object getCorrespondingPart(Vector inParts, String searchedType) {
        Object result = null;
        for (int i = 0; i < inParts.size(); i = i+2) {
            String inType = (String) inParts.get(i);
            if (inType.equals(searchedType)) {
                result = inParts.get(i+1);
                inParts.removeElementAt(i);
                inParts.removeElementAt(i);
                break;
            }
        }
        return result;
    }


}