/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bindingframework.configuration;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

import org.objectweb.orchestra.util.BPELTrace;


/**
 * BindingFramework EndpointList class
 *
 * @author Eric Hardesty
 * @version $Id: BFEndpointList.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class BFEndpointList {
    // class members
    Properties propInfo = null;
    Hashtable hEndpoints = new Hashtable();

    /**
     * constructor.
     */
    public BFEndpointList(Properties prop)  {
        this.propInfo = prop;
    }

    public Properties getProperties() {
        return this.propInfo;
    }

    public void addEndpoint(BFEndpoint be) {
        this.hEndpoints.put(be.getId(), be);
    }

    public Hashtable getEndpoints() {
        return this.hEndpoints;
    }

    public String getId() {
        String id = this.propInfo.getProperty("id");
        if (id == null) {
            id = "";
        }
        return id;
    }

    public BFEndpoint getDefault() {
        String def = this.propInfo.getProperty("default");
        BFEndpoint bfe = (BFEndpoint) this.hEndpoints.get(def);
        return bfe;
    }

    public void printDebug() {
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("  BFEndpointList: " + this + " with " + this.propInfo);
        for (Enumeration e = this.hEndpoints.elements(); e.hasMoreElements();) {
            BFEndpoint bfe = (BFEndpoint) e.nextElement();
            bfe.printDebug();
        }

    }
}