/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.model;


/**
 * ModelEdge.java
 *
 * @author Bill Murray
 * @version $Id: ModelEdge.java 5 2006-10-20 14:37:44Z lejeuneg $
 *          <p>
 *          This class extends ModelAttributes and is the class for "edges"
 */
public class ModelEdge extends ModelAttributes {

    private final ModelPort srcPort;

    private final ModelPort destPort;

    private final int grouping;

    /**
     * getSrcPort - returns the "source" port for the edge.
     *
     * @return - ModelPort
     */
    public ModelPort getSrcPort() {
        return this.srcPort;
    }

    /**
     * getSrcKey - returns the key for the "source" port for the edge.
     *
     * @return - String
     */
    public String getSrcKey() {
        return this.srcPort.getKey();
    }

    /**
     * getDestPort - returns the "destination" port for the edge.
     *
     * @return - ModelPort
     */
    public ModelPort getDestPort() {
        return this.destPort;
    }

    /**
     * getDestKey - returns the key for the "destination" port for the edge.
     *
     * @return - String
     */
    public String getDestKey() {
        return this.destPort.getKey();
    }

    /**
     * getGrouping - returns the grouping policy for the edge.
     *
     * @return - int
     */
    public int getGrouping() {
        return this.grouping;
    }

    /**
     * ModelEdge - constructor - create an edge connecting the 2 ports.
     *
     * @param srcPort
     * @param destPort
     * @param grouping - the grouping policy for the edge. Either "single" or
     *                   "mulitple"
     */
    public ModelEdge(ModelPort srcPort, ModelPort destPort, int grouping) {
        super(srcPort.getKey() + destPort.getKey());
        this.srcPort = srcPort;
        this.destPort = destPort;
        this.grouping = grouping;
    }
}