/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.ContainerDisplay;
import graph.GraphConstants;
import graph.GraphContainer;
import graph.GraphItem;
import graph.GraphLayoutManager;
import graph.GraphPort;
import graph.TopoTrace;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.Vector;

import org.objectweb.orchestra.bpem.model.ModelNode;
import org.objectweb.orchestra.bpem.model.ModelPort;



/**
 * BPEMGraphitGraphModel.java.
 *
 * @author Bill Murray
 * @version $Id: BPEMGraphitContainerDisplay.java,v 1.2 2005/02/28 21:17:34 bmurray
 *          Exp $
 *          <p>
 *          This class extends the GraphIt ContainerDisplay to provide
 *          custom painting and layout.
 */
public class BPEMGraphitContainerDisplay extends ContainerDisplay {

    /** Dimensions of the ports subareas. */
    protected int portsW = 0;
    protected int portsH = 0;
    protected int northPortsW = 0;
    protected int northPortsH = 0;
    protected int southPortsW = 0;
    protected int southPortsH = 0;
    protected int westPortsW = 0;
    protected int westPortsH = 0;
    protected int eastPortsW = 0;
    protected int eastPortsH = 0;
    GraphPort[] northPorts;
    GraphPort[] southPorts;
    GraphPort[] westPorts;
    GraphPort[] eastPorts;

    /** Dimensions of the container's limit. */
    protected int limitDX = 0;
    protected int limitDY = 0;
    protected int limitW = 0;
    protected int limitH = 0;
    protected boolean roundLimit = false;

    private final ModelNode mn;

    public BPEMGraphitContainerDisplay(GraphContainer graphContainer) {
        super(graphContainer);
        this.mn = (ModelNode) graphContainer.getUserObject();
        //
        // Set the margin out to the ports big enough to allow for the
        // "bus" to be withing the contianer.
        //
        this.LimitMargin = 10;
    }
    /**
     * Determines the relative coordinates of the elements displayed inside.
     * Computes the total size of the object.
     */
    public void layout(Graphics g, int phase) {

        final String methLabel = "layout";
        TopoTrace.debug(this, methLabel, "==> phase="+phase+" graphContainer="+this.graphContainer);
        super.layout(g, phase);
        //if (graphContainer.expanded) {
        if (this.graphContainer.getExpanded()) {
            this.toggleImage = GraphConstants.retractIcon;
        } else {
            this.toggleImage = GraphConstants.expandIcon;
        }
        this.toggleW = this.toggleImage.getIconWidth();
        this.toggleH = this.toggleImage.getIconHeight();

        // add a peripheral square area for ports
        this.portsW = 0;
        this.northPortsW = 0;
        this.northPortsH = 0;
        this.southPortsW = 0;
        this.southPortsH = 0;
        this.westPortsW = 0;
        this.westPortsH = 0;
        this.eastPortsW = 0;
        this.eastPortsH = 0;
//      ###if (phase > 1) {
        Vector vNorthPorts = new Vector();
        Vector vSouthPorts = new Vector();
        Vector vWestPorts = new Vector();
        Vector vEastPorts = new Vector();
        for (int i = 0; i < this.graphContainer.nbOfPorts(); i++) {
            GraphPort port = this.graphContainer.getPort(i);
            int portOrientation = getPortOrientation(port);
            port.setOrientation(portOrientation);
            // force layout of the port (PHASE 0 only)
            port.layout(g, 0);
            switch(portOrientation) {
            case NORTH:
                vNorthPorts.add(port);
                this.northPortsW += port.getW();
                this.northPortsH = Math.max(this.northPortsH, port.getH());
                break;
            case SOUTH:
                vSouthPorts.add(port);
                this.southPortsW += port.getW();
                this.southPortsH = Math.max(this.southPortsH, port.getH());
                break;
            case WEST:
                vWestPorts.add(port);
                this.westPortsW = Math.max(this.westPortsW, port.getW());
                this.westPortsH += port.getH();
                break;
            case EAST:
                vEastPorts.add(port);
                this.eastPortsW = Math.max(this.eastPortsW, port.getW());
                this.eastPortsH += port.getH();
                break;
            }
            this.portsW = Math.max(this.northPortsW, this.southPortsW);
            this.portsH = Math.max(this.westPortsH, this.eastPortsH);
        }
        this.northPorts = (GraphPort[]) vNorthPorts.toArray(new GraphPort[0]);
        this.southPorts = (GraphPort[]) vSouthPorts.toArray(new GraphPort[0]);
        this.westPorts = (GraphPort[]) vWestPorts.toArray(new GraphPort[0]);
        this.eastPorts = (GraphPort[]) vEastPorts.toArray(new GraphPort[0]);
//      ###}

        // layout the children area
        GraphLayoutManager lm = this.graphContainer.getLayout();
        lm.layout(g, phase);
        //###lm.relax(); //###
        //if (graphContainer.expanded) {
        if (this.graphContainer.getExpanded()) {
            this.subW = lm.getW();
            this.subH = lm.getH();
        } else {
            this.subW = 0;
            this.subH = 0;
        }

        // compute the relative location of each element
        this.toggleDX = this.westPortsW + 2*this.LimitMargin;
        this.toggleDY = this.northPortsH + 2*this.LimitMargin;

        this.coreDX = this.toggleDX + this.toggleW + this.scaledCoreMargin;
        this.coreDY = this.toggleDY;
        //
        // Put this containers chidren "underneath" the "core"
        //
//      subDX = coreDX + coreW + scaledCoreMargin;
//      subDY = toggleDY;
        this.subDX = this.coreDX;
        this.subDY = this.toggleDY + this.coreH;
        //
        // compute the total dimensions
        //
//      w = 4*LimitMargin + toggleW + coreW + 2*scaledCoreMargin + subW + westPortsW + eastPortsW;
        this.w = 4*this.LimitMargin + this.toggleW + Math.max(this.coreW, this.subW) + 1*this.scaledCoreMargin
        + this.westPortsW + this.eastPortsW;
        this.w = Math.max(this.w, this.northPortsW);
        this.w = Math.max(this.w, this.southPortsW);
//      h = 4*LimitMargin + Math.max(coreH+2*scaledCoreMargin,subH) + northPortsH + southPortsH;
        this.h = 4*this.LimitMargin + this.coreH + this.scaledCoreMargin + this.subH + this.northPortsH
        + this.southPortsH;
        this.h = Math.max(this.h, this.westPortsH);
        this.h = Math.max(this.h, this.eastPortsH);

        this.limitDX = this.westPortsW+this.LimitMargin;
        this.limitDY = this.northPortsH+this.LimitMargin;
        this.limitW = this.w - 2*this.LimitMargin - this.westPortsW - this.eastPortsW;
        this.limitH = this.h - 2*this.LimitMargin - this.northPortsH - this.southPortsH;
        TopoTrace.debug(this, methLabel, "<== w="+this.w+" h="+this.h+" subW="+this.subW+" subH="+this.subH);
    }

    /**
     * Determines the absolute coordinates of the elements displayed inside.
     */
    public void setLocation(int x, int y, int phase) {
        final String methLabel = "setLocation";
        TopoTrace.debug(this, methLabel, "==> phase="+phase+" graphContainer="+this.graphContainer+" x="+x+" y="+y);
        super.setLocation(x, y, phase);

        if (phase == 1) {
            // set all ports at center of the node
            int cx = x+this.w/2;
            int cy = y+this.h/2;
            for (int i = 0; i < this.graphNode.nbOfPorts(); i++) {
                GraphPort port = this.graphNode.getPort(i);
                port.setLocation(cx-port.getW()/2, cy-port.getH()/2, phase);
            }
        } else {
            // arrange ports (even if not shown)
            boolean arranged = true;
            int nbLoops = 0;
            do {
                arranged = true;
                //###arranged = arranged && GraphPort.arrange(northPorts,GraphItem.XCOORD,SanGraphAdapter.CNX);
                arranged = arranged && GraphPort.arrange(this.northPorts, GraphItem.XCOORD, -1);
                int northPortX = x + (this.w-this.northPortsW)/2;
                int northPortY = y;
                for (int i = 0; i < this.northPorts.length; i++) {
                    GraphPort port = this.northPorts[i];
                    port.setLocation(northPortX, northPortY, phase);
                    northPortX += port.getW();
                }

                //###arranged = arranged && GraphPort.arrange(southPorts,GraphItem.XCOORD,SanGraphAdapter.CNX);
                arranged = arranged && GraphPort.arrange(this.southPorts, GraphItem.XCOORD, -1);
                int southPortX = x + (this.w-this.southPortsW)/2;
                int southPortY = 0; // depends on port's height
                for (int i = 0; i < this.southPorts.length; i++) {
                    GraphPort port = this.southPorts[i];
                    southPortY = y + this.h - port.getH();
                    port.setLocation(southPortX, southPortY, phase);
                    southPortX += port.getW();
                }

                //###arranged = arranged && GraphPort.arrange(westPorts,GraphItem.YCOORD,SanGraphAdapter.CNX);
                arranged = arranged && GraphPort.arrange(this.westPorts, GraphItem.YCOORD, -1);
                int westPortX = x;
                int westPortY = y + (this.h-this.westPortsH)/2;
                for (int i = 0; i < this.westPorts.length; i++) {
                    GraphPort port = this.westPorts[i];
                    port.setLocation(westPortX, westPortY, phase);
                    westPortY += port.getH();
                }

                //###arranged = arranged && GraphPort.arrange(eastPorts,GraphItem.YCOORD,SanGraphAdapter.CNX);
                arranged = arranged && GraphPort.arrange(this.eastPorts, GraphItem.YCOORD, -1);
                int eastPortX = 0; // depends on port's width
                int eastPortY = y + (this.h-this.eastPortsH)/2;
                for (int i = 0; i < this.eastPorts.length; i++) {
                    GraphPort port = this.eastPorts[i];
                    eastPortX = x + this.w - port.getW();
                    port.setLocation(eastPortX, eastPortY, phase);
                    eastPortY += port.getH();
                }
                nbLoops++;
            } while (!arranged && (nbLoops < 10));
            TopoTrace.debug(this, methLabel, "container ports arranged in nbLoops="+nbLoops);
        }
        TopoTrace.debug(this, methLabel, "<== ");
    }

    public void setRoundLimit(boolean b) {
        this.roundLimit = b;
    }

    /**
     * paint - Sets the color based on if the BPEL statement has
     * been executed. Also set the tooltip that can change during
     * refresh.
     */
    public void paint(Graphics g) {
        if (this.mn != null) {
            this.graphContainer.setToolTipText(this.mn.getAllAttrs());
            if (this.mn.isExecuted()) {
                this.limitColor = new Color(0, 175, 0);
            } else {
                this.limitColor = new Color(250, 0, 50);
            }
        }
        super.paint(g);
    }
    /**
     * Paints the container's limit.
     */
    public void paintLimit(Graphics g) {
        if (!this.graphContainer.getShowLimit()) {
            return;
        }
        Graphics2D g2 = (Graphics2D) g;
        g2.setStroke(this.limitStroke);
        g.setColor(this.limitColor);
        if (this.roundLimit) {
            g.drawRoundRect(this.x+this.limitDX, this.y+this.limitDY, this.limitW, this.limitH, this.w/2, this.h/2);
        } else {
            g.drawRect(this.x+this.limitDX, this.y+this.limitDY, this.limitW, this.limitH);
        }
    }

    /**
     * Paints the container's toggle.
     */
    public void paintToggle(Graphics g) {
        if (!this.graphContainer.getShowToggle()) {
            return;
        }
        this.toggleImage.paintIcon(null, g, this.x+this.toggleDX, this.y+this.toggleDY);
    }

    /**
     * getPortOrientation - by looking at what end of the edge we
     * are on, we can determine if we want the port to be on top
     * or on bottom.
     *
     * @param port
     * @return - int - either NORTH or SOUTH
     */
    private int getPortOrientation(GraphPort port) {
        ModelPort mp = (ModelPort) port.getUserObject();
        if (mp.isInbound()) {
            return NORTH;
        }
        return SOUTH;
    }
}
