/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.DisplayManager;
import graph.Graph;
import graph.GraphAdapter;
import graph.GraphCircleLayout;
import graph.GraphConstants;
import graph.GraphContainer;
import graph.GraphEdge;
import graph.GraphFlowLayout;
import graph.GraphIcon;
import graph.GraphItem;
import graph.GraphLayeredLayout;
import graph.GraphLayoutManager;
import graph.GraphMultiEdge;
import graph.GraphNode;
import graph.GraphNodeGroup;
import graph.GraphNodeSet;
import graph.GraphPort;
import graph.TopoTrace;

import java.awt.Color;
import java.awt.Paint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import org.objectweb.orchestra.bpem.controller.BPEM;
import org.objectweb.orchestra.bpem.model.ExecutionModel;
import org.objectweb.orchestra.bpem.model.ModelEdge;
import org.objectweb.orchestra.bpem.model.ModelNode;
import org.objectweb.orchestra.bpem.model.ModelPort;



/**
 * BPEMGraphitGraphAdapter.java
 *
 * @author Bill Murray
 * @version $Id: BPEMGraphitGraphAdapter.java,v 1.1 2005/03/04 15:48:02 bmurray
 *          Exp $
 *          <p>
 *          This class handles events from the graph.
 */
public class BPEMGraphitGraphAdapter extends GraphAdapter implements ActionListener {
    public final String RCSId = "@(#) $Id: BPEMGraphitGraphAdapter.java 5 2006-10-20 14:37:44Z lejeuneg $";

    private final ExecutionModel executionModel;

    public BPEMGraphitGraphAdapter(ExecutionModel executionModel) {
        final String methLabel = "SampleGraphAdapter";
        TopoTrace.debug(this, methLabel, "==>");
        this.executionModel = executionModel;
        // initialize preferences
        this.prefKey = "SampleGraphAdapter";
        super.applyPreferences();
        applyPreferences();
        //
        // Force port icons off, full labels
        //
        this.showPortIcon = false;
        this.setItemLabelSizePolicy(DisplayManager.LABEL_FULL);
        this.setPortLabelSizePolicy(DisplayManager.LABEL_FULL);

        TopoTrace.debug(this, methLabel, "<==");
    }

    public void applyPreferences() {
        final String methLabel = "applyPreferences";
        TopoTrace.debug(this, methLabel, "==> ");
        //String prefKey = getPrefKey();
        //###layersVisible =
        // Preferences.getPreference(prefKey+".layersVisible",layersVisible);
        TopoTrace.debug(this, methLabel, "<== ");
    }

    public int getNbOfLayers() {
        return (3);
    }

    public boolean leftClick(Graph graph, GraphItem graphItem, int atX,
            int atY, boolean realClick, int shiftPressed) {
        final String methLabel = "leftClick";
        TopoTrace.debug(this, methLabel, "==> realClick=" + realClick
                + " shiftPressed=" + shiftPressed);
        boolean rc = false;
        //GraphModel model = graph.getModel();
        clearSelectedItems();
        clearTargetItems();
        //
        // See if it was a node that was selected
        //
        if (graphItem == null)
            return rc;
        Object userObject = graphItem.getUserObject();
        if ((userObject instanceof ModelNode) == false)
            return rc;
        //
        // Tell all listeners that a node was selected
        //
        this.executionModel.selectModelNode((ModelNode) userObject);
        return rc;

        //        if (graphItem != null) {
        //            addSelectedItem(graphItem);
        //            if (graphItem instanceof GraphNode) {
        //                GraphNode graphNode = (GraphNode) graphItem;
        //                // find remote nodes
        //                for (int j = 0; j < graphNode.nbOfPorts(); j++) {
        //                    GraphPort graphPort = graphNode.getPort(j);
        //                    for (int i = 0; i < graphPort.nbOfEdges(); i++) {
        //                        GraphEdge edge = graphPort.getEdge(i);
        //                        GraphPort remPort = edge.getSrcPort();
        //                        if (remPort == graphPort) {
        //                            remPort = edge.getDstPort();
        //                        }
        //                        GraphNode remNode = remPort.getNode();
        //                        addTargetItem(remNode);
        //                        // neighbour's container is also target
        //                        GraphNodeSet parent = remNode.getParent();
        //                        if ((parent != null)
        //                                && (parent instanceof GraphContainer)) {
        //                            addTargetItem(parent);
        //                        }
        //                    }
        //                }
        //            } else if (graphItem instanceof GraphPort) {
        //                GraphPort graphPort = (GraphPort) graphItem;
        //                // find remote ports
        //                for (int i = 0; i < graphPort.nbOfEdges(); i++) {
        //                    GraphEdge edge = graphPort.getEdge(i);
        //                    GraphPort remPort = edge.getSrcPort();
        //                    if (remPort == graphPort) {
        //                        remPort = edge.getDstPort();
        //                    }
        //                    addTargetItem(remPort);
        //                }
        //            }
        //        }
        //        // update background of all items
        //        dressItems(model);
        //        TopoTrace.debug(this, methLabel, "<==");
        //        return (rc);
    }

    public boolean rightClick(Graph graph, GraphItem graphItem, int atX, int atY) {
        final String methLabel = "rightClick";
        TopoTrace.debug(this, methLabel, "==>");
        boolean rc = false;
        TopoTrace.debug(this, methLabel, "graphItem=" + graphItem);
        // memorize current item clicked for default popup actions
        this.graph = graph;
        this.rightItem = graphItem;
        displayPopup(graph, graphItem, atX, atY);
        TopoTrace.debug(this, methLabel, "<==");
        return (rc);
    }

    /**
     * Creates a popup menu for the specified object.
     */
    public void displayPopup(Graph graph, GraphItem graphItem, int x, int y) {
        //final String methLabel = "displayPopup";
        JPopupMenu popup = getDefaultPopup(graphItem);
        popup.setLabel(graphItem.getLabel());

        if (!(graphItem instanceof GraphNodeGroup)) {
            if (graphItem instanceof GraphNodeSet) {
                // separate Sample actions from default graph options
                popup.addSeparator();
                popup.addSeparator();
            }
            JMenuItem menuItem = popup.add("Action1");
            menuItem.setIcon(null);
            menuItem.setFont(GraphConstants.GUIFont);
            popup.addSeparator();
        }

        popup.setInvoker(graph); // avoids truncating at window's border
        popup.show(graph, x, y);
    }

    /**
     * Implementation of ActionListener interface.
     *
     * @param e
     *            the ActionEvent
     */
    public void actionPerformed(ActionEvent evt) {
        final String methLabel = "actionPerformed";
        TopoTrace.debug(this, methLabel, "evt=" + evt);
        super.actionPerformed(evt);
        int id = evt.getID();
        switch (id) {
        case 1:
            //###performAction1();
            break;

        default:
            TopoTrace.error("ERR_InvalidAction id=" + id);
        }
    }

    /**
     * Returns the ToolTip text currently associated with the specific item.
     * Depending on showToolTipText variable, returns a full or reduced text.
     */
    public String getToolTipText(GraphItem graphItem) {
        // reduced tool tip text only for edges and multiedges
        if (this.showToolTipText) {
            return (graphItem.getToolTipText());
        }
        return (null);
    }

    /**
     * Returns the color associated with the item (selection,...)
     */
    public Paint getContextBackground(GraphItem graphItem) {
        Paint p = null;
        if (graphItem instanceof GraphNodeGroup)
            return (null);
        if (isSelected(graphItem)) {
            p = GraphConstants.selectionPaint;
        } else if (isTarget(graphItem)) {
            p = GraphConstants.targetPaint;
        } else if (isHighlighted(graphItem)) {
            p = GraphConstants.highlightPaint;
        }
        return (p);
    }

    public void dress(GraphItem graphItem) {
        final String methLabel = "dress";
        TopoTrace.debug(this, methLabel, "==> graphItem=" + graphItem);
        if (graphItem instanceof GraphNodeGroup) {
            dressGroup((GraphNodeGroup) graphItem);
        } else if (graphItem instanceof GraphContainer) {
            dressContainer((GraphContainer) graphItem);
        } else if (graphItem instanceof GraphNode) {
            dressNode((GraphNode) graphItem);
        } else if (graphItem instanceof GraphPort) {
            dressPort((GraphPort) graphItem);
        } else if (graphItem instanceof GraphEdge) {
            dressEdge((GraphEdge) graphItem);
        } else if (graphItem instanceof GraphMultiEdge) {
            dressMultiEdge((GraphMultiEdge) graphItem);
        }
    }

    public void dressGroup(GraphNodeGroup group) {
        boolean isLayeredLayout = false;
        GraphLayoutManager groupLayout = group.getLayout();
        if (groupLayout instanceof GraphLayeredLayout) {
            group.setIcon(BPEM.createSanIcon("layeredLayout.gif"));
            isLayeredLayout = true;
        } else if (groupLayout instanceof GraphCircleLayout) {
            group.setIcon(BPEM.createSanIcon("circleLayout.gif"));
        } else if (groupLayout instanceof GraphFlowLayout) {
            if (groupLayout.getOrientation() == GraphLayoutManager.HORIZONTAL) {
                group.setIcon(BPEM.createSanIcon("hrzFlowLayout.gif"));
            } else {
                group.setIcon(BPEM.createSanIcon("vrtFlowLayout.gif"));
            }
        }
        String toolTipText = "*** Subgraph ***\n";
        if (group.getKey().equals("#")) {
            group.setLabel("Root");
            // use same background as graph panel
            //### utiliser 2 preferences: graphBg et modeBg
            group.setBackground(Color.white);
        } else if (group.getKey().equals("#0")) {
            group.setLabel("All");
            toolTipText += "All Nodes";
        } else if (group.getKey().equals("##")) {
            group.setLabel("Others");
            toolTipText += "Nodes not connected";
            if (isLayeredLayout) {
                groupLayout.setAlignment(GraphLayoutManager.LEFT);
            }
        } else {
            String subnetNb = group.getKey().substring(1);
            group.setLabel(subnetNb);
            toolTipText += "Sub-network " + subnetNb
            + " of interconnected Nodes";
        }
        group.setToolTipText(toolTipText);
        group.setLimitColor(Color.gray);
        group.setLimitStroke(GraphConstants.DashedStroke);
    }

    public void dressContainer(GraphContainer container) {
        //Hashtable sampleObject = (Hashtable) container.getUserObject();
        //container.setLabel((String) sampleObject.get("Name"));
        ModelNode sampleObject = (ModelNode) container.getUserObject();
        container.setLabel(sampleObject.getLongName());
        container.setContextBackground(getContextBackground(container));
        //        String sampleObjectType = (String) sampleObject.get("Type");
        //        container.setIcon(new ImageIcon("sample/images/" + sampleObjectType
        //                + ".gif"));
        //###container.setLayer(FABRIC_LAYER);
        container.setLimitColor(Color.black);
    }

    public void dressNode(GraphNode node) {
        //        Hashtable sampleObject = (Hashtable) node.getUserObject();
        //        node.setLabel((String) sampleObject.get("Name"));
        ModelNode sampleObject = (ModelNode) node.getUserObject();
        node.setLabel(sampleObject.getLongName());
        node.setContextBackground(getContextBackground(node));
        //        String sampleObjectType = (String) sampleObject.get("Type");
        //        node
        //                .setIcon(new ImageIcon("sample/images/" + sampleObjectType
        //                        + ".gif"));
        //###node.setLayer(HOST_LAYER);
    }

    public void dressPort(GraphPort port) {
        //        Hashtable sampleObject = (Hashtable) port.getUserObject();
        //        port.setLabel((String) sampleObject.get("Name"));
        ModelPort sampleObject = (ModelPort) port.getUserObject();
        port.setLabel(sampleObject.getKey());
        port.setContextBackground(getContextBackground(port));
        port.setIcon(GraphIcon.getIcon(GraphIcon.SQUARE, 6, 6, Color.gray,
                Color.black));
    }

    public void dressEdge(GraphEdge edge) {
        //        Hashtable sampleObject = (Hashtable) edge.getUserObject();
        //        edge.setLabel((String) sampleObject.get("Name"));
        ModelEdge sampleObject = (ModelEdge) edge.getUserObject();
        edge.setLabel(sampleObject.getKey());
        edge.setContextBackground(getContextBackground(edge));
        edge.setCurveColor(Color.black);
    }

    public void dressMultiEdge(GraphMultiEdge multi) {
        multi.setContextBackground(getContextBackground(multi));
        // compute complex label
        String multiLabel = "";
        for (int i = 0; i < multi.nbOfEdges(); i++) {
            GraphEdge edge = multi.getEdge(i);
            String edgeLabel = edge.getLabel();
            if (edgeLabel == null)
                edgeLabel = "<not labelled>";
            multiLabel += "" + i + ": " + edgeLabel + "\n";
        }
        multi.setLabel(multiLabel);
        multi.setCurveColor(Color.blue);
    }
}