/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.Graph;
import graph.GraphAdapter;
import graph.GraphInterface;
import graph.TopoTrace;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;

/**
 * BPEMGraphitGraphArea.java
 *
 * @author Bill Murray
 * @version $Id: BPEMGraphitGraphArea.java 5 2006-10-20 14:37:44Z lejeuneg $
 * <p>
 * This class is a customized GraphArea for BPEM. Unfortunately, this
 * class can not extend GraphArea because that base class provides no
 * way to not have the visual elements it creates.
 */
public class BPEMGraphitGraphArea extends JPanel {

    public final String RCSId = "@(#) $Id: BPEMGraphitGraphArea.java 5 2006-10-20 14:37:44Z lejeuneg $";

    protected BPEMGraphitGraphArea area;

    protected Graph graph;

    protected JScrollPane graphScroller;

    protected GraphInterface newGraphInterface;

    protected GraphAdapter graphAdapter;

    protected String prefKey;

    /**
     * BPEMGraphitGraphArea - constructor
     *
     * @param graph - The graph structure
     */
    public BPEMGraphitGraphArea(Graph graph) {
        this.area = this;
        this.graph = graph;
        this.graphScroller = new JScrollPane(graph);
        this.graphScroller.getViewport().setScrollMode(JViewport.BLIT_SCROLL_MODE);
        graph.setScrollPane(this.graphScroller);
        this.graphAdapter = graph.getAdapter();
        //
        // initialize preferences
        //
        this.prefKey = "GraphArea";
        applyPreferences();

        setLayout(new BorderLayout());
        add(this.graphScroller, BorderLayout.CENTER);
    }

    public String getPrefKey() {
        return this.prefKey;
    }

    public void applyPreferences() {
        final String methLabel = "applyPreferences";
        TopoTrace.debug(this, methLabel, "==> ");
        //String prefKey = getPrefKey();
        //###autoRefresh =
        // Preferences.getPreference(prefKey+".autoRefresh",autoRefresh);
        TopoTrace.debug(this, methLabel, "<== ");
    }

    public void reScale(int value) {
        this.graphAdapter.setScale((double) value / 100);
        this.graph.setLayoutPhase(0);
        this.graph.repaint();

    }
}