/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.GraphEdge;
import graph.GraphMultiEdge;
import graph.GraphNode;
import graph.GraphPort;

/**
 * BPEMGraphitGraphMultiEdge.java
 *
 * @author Bill Murray
 * @version $Id: BPEMGraphitGraphMultiEdge.java,v 1.2 2005/02/28 21:17:34
 *          bmurray Exp $
 *          <p>
 *          This class extends the GraphMultiEdge to account for the way we need
 *          to hide edges.
 */
public class BPEMGraphitGraphMultiEdge extends GraphMultiEdge {

    /**
     * @param key
     */
    public BPEMGraphitGraphMultiEdge(String key) {
        super(key);
    }

    /**
     * isHidden - Determines if this edge is to be displayed or not. This is
     * complicated by the "underretracted" states of the ports and parents.
     * Basically, the edge keeps no state. Instead, it checks to see if any of
     * the ports are hidden or underretracted, or if any parent in the parent
     * chain is hidden or underretracted.
     */
    public boolean isHidden() {
        if (!this.graphAdapter.showMulti)
            return (true);
        for (int i = 0; i < this.vPorts.size(); i++) {
            GraphPort port = (GraphPort) this.vPorts.get(i);
            // Hide if any ports/parents are hidden
            if (anyHiddenPort(port))
                return true;
        }
        return (super.isHidden());
    }

    private boolean anyHiddenPort(GraphPort port) {
        if (port.isUnderRetracted())
            return true;
        if (port.isHidden())
            return true;
        return anyHiddenParent(port.getNode());
    }

    private boolean anyHiddenParent(GraphNode node) {
        if (node == null)
            return false;
        if (node.isHidden())
            return true;
        if (node.isUnderRetracted())
            return true;

        return anyHiddenParent(node.getParent());
    }

    /**
     * intersects - We want an intersection only if the edge
     * passed in has a common source or destination port with
     * one of our edges. In other words, to have an intersection,
     * the edge direction needs to be the same.
     */
    public boolean intersects(GraphEdge edge) {

        for (int i = 0; i < this.nbOfEdges(); i++) {
            GraphEdge myEdge = this.getEdge(i);
            if (myEdge.getSrcPort() == edge.getSrcPort()) return true;
            if (myEdge.getDstPort() == edge.getDstPort()) return true;
        }
        return false;
    }
}