/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

/**
 * TVCellRenderer.java
 *
 * @author Bill Murray
 * @version $Id: TVCellRenderer.java 5 2006-10-20 14:37:44Z lejeuneg $
 *          <p>
 *          This class extends DefaultTreeCellRenderer and is responsible for
 *          setting the tree view icon and tooltip.
 */
public class TVCellRenderer extends DefaultTreeCellRenderer {

    private ImageIcon iconExecuted;

    private ImageIcon iconNotExecuted;

    /**
     *
     */
    public TVCellRenderer() {
        super();
        this.iconExecuted = createImageIcon("images/check.gif", "Statement executed");
        this.iconNotExecuted = createImageIcon("images/nocheck.gif",
        "Statement not executed");
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.swing.tree.TreeCellRenderer#getTreeCellRendererComponent(javax.swing.JTree,
     *      java.lang.Object, boolean, boolean, boolean, int, boolean)
     */
    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean sel, boolean expanded, boolean leaf, int row,
            boolean hasFocus) {
        super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
                row, hasFocus);

        DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
        TVNodeInfo nodeInfo;

        try {
            nodeInfo = (TVNodeInfo) node.getUserObject();
            if (nodeInfo != null) {
                if (nodeInfo.getModelNode().isExecuted()) {
                    setIcon(this.iconExecuted);
                } else {
                    setIcon(this.iconNotExecuted);
                }
                setToolTipText(nodeInfo.getModelNode().getAllAttrs());
            }
        } catch (Exception ex) {
            // Nothing
        }

        return this;
    }

    protected static ImageIcon createImageIcon(String path, String description) {
        java.net.URL imgURL = TVCellRenderer.class.getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL, description);
        }
        System.err.println("Couldn't find file: " + path);
        return null;
    }

}