/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import javax.swing.JTable;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.objectweb.orchestra.bpem.model.ExecutionModel;
import org.objectweb.orchestra.bpem.model.ModelListener;
import org.objectweb.orchestra.bpem.model.ModelNode;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;



/**
 * ViewAsProperties.java
 *
 * @author Bill Murray
 * @version $Id: ViewAsProperties.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class ViewAsProperties extends JTable implements ModelListener {

    private final ExecutionModel model;
    private TableColumnModel tcModel;
    private TableColumn nameColumn;
    private TableColumn valueColumn;
    private PropertyTableModel tableModel;
    private ModelNode prevNode = null;

    public ViewAsProperties(ExecutionModel model) {
        super(new PropertyTableModel(0, 2));
        this.model = model;

        model.addListener(this);

        this.tcModel = this.getColumnModel();
        this.nameColumn = this.tcModel.getColumn(0);
        this.nameColumn.setHeaderValue("Name");
        this.nameColumn.setMaxWidth(300);
        this.nameColumn.setMinWidth(170);
        this.valueColumn = this.tcModel.getColumn(1);
        this.valueColumn.setHeaderValue("Value");
        this.tableModel = (PropertyTableModel) this.getModel();
    }

    public void clear() {
        this.tableModel.setRowCount(0);
    }

    /**
     * displayAll - This is called by the data model when a new BPEL
     * process has been selected (The "clear" funciton is called first
     * by the model) We want to display data for the first node as the
     * initial display.
     */
    public void displayAll() {
        nodeSelected(this.model.getModelRoot().getFirstChildModelNode());
    }

    public void nodeSelected(ModelNode mn) {
        this.prevNode = mn;
        refresh();
    }

    /**
     * refresh - called from the model and also internally. Note
     * that we do not do a refresh if this window is not "Showing"
     */
    public void refresh() {
        if ((this.prevNode != null) && (isShowing() == true)) {
            dispNode(this.prevNode);
        }
    }

    private void dispNode(ModelNode mn) {
        NamedNodeMap attrs = mn.getAttributes();

        clear();
        this.tableModel.setRowCount(attrs.getLength());

        for (int j = 0; j < attrs.getLength(); j++) {
            Node tmp = attrs.item(j);
            //
            // Put attr name into table
            //
            String name = tmp.getNodeName();
            this.setValueAt(name, j, 0);
            //
            // Put attr value into table
            //
            String value = tmp.getNodeValue();
            this.setValueAt(value, j, 1);
        }
    }

}