/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.client;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.objectweb.orchestra.data.BpelProcessData;
import org.objectweb.orchestra.exception.BPELAdminException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.SimpleCallbackHandler;
import org.objectweb.orchestra.util.TestElement;
import org.objectweb.orchestra.util.Util;

import org.objectweb.orchestra.admin.BpelAdminSession;
import org.objectweb.orchestra.admin.BpelAdminSessionHome;
import org.objectweb.orchestra.admin.BpelAdminSessionUtil;

import org.objectweb.orchestra.engine.Manager;
import org.objectweb.orchestra.engine.ManagerHome;
import org.objectweb.orchestra.engine.ManagerUtil;

/**
 * AdminProcessClient.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: AdminProcessClient.java 47 2006-11-28 09:24:16Z souillac $
 *
 */
public class AdminProcessClient {

    private static String sep = File.separator;

    public static void main(String args[]) throws LoginException {
        System.out.println("Running AdminProcessClient...");

        char[] password = BPELConstants.PASSWORD.toCharArray();
        SimpleCallbackHandler handler = new SimpleCallbackHandler(BPELConstants.USERNAME, password);
        LoginContext lc = new LoginContext(BPELConstants.LOGIN_MODULE, handler);
        lc.login();

        String choice = args[0];
        System.out.println("choice = " + choice);

        try {
            if (choice.equals("create")) {
                create(args[1]);
            } else if (choice.equals("delete")) {
                deleteBPELProcess(args[1]);
            } else if (choice.equals("deploy")) {
                String[] outServices = null;
                if (args.length>=6) {
                    outServices = new String[args.length - 6];
                    for (int i = 6; i < args.length; i++) {
                        outServices[i - 6] = args[i];
                    }
                }
                String srcDir=args[4];
                if (srcDir.equals("null")) {
                    srcDir = null;
                }
                boolean skipwsdl = false;
                if (args[5].equalsIgnoreCase("true")) {
                    skipwsdl = true;
                }
                deploy(args[1], args[2], args[3], srcDir, outServices, skipwsdl);
            } else if (choice.equals("undeploy")) {
                undeploy(args[1]);
            } else if (choice.equals("deployWS")) {
                String[] outServices = null;
                if (args.length>=4) {
                    outServices = new String[args.length - 4];
                    for (int i = 4; i < args.length; i++) {
                        outServices[i - 4] = args[i];
                    }
                }
                String srcDir=args[3];
                if (srcDir.equals("null")) {
                    srcDir = null;
                }
                deployWS(args[1], args[2], srcDir, outServices);
            } else if (choice.equals("launch")) {
                String arguments[] = new String[args.length - 3];
                for (int i = 3; i < args.length; i++) {
                    arguments[i - 3] = args[i];
                }
                launch(args[1], args[2], arguments);
            } else if (choice.equals("printDeployed")) {
                printDeployed();
            } else if (choice.equals("printUndeployed")) {
                printUndeployed();
            } else if (choice.equals("printProcess")) {
                printProcess(args[1]);
            } else if (choice.equals("deployTest")) {
                deployTest(args[1]);
            } else if (choice.equals("launchAllTests")) {
                launchAllTests(args[1]);
            } else if (choice.equals("cleanAllTests")) {
                cleanAllTests();
            } else if (choice.equals("printProcessEntries")) {
                printProcessEntries();
            } else if (choice.equals("restart")) {
                restart(args[1], Integer.parseInt(args[2]));
            } else if (choice.equals("deployUtils")) {
                deployExternalWS();
            } else if (choice.equals("restartEngine")) {
                restartEngine();
            } else if (choice.equals("compileClient")) {
                String processName = args[1];
                String dir = args[2];
                String includes = null;
                String excludes = null;
                String classpath = null;
                for (int i=3; i+1<args.length; i=i+2) {
                    if (args[i].equals("-i")) {
                        includes = args[i+1];
                    } else if (args[i].equals("-e")) {
                        excludes = args[i+1];
                    } else if (args[i].equals("-ccp")) {
                        classpath = args[i+1];
                    }
                }
                compileClient(processName, dir, includes, excludes, classpath);
            } else if (choice.equals("engineMode")) {
                engineMode(args[1]);
            } else if (choice.equals("monitor")) {
                monitor(args[1]);
            } else if (choice.equals("deploySamples")) {
                deployAllSamples();
            }

        } catch (Exception e) {
            System.out.println("Exception caught in AdminProcessClient : \n"+e);
            printExceptionCauses(e);
        }
        System.out.println("Finishing AdminProcessClient...");

    }

    private static void printExceptionCauses(Throwable e) {
        while (e.getCause() != null) {
            System.out.println("Caused by : "+e.getCause());
            e = e.getCause();
        }
    }

    private static void create(String processName) throws Exception {
        System.out.println("AdminProcessClient create processName = " + processName);
        // creator hard coded for now. Could be a parameter
        BpelAdminSession as = getAdmin();
        as.createProcess(processName, "bpeladmin");
        as.remove();
    }

    private static void deleteBPELProcess(String processName) throws Exception {
        System.out.println("AdminProcessClient delete processName = " + processName);
        try {
            BpelAdminSession as = getAdmin();
            as.deleteBPELProcess(processName);
            as.remove();
        } catch (BPELAdminException e) {
            System.out.println("Exception caught in AdminProcessClient.delete : "+e);
            printExceptionCauses(e);
        }
    }

    private static void deploy(String processName, String bpelFile, String wsdlFile, String srcDir, String[] outServices,
            boolean skipwsdl) throws Exception {
        String out = null;
        if (outServices != null) {
            out="";
            for (int i=0; i<outServices.length; i++) {
                out = out + outServices[i] + ", ";
            }
        }
        System.out.println("AdminProcessClient deploy processName = " + processName + ", bpelFile = " + bpelFile
                + ", wsdlFile = "+wsdlFile+ ", srcDir = "+srcDir+", outServices = "+out + ", skipwsdl = " + skipwsdl);
        BpelAdminSession as = getAdmin();
        as.deployProcess(processName, bpelFile, wsdlFile, outServices, srcDir, skipwsdl);
        as.remove();
    }

    private static void undeploy(String processName) throws Exception {
        System.out.println("AdminProcessClient undeploy processName = " + processName);
        BpelAdminSession as = getAdmin();
        as.undeployBPELProcess(processName);
        as.remove();
    }

    private static void deployWS(String processName, String wsdlFile, String srcDir, String[] outServices) throws Exception {
        String out = null;
        if (outServices != null) {
            out="";
            for (int i=0; i<outServices.length; i++) {
                out = out + outServices[i] + ", ";
            }
        }
        System.out.println("AdminProcessClient deployWS processName = " + processName + ", wsdlFile = " + wsdlFile
                + ", srcDir = "+srcDir+", outServices = "+out);
        BpelAdminSession as = getAdmin();
        as.deployProcess(processName, null, wsdlFile, outServices, srcDir, false);
        as.remove();
    }

    private static void launch(String processName, String className, String[] arguments) throws Exception {
        String args = null;
        if (arguments != null) {
            args="";
            for (int i=0; i<arguments.length; i++) {
                args = args + arguments[i] + ", ";
            }
        }
        System.out.println("AdminProcessClient launch processName = " + processName + ", className = " + className
                + ", arguments = "+args);
        BpelAdminSession as = getAdmin();
        as.launchProcess(processName, className, arguments);
        as.remove();
    }

    private static void printDeployed() throws Exception {
        System.out.println("AdminProcessClient printDeployed");
        BpelAdminSession as = getAdmin();
        Collection c = as.getDeployedProcesses();
        as.remove();
        if (c.size() <= 1) {
            System.out.println("There is " + c.size() + " deployed process");
        } else {
            System.out.println("There are " + c.size() + " deployed processes");
        }
        for (Iterator it = c.iterator(); it.hasNext();) {
            BpelProcessData p = (BpelProcessData) it.next();
            System.out.println("id = " + p.getProcessId() + " ; name = " + p.getProcessName() + " ; deployed = "
                    + p.getDeployed());
        }
    }

    private static void printUndeployed() throws Exception {
        System.out.println("AdminProcessClient printUndeployed");
        BpelAdminSession as = getAdmin();
        Collection c = as.getUndeployedProcesses();
        as.remove();
        if (c.size() <= 1) {
            System.out.println("There is " + c.size() + " undeployed process");
        } else {
            System.out.println("There are " + c.size() + " undeployed processes");
        }
        for (Iterator it = c.iterator(); it.hasNext();) {
            BpelProcessData p = (BpelProcessData) it.next();
            System.out.println("id = " + p.getProcessId() + " ; name = " + p.getProcessName() + " ; deployed = "
                    + p.getDeployed());
        }
    }

    private static void printProcess(String processName) throws Exception {
        System.out.println("AdminProcessClient printProcess processName = " + processName);
        BpelAdminSession as = getAdmin();
        BpelProcessData p = as.getProcess(processName);
        as.remove();
        System.out.println("Process Name : " + p.getProcessName());
        System.out.println("Process Id : " + p.getProcessId());
        System.out.println("Process deployed : " + p.getDeployed());
    }

    private static void deployTest(String processName) throws Exception {
        System.out.println("AdminProcessClient deployTest processName = " + processName);
        Vector tests = getTests();
        String[] outServices = null;
        for (int i=0; i<tests.size(); i++) {
            TestElement te = (TestElement) tests.get(i);
            if (te.getProcessName().equals(processName)) {
                outServices = te.getOutServices();
                break;
            }
        }
        if (outServices != null && outServices.length > 0) {
            deployExternalWS();
        }
        String testRep = BPELConstants.TESTS_JUNIT_PACK_DIR+sep+processName;
        String base = testRep+sep+processName;
        String bpelFile = base + ".bpel";
        String wsdlFile = base + ".wsdl";
        deploy(processName, bpelFile, wsdlFile, testRep, outServices, false);
    }

    private static void deployAllSamples() throws Exception {
        System.out.println("AdminProcessClient deployAllSamples");
        Vector samples = readTests("sample");
        String jars = BPELConstants.BUILD_JARS_DIR;
        String pathSeparator = Util.getPathSeparator();
        String cp = "";
        for (int i = 0; i < samples.size(); i++) {
            String processName = ((TestElement) samples.get(i)).getProcessName();
            String sampleRep = BPELConstants.SAMPLES_DIR+sep+processName;
            String base = sampleRep+sep+processName;
            String bpelFile = base+".bpel";
            String wsdlFile = base+".wsdl";
            deleteBPELProcess(processName);
            create(processName);
            deploy(processName, bpelFile, wsdlFile, sampleRep+sep+BPELConstants.SAMPLES_CLIENT_PACK,
                    ((TestElement) samples.get(i)).getOutServices(), false);
            cp = cp + jars + File.separator + processName +".jar" + pathSeparator;
        }
        compileClient("unitSamples", BPELConstants.TESTS_JUNIT_PACK_DIR+sep+"unitSamples", "", "", cp);
    }

    private static void deployAllTests() throws Exception {
        System.out.println("AdminProcessClient deployAllTests");
        Vector tests = getTests();
        for (int i = 0; i < tests.size(); i++) {
            String processName = ((TestElement) tests.get(i)).getProcessName();
            String testRep = BPELConstants.TESTS_JUNIT_PACK_DIR+sep+processName;
            String base = testRep+sep+processName;
            String bpelFile = base+".bpel";
            String wsdlFile = base+".wsdl";
            deleteBPELProcess(processName);
            create(processName);
            deploy(processName, bpelFile, wsdlFile, testRep, ((TestElement) tests.get(i)).getOutServices(), false);
        }
    }

    private static void cleanAllTests() throws Exception {
        System.out.println("AdminProcessClient cleanAllTests");
        Vector tests = getTests();
        for (int i = 0; i < tests.size(); i++) {
            String processName = ((TestElement) tests.get(i)).getProcessName();
            deleteBPELProcess(processName);
        }
        Vector extWS = getExtWS();
        for (int i = 0; i < extWS.size(); i++) {
            String processName = ((String[]) extWS.get(i))[0];
            undeployExtWS(processName);
        }
        Vector samples = readTests("sample");
        for (int i = 0; i < samples.size(); i++) {
            String processName = ((TestElement) samples.get(i)).getProcessName();
            deleteBPELProcess(processName);
        }
    }

    private static void undeployExtWS(String processName) throws Exception {
        System.out.println("AdminProcessClient undeployExtWS processName = " + processName);
        BpelAdminSession as = getAdmin();
        as.undeployExtWS(processName);
        as.remove();
    }

    private static void launchAllTests(String launchonly) throws Exception {
        System.out.println("AdminProcessClient launchAllTests " + launchonly);
        if (launchonly.equals("false")) {
            deployExternalWS();
            deployAllSamples();
            deployAllTests();
        }
        launchJunit();
    }

    private static Vector readTests(String prefix) throws Exception {
        Vector v = new Vector();
        HashMap alias = new HashMap();
        String file = BPELConstants.TESTS_DIR + sep + "testsList.txt";

        File f = new File(file);
        BufferedReader br = new BufferedReader(new FileReader(f));
        String line = br.readLine();
        while (line != null) {
            String[] elements = line.split(":");
            if (elements.length >= 1) {
                if (elements[0].equals(prefix) && (elements[0].equals("test") || elements[0].equals("sample"))) {
                    //it is not a comment
                    //substring remove " character
                    String processName = elements[1].substring(1, elements[1].length() - 1);
                    String[] outServices = new String[0];
                    if (elements.length>2) {
                        String outWs = elements[2].substring(elements[2].indexOf("\"")+1);
                        outWs = outWs.substring(0, outWs.indexOf("\""));
                        outServices = outWs.split(",");
                        for (int i = 0; i<outServices.length; i++) {
                            String st = outServices[i];
                            st = BPELConstants.BPEL_HOME + sep + (String) alias.get(st);
                            outServices[i] = st;
                        }
                    }
                    TestElement tmp = new TestElement(processName, outServices);
                    v.addElement(tmp);
                } else if (elements[0].equals("alias")) {
                    String key = elements[1].substring(0, elements[1].indexOf("="));
                    String value = elements[1].substring(elements[1].indexOf("=")+2);
                    value = value.substring(0, value.indexOf("\""));
                    alias.put(key, value);
                } else if (elements[0].equals(prefix) && elements[0].equals("extws")) {
                    String processName = elements[1].substring(1, elements[1].length() - 1);
                    String wsdlAlias = elements[2].substring(elements[2].indexOf("=")+2);
                    wsdlAlias = wsdlAlias.substring(0, wsdlAlias.indexOf("\""));
                    String wsdl = BPELConstants.BPEL_HOME + sep + (String) alias.get(wsdlAlias);
                    String implDir = elements[3].substring(elements[3].indexOf("=")+2);
                    implDir = BPELConstants.BPEL_HOME + sep + implDir.substring(0, implDir.indexOf("\""));
                    String[] extWSTab = new String[]{processName, wsdl, implDir};
                    v.addElement(extWSTab);
                }
            }
            line = br.readLine();
        }
        return v;
    }

    private static Vector getTests() throws Exception {
        return readTests("test");
    }

    private static Vector getExtWS() throws Exception {
        return readTests("extws");
    }

    private static void deployExternalWS() throws Exception {
        System.out.println("AdminProcessClient deployExternalWS");
        Vector extWs = getExtWS();
        BpelAdminSession as = getAdmin();
        as.deployUtils(extWs);
        as.remove();
    }

    private static void launchJunit() throws Exception {
        System.out.println("AdminProcessClient launchJunit");
        BpelAdminSession as = getAdmin();
        as.launchJunit();
        as.remove();
    }

    private static void printProcessEntries() throws Exception {
        System.out.println("AdminProcessClient printProcessEntries");
        BpelAdminSession as = getAdmin();
        Collection processes = as.getDeployedProcesses();
        for (Iterator it = processes.iterator(); it.hasNext();) {
            BpelProcessData p = (BpelProcessData) it.next();
            String processName = p.getProcessName();
            System.out.println("### Process : "+processName);
            Vector processEntries = (Vector) as.getProcessEntries(processName);
            System.out.println("   wsdl = "+processEntries.get(0));
            for (int i=1; i+5<processEntries.size(); i=i+6) {
                System.out.println();
                System.out.println("   name      = "+processEntries.get(i));
                System.out.println("   service   = "+processEntries.get(i+1));
                System.out.println("   port      = "+processEntries.get(i+2));
                System.out.println("   binding   = "+processEntries.get(i+3));
                System.out.println("   portType  = "+processEntries.get(i+4));
                System.out.println("   operation = "+processEntries.get(i+5));
            }
        }
        as.remove();
    }

    private static void restart(String processName, int bpiId) throws Exception {
        System.out.println("AdminProcessClient restart, processName = "+processName+" bpiId = "+bpiId);
        Manager ms = getManager();
        ms.restart(processName, bpiId);
        ms.remove();
    }

    private static void restartEngine() throws Exception {
        System.out.println("AdminProcessClient restartEngine");
        Manager ms = getManager();
        ms.restartEngine();
        ms.remove();
    }

    private static void compileClient(String processName, String dir, String includes, String excludes, String classpath)
    throws Exception {
        System.out.println("AdminProcessClient compileClient processName = " + processName+", dir = "+dir+", includes = "
                + includes + ", excludes = " + excludes + ", classpath = " + classpath);
        BpelAdminSession as = getAdmin();
        as.compileClient(processName, dir, includes, excludes, classpath);
        as.remove();
    }

    private static void engineMode(String mode) throws Exception {
        System.out.println("AdminProcessClient setEngineMode mode = " + mode);
        int intMode = BPELConstants.WITH_EJB;
        if (mode.equals("mem")) {
            intMode = BPELConstants.WITHOUT_EJB;
        }
        BpelAdminSession as = getAdmin();
        as.setEngineMode(intMode);
        as.remove();
    }

    private static void monitor(String monitor) throws Exception {
        System.out.println("AdminProcessClient monitor monitor = " + monitor);
        BpelAdminSession as = getAdmin();
        as.setMonitor(monitor);
        as.remove();
    }

    private static BpelAdminSession getAdmin() throws Exception {
        BpelAdminSessionHome home = BpelAdminSessionUtil.getHome();
        return home.create();
    }

    private static Manager getManager() throws Exception {
        ManagerHome home = ManagerUtil.getHome();
        return home.create();
    }
}
