/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.client;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;

import org.objectweb.orchestra.exception.BPELAdminException;
import org.objectweb.orchestra.util.BPELConstants;

import org.objectweb.orchestra.admin.InsertAuthRoleUser;
import org.objectweb.orchestra.admin.InsertAuthRoleUserHome;
import org.objectweb.orchestra.admin.InsertAuthRoleUserUtil;
import org.objectweb.orchestra.admin.InsertDfltSetup;
import org.objectweb.orchestra.admin.InsertDfltSetupHome;
import org.objectweb.orchestra.admin.InsertDfltSetupUtil;


/**
 * DbInitClient.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: DbInitClient.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class DbInitClient {

    public static void main(final String args[]) {
        System.out.println("Running DbInitClient...");

        InsertAuthRoleUserHome lHome = null;
        InsertAuthRoleUser lBean = null;
        InsertDfltSetupHome sHome = null;
        InsertDfltSetup sBean = null;

        try {
            lHome = InsertAuthRoleUserUtil.getHome();
            lBean = lHome.create();
            sHome = InsertDfltSetupUtil.getHome();
            sBean = sHome.create();
        } catch (NamingException e) {
            System.out.println("NamingException in DbInitClient :\n" + e);
        } catch (CreateException e) {
            System.out.println("CreateException in DbInitClient :\n" + e);
        } catch (RemoteException e) {
            System.out.println("RemoteException in DbInitClient :\n" + e);
        }

        try {
            lBean.initializeUser("bpeladmin", "bpeladmin", "bpel@bull.net");
            lBean.initializeAuthRole("admin", "admin");
            lBean.initialize("bpeladmin", "admin");
            sBean.initializeDfltSetup(true, true, BPELConstants.DEBUG, BPELConstants.DEBUG, BPELConstants.ARCHIVE);
        } catch (RemoteException e) {
            System.out.println("RemoteException in DbInitClient when initializing database :\n"+ e);
        } catch (BPELAdminException e) {
            System.out.println("BPELAdminException in DbInitClient when initializing database :\n"+ e);
        }

        try {
            lBean.remove();
            sBean.remove();
        } catch (RemoteException e) {
            System.out.println("RemoteException in DbInitClient when initializing database :\n"+ e);
        } catch (RemoveException e) {
            System.out.println("RemoveException in DbInitClient when initializing database :\n"+ e);
        }

        System.out.println("Finishing DbInitClient...");

    }

}
