/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.client;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.objectweb.orchestra.admin.Activity;
import org.objectweb.orchestra.data.BPIData;
import org.objectweb.orchestra.exception.BPELAdminException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.SimpleCallbackHandler;

import org.objectweb.orchestra.admin.BpelAdminSession;
import org.objectweb.orchestra.admin.BpelAdminSessionHome;
import org.objectweb.orchestra.admin.BpelAdminSessionUtil;


/**
 * TestAdminClient.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: TestAdminClient.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class TestAdminClient {

    public static void main(final String[] args) throws LoginException {
        System.out.println("TestAdminClient");

        char[] password = BPELConstants.PASSWORD.toCharArray();
        SimpleCallbackHandler handler = new SimpleCallbackHandler(BPELConstants.USERNAME, password);
        LoginContext lc = new LoginContext(BPELConstants.LOGIN_MODULE, handler);
        lc.login();

        //testInstanceActivities(args[0]);

        //testCreateDeployProcess();

        //testSetEngineMode(BPELConstants.WITHOUT_EJB);

        //testCleanBPI(Integer.parseInt(args[0]));

        testGetRunningInstances(args[0], Integer.parseInt(args[1]), Integer.parseInt(args[2]));

    }

    public static void testInstanceActivities(String processName) {
        BpelAdminSessionHome oHome = null;
        BpelAdminSession operator = null;
        try {
            oHome = BpelAdminSessionUtil.getHome();
            operator = oHome.create();
            Collection inst = operator.getProcessInstances(processName);
            for (Iterator i = inst.iterator(); i.hasNext();) {
                BPIData pi = (BPIData) i.next();
                System.out.println("Instance : " + pi.getBPIId());
                Collection acti = operator.getInstanceActivities(pi.getBPIId());
                for (Iterator j = acti.iterator(); j.hasNext();) {
                    Activity act = (Activity) j.next();
                    System.out.println("Activity : statement nb" + act.getStatementNb() + ", type : " + act.getActivityType());
                }
                System.out.println("end instance : "+ pi.getBPIId());
            }
            operator.remove();
        } catch (javax.ejb.CreateException e) {
            System.out.println("CreateException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (NamingException e) {
            System.out.println("NamingException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (RemoteException e) {
            System.out.println("RemoteException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (BPELAdminException e) {
            System.out.println("BPELAdminException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (RemoveException e) {
            System.out.println("RemoveException in TestAdminClient :\n" + e);
            System.exit(2);
        }
    }

    public static void testCleanBPI(int bpiId) {
        BpelAdminSessionHome oHome = null;
        BpelAdminSession operator = null;
        try {
            oHome = BpelAdminSessionUtil.getHome();
            operator = oHome.create();
            operator.terminateInstance(bpiId);
            operator.remove();
        } catch (javax.ejb.CreateException e) {
            System.out.println("CreateException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (NamingException e) {
            System.out.println("NamingException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (RemoteException e) {
            System.out.println("RemoteException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (BPELAdminException e) {
            System.out.println("BPELAdminException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (RemoveException e) {
            System.out.println("RemoveException in TestAdminClient :\n" + e);
            System.exit(2);
        }
    }

    public static void testSetEngineMode(int mode) {
        BpelAdminSessionHome oHome = null;
        BpelAdminSession operator = null;
        try {
            oHome = BpelAdminSessionUtil.getHome();
            operator = oHome.create();
            operator.setEngineMode(mode);
            operator.remove();
        } catch (javax.ejb.CreateException e) {
            System.out.println("CreateException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (NamingException e) {
            System.out.println("NamingException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (RemoteException e) {
            System.out.println("RemoteException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (RemoveException e) {
            System.out.println("RemoveException in TestAdminClient :\n" + e);
            System.exit(2);
        }
    }

    public static void testGetRunningInstances(String processName, int offset, int numrows) {
        BpelAdminSessionHome aHome = null;
        BpelAdminSession admin = null;
        try {
            aHome = BpelAdminSessionUtil.getHome();
            admin = aHome.create();
            Collection result = admin.getRunningInstances(processName, offset, numrows);
            admin.remove();
            Iterator it = result.iterator();
            while (it.hasNext()) {
                it.next();
                System.out.println("ici");
            }
        } catch (javax.ejb.CreateException e) {
            System.out.println("CreateException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (NamingException e) {
            System.out.println("NamingException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (RemoteException e) {
            System.out.println("RemoteException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (RemoveException e) {
            System.out.println("RemoveException in TestAdminClient :\n" + e);
            System.exit(2);
        } catch (BPELAdminException e) {
            System.out.println("BPELAdminException in TestAdminClient :\n" + e);
            System.exit(2);
        }
    }

}
