/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import javax.ejb.EntityContext;

/**
 * BpelDfltSetupBean.java
 *
 * @version $Id: BpelDfltSetupBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb.bean name="BpelDfltSetupBean"
 *           description="Deployment descriptor for the default setup bean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/BpelDfltSetupHome"
 *           local-jndi-name="org/objectweb/orchestra/data/BpelDfltSetupHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.BpelDfltSetupHomeLocal"
 * @ejb.interface local-class="org.objectweb.orchestra.data.BpelDfltSetupLocal"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @ejb.permission role-name="admin"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelDfltSetup"
 *                     automatic-pk="true"
 *
 * @ejb:finder signature="org.objectweb.orchestra.data.BpelDfltSetupLocal findRow()"
 * 			query="SELECT OBJECT(o) From BpelDfltSetupBean o"
 * 			result-type-mapping="Local"
 * 			transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 * @jonas.max-cache-size @BpelDfltSetupBean.max-cache-size@
 */

public abstract class BpelDfltSetupBean implements javax.ejb.EntityBean {

    /**
     * EJBCREATE
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(boolean log, boolean trace, String loglevel, String tracelevel, String historic) throws javax.ejb.CreateException {
        setDLog(log);
        setDTrace(trace);
        setDLogLevel(loglevel);
        setDTraceLevel(tracelevel);
        setDHistoric(historic);
        return null;
    }

    // persistent fields

    /**
     * @return Integer : The BPI Id
     *
     * @ejb.pk-field
     * @ejb.interface-method
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @param id :
     *            Id of the BPI
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * Retrieve the Default log attribute.
     *
     * @return Returns a boolean representing the default log attribute.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="log"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract boolean getDLog();

    /**
     * Set the Default log attribute.
     *
     * @param log The default log attribute. Is set at creation time.
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setDLog( boolean log);

    /**
     * Retrieve the default LogLevel attribute.
     *
     * @return Returns a string representing the default LogLevel attribute.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="loglevel"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract String getDLogLevel();

    /**
     * Set the default logLevel attribute (Debug/Info/Error).
     *
     * @param historic The default LogLevel attribute. Is set at creation time.
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setDLogLevel( String traceLevel);

    /**
     * Retrieve the default trace.
     *
     * @return Returns a boolean representing the default trace attribute.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="trace"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract boolean getDTrace();

    /**
     * Set the default trace attribute.
     *
     * @param trace The default trace attribute. Is set at creation time.
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setDTrace( boolean trace);

    /**
     * Retrieve the default TraceLevel attribute.
     *
     * @return Returns a string representing the default TraceLevel attribute.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="tracelevel"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract String getDTraceLevel();

    /**
     * Set the default traceLevel attribute (Debug/Info/Error).
     *
     * @ejb:interface-method
     * @param historic The default TraceLevel attribute. Is set at creation time.
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setDTraceLevel( String traceLevel);

    /**
     * Retrieve the default Historic attribute.
     *
     * @return Returns a string representing the default Historic attribute.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="historic"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract String getDHistoric();

    /**
     * Set the default Historic attribute (Purge/Archive).
     *
     * @ejb:interface-method
     * @param historic The default Historic attribute. Is set at creation time.
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setDHistoric( String historic);

    public void setEntityContext(EntityContext ec) {
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbRemove() throws javax.ejb.RemoveException {
        // Bean Method
    }

}
