/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.util.Collection;
import java.util.Date;

import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import org.objectweb.jonas.service.ServiceException;

/**
 * BpelProcessBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BpelProcessBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb.bean name="BpelProcessBean"
 *           description="Deployment descriptor for the process bean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/BpelProcessHome"
 *           local-jndi-name="org/objectweb/orchestra/data/BpelProcessHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.BpelProcessHomeLocal"
 * @ejb.interface local-class="org.objectweb.orchestra.data.BpelProcessLocal"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelProcess"
 *                     automatic-pk="true"
 *
 * @ejb:finder signature="java.util.Collection findAllBPI()"
 *             query="SELECT OBJECT(o) From BpelProcessBean o"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.data.BPIEntityLocal findById(java.lang.Integer bpiId)"
 *             query="SELECT OBJECT(o) From BpelProcessBean as o Where o.id = ?1"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.data.BpelProcessLocal findByName(java.lang.String name)"
 *             query="SELECT OBJECT(o) From BpelProcessBean as o Where o.name = ?1"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findDeployed()"
 *             query="SELECT OBJECT(o) From BpelProcessBean as o Where o.deployed = true"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findUndeployed()"
 *             query="SELECT OBJECT(o) From BpelProcessBean as o Where o.deployed = false"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 * @jonas.max-cache-size @BpelProcessBean.max-cache-size@
 */

public abstract class BpelProcessBean implements javax.ejb.EntityBean {

    /**
     * EJBCREATE
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(String processName, String creator) throws javax.ejb.CreateException {
        setName(processName);
        setCreator(creator);
        BpelDfltSetupHomeLocal bshl = null;
        BpelDfltSetupLocal bsl = null;
        try {
            bshl = BpelDfltSetupBeanUtil.getLocalHome();
            bsl = bshl.findRow();
            this.setLog(bsl.getDLog());
            this.setTrace(bsl.getDTrace());
            this.setLogLevel(bsl.getDLogLevel());
            this.setTraceLevel(bsl.getDTraceLevel());
            this.setHistoric(bsl.getDHistoric());
            setCreationDate(new Date());
        } catch (NamingException e) {
            throw new ServiceException(e.getMessage());
        } catch (FinderException e) {
            throw new ServiceException(e.getMessage());
        }
        return null;
    }

    // persistent fields

    /**
     * @return Integer : The Process Id
     *
     * @ejb.pk-field
     * @ejb.interface-method
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @param id : Id of the Process
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * @return Name of the Process
     *
     * @ejb:persistence column-name="name"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getName();

    /**
     * @param name :
     *            Name of the Process
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setName(String name);

    /**
     * @return Name of the Process
     *
     * @ejb:persistence column-name="creator"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getCreator();

    /**
     * @param name :
     *            Name of the Process
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setCreator(String creator);

    /**
     * @return The creation date of the Process
     *
     * @ejb:persistence column-name="creationDate"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Date getCreationDate();

    /**
     * @param state : the creation date of the Process
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setCreationDate(Date creationDate);

    /**
     * @return Boolean true if this process is deployed
     *
     * @ejb:persistence column-name="deployed"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Boolean getDeployed();

    /**
     * @param deployed :
     *            the deployment state of this process
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setDeployed(Boolean deployed);

    /**
     * Retrieve the Process log attribute.
     *
     * @return Returns a boolean representing the log attribute of this Process.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="log"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract boolean getLog();

    /**
     * Set the Process log attribute.
     *
     * @param log The log attribute of this Process. Is set at creation time.
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setLog( boolean log);

    /**
     * Retrieve the Process LogLevel attribute.
     *
     * @return Returns a string representing the LogLevel attribute of this Process.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="loglevel"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract String getLogLevel();

    /**
     * Set the Process logLevel attribute (Debug/Info/Error).
     *
     * @param historic The LogLevel attribute of this Process. Is set at creation time.
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setLogLevel( String traceLevel);

    /**
     * Retrieve the Process trace.
     *
     * @return Returns a boolean representing the trace attribute of this Process.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="trace"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract boolean getTrace();

    /**
     * Set the Process trace attribute.
     *
     * @param trace The trace attribute of this Process. Is set at creation time.
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setTrace( boolean trace);

    /**
     * Retrieve the Process TraceLevel attribute.
     *
     * @return Returns a string representing the TraceLevel attribute of this Process.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="tracelevel"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract String getTraceLevel();

    /**
     * Set the Process traceLevel attribute (Debug/Info/Error).
     *
     * @ejb:interface-method
     * @param historic The TraceLevel attribute of this Process. Is set at creation time.
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setTraceLevel( String traceLevel);

    /**
     * Retrieve the Process Historic attribute.
     *
     * @return Returns a string representing the Historic attribute of this Process.
     * @ejb:interface-method
     *
     * @ejb:persistence column-name="historic"
     * @ejb:transaction type="@tx.entity.getMethod@"
     **/
    public abstract String getHistoric();

    /**
     * Set the Process Historic attribute (Purge/Archive).
     *
     * @ejb:interface-method
     * @param historic The Historic attribute of this Process. Is set at creation time.
     * @ejb:transaction type="@tx.entity.setMethod@"
     **/
    public abstract void setHistoric( String historic);

    /**
     * @return the BPIs of this Process
     *
     * @ejb:interface-method
     * @ejb:relation name="Process-BPI"
     *               role-name="Process-has-BPI"
     *               target-multiple="yes"
     * @ejb:transaction type="@tx.entity.getMethod@"
     *
     */
    public abstract Collection getBPIs();

    /**
     * Set the BPIs of this Process
     *
     * @param BPIs :
     *            the BPIs of this process
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPIs(Collection BPIs);

    /**
    *
    * @ejb.select
    *     signature="java.lang.Integer ejbGetDeployedProcessCnt()"
    *     view-type="local" result-type-mapping="Local"
    *     query="SELECT COUNT(p.id) From BpelProcessBean as p where p.deployed=true"
    *     transaction-type="Supports"
    *
    */
   public abstract int ejbSelectDeployedProcessCnt() throws FinderException;

   /**
    * @ejb.home-method view-type="local"
    * @ejb:transaction type="Supports"
    *
    **/
   public int ejbHomeGetDeployedProcessCnt() throws FinderException {
       return ejbSelectDeployedProcessCnt();
   }

   /**
   *
   * @ejb.select
   *     signature="java.lang.Integer ejbGetUnDeployedProcessCnt()"
   *     view-type="local" result-type-mapping="Local"
   *     query="SELECT COUNT(p.id) From BpelProcessBean as p where p.deployed=false"
   *     transaction-type="Supports"
   *
   */
  public abstract int ejbSelectUnDeployedProcessCnt() throws FinderException;

  /**
   * @ejb.home-method view-type="local"
   * @ejb:transaction type="Supports"
   *
   **/
  public int ejbHomeGetUnDeployedProcessCnt() throws FinderException {
      return ejbSelectUnDeployedProcessCnt();
  }

   public void setEntityContext(EntityContext ec) {
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbRemove() throws javax.ejb.RemoveException {
        // Bean Method
    }

}