/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.NoSuchObjectLocalException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;

import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.EntityBeansUtil;
import org.objectweb.orchestra.util.Util;


/**
 * CSCache.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: CSCache.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class CSCache {

    private static Vector cslist = new Vector();
    private static int csNb = 0;
    private static Object mutex = new Object();

    public static void createCS(int bpiId, String csName, Object csValue) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache createCS");
        }
        Object val;
        try {
            val = Util.putValueObject(csValue);
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException CSCache createCS : ", e);
        }
        if (BPELEngine.isWithEjb()) {
            try {
                CSHomeLocal cshl = CSBeanUtil.getLocalHome();
                cshl.create(new Integer(bpiId), csName, val);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException CSCache create :\n" , e);
            } catch (CreateException e) {
                throw new BPELEngineException("CreateException CSCache create :\n" , e);
            }
        } else {
            int csId;
            synchronized (mutex) {
                csId = csNb++;
            }
            CSData csd  = new CSData(csId, bpiId, csName, val);
            cslist.add(csd);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache end createCS");
        }
    }

    public static void createCS(int bpiId, String csName, Object csValue, int scopeId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache createCS");
        }
        Object val;
        try {
            val = Util.putValueObject(csValue);
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException CSCache createCS : ", e);
        }
        if (BPELEngine.isWithEjb()) {
            try {
                CSHomeLocal cshl = CSBeanUtil.getLocalHome();
                CSLocal csl = cshl.create(new Integer(bpiId), csName, val);
                ScopeLocal scope = EntityBeansUtil.findScopeBean(scopeId);
                csl.setScope(scope);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException CSCache create :\n" , e);
            } catch (CreateException e) {
                throw new BPELEngineException("CreateException CSCache create :\n" , e);
            }
        } else {
            int csId;
            synchronized (mutex) {
                csId = csNb++;
            }
            CSData csd  = new CSData(csId, bpiId, csName, val, scopeId);
            cslist.add(csd);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache end createCS");
        }
    }

    public static Collection findCS(String csName) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache findCS");
        }
        Vector result = new Vector();

        if (BPELEngine.isWithEjb()) {
            CSHomeLocal cshl;
            Collection csCol = null;
            try {
                cshl = CSBeanUtil.getLocalHome();
            } catch (NamingException e1) {
                throw new BPELEngineException("NamingException CSCache findCS :\n" , e1);
            }
            // ************ This code should be changed when the container will handle concurrent
            // ************ access to finder and ejbRemove
            while (true) {
                try {
                    csCol = cshl.findCS(csName);
                    break;
                } catch (FinderException e) {
                    //Should never happen if the EJB container synchronized
                    //finder methods and ejb set methods access
                    //a finder method returning a collection should never throw finderexception
                    //when no result is found, an empty collection is returned
                    sleep();
                }
            }
            result = convert(csCol);
        } else {
            synchronized (cslist) {
                for (Iterator it = cslist.iterator(); it.hasNext();) {
                    CSData csd = (CSData) it.next();
                    if (csd.getCSName().equals(csName)) {
                        result.add(csd);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache end findCS");
        }
        return result;
    }

    private static void sleep() {
        try {
            Thread.sleep(100);
        } catch (InterruptedException e2) {
            e2.printStackTrace();
        }
    }

    public static Vector findByBpiNameAndScope(int bpiId, String csName, int scopeId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache findByNameAndBpi");
        }
        Vector result = new Vector();

        if (BPELEngine.isWithEjb()) {
            CSHomeLocal cshl;
            Collection csCol = null;
            try {
                cshl = CSBeanUtil.getLocalHome();
                csCol = cshl.findByBpiNameAndScope(new Integer(bpiId), csName, new Integer(scopeId));
                result = convert(csCol);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException CSCache findByNameAndBpi :\n" , e);
            } catch (FinderException e) {
                // Nothing
                //throw new BPELEngineException("FinderException CSCache findByNameAndBpi :\n" , e);
            }
        } else {
            synchronized (cslist) {
                for (Iterator it = cslist.iterator(); it.hasNext();) {
                    CSData csd = (CSData) it.next();
                    if ((csd.getCSName().equals(csName)) && (csd.getBPIId() == bpiId) && (csd.getScopeId() == scopeId)) {
                        result.add(csd);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache end findByNameAndBpi");
        }
        return result;
    }

    public static Vector findByNameAndBpi(String csName, int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache findByNameAndBpi");
        }
        Vector result = new Vector();

        if (BPELEngine.isWithEjb()) {
            CSHomeLocal cshl;
            Collection csCol = null;
            try {
                cshl = CSBeanUtil.getLocalHome();
                csCol = cshl.findByNameAndBpi(csName, new Integer(bpiId));
                result = convert(csCol);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException CSCache findByNameAndBpi :\n" , e);
            } catch (FinderException e) {
                // Nothing
                //throw new BPELEngineException("FinderException CSCache findByNameAndBpi :\n" , e);
            }
        } else {
            synchronized (cslist) {
                for (Iterator it = cslist.iterator(); it.hasNext();) {
                    CSData csd = (CSData) it.next();
                    if ((csd.getCSName().equals(csName)) && (csd.getBPIId() == bpiId)) {
                        result.add(csd);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache end findByNameAndBpi");
        }
        return result;
    }

    /**
     * Removes all CSBeans for a given bpi
     */
    public static void removeCS(int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache removeCS, bpiId = "+bpiId);
        }
        if (BPELEngine.isWithEjb()) {
            Collection csCol = null;
            CSLocal cs = null;
            try {
                CSHomeLocal cshl = CSBeanUtil.getLocalHome();
                csCol = cshl.findByBpi(bpiId);
            } catch (javax.naming.NamingException e) {
                throw new BPELEngineException("NamingException BPIMdb removeCS  : " , e);
            } catch (FinderException e1) {
                // Nothing
                //throw new BPELEngineException("FinderException BPIMdb removeCS  : " , e1);
            }

            if (csCol != null) {
                for (java.util.Iterator it = csCol.iterator(); it.hasNext();) {
                    cs = (CSLocal) it.next();
                    cs.getId().toString();
                    cs.getBPIId();
                    cs.getCSName();
                    try {
                        cs.remove();
                    } catch (EJBException e2) {
                        throw new BPELEngineException("EJBException BPIMdb removeCS  : " , e2);
                    } catch (RemoveException e2) {
                        throw new BPELEngineException("RemoveException BPIMdb removeCS  : " , e2);
                    }
                }
            }
        } else {
            Vector vectCS = new Vector();
            synchronized (cslist) {
                for (Iterator it = cslist.iterator(); it.hasNext();) {
                    CSData csd = (CSData) it.next();
                    if (csd.getBPIId() == bpiId) {
                        vectCS.add(csd);
                    }
                }
                cslist.removeAll(vectCS);
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache end removeCS");
        }
    }

    public static void setCSScope(int csId, int scopeId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache setCSScope");
        }
        if (BPELEngine.isWithEjb()) {
            try {
                CSHomeLocal cshl = CSBeanUtil.getLocalHome();
                CSLocal cs = cshl.findByPrimaryKey(new Integer(csId));
                ScopeHomeLocal shl = ScopeBeanUtil.getLocalHome();
                ScopeLocal scope = shl.findByPrimaryKey(new Integer(scopeId));
                cs.setScope(scope);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException CSCache setCSScope : ", e);
            } catch (FinderException e) {
                throw new BPELEngineException("FinderException CSCache setCSScope : ", e);
            }
        } else {
            synchronized (cslist) {
                for (Iterator it = cslist.iterator(); it.hasNext();) {
                    CSData csd = (CSData) it.next();
                    if (csd.getCSId() == csId) {
                        csd.setScopeId(scopeId);
                        break;
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache end setCSScope");
        }
    }

    public static void setCSValue(int csId, Object value) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache setCSValue");
        }
        Object val;
        try {
            val = Util.putValueObject(value);
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException CSCache createCS : ", e);
        }
        if (BPELEngine.isWithEjb()) {
            try {
                CSHomeLocal cshl = CSBeanUtil.getLocalHome();
                CSLocal cs = cshl.findByPrimaryKey(new Integer(csId));
                cs.setCSValue(val);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException CSCache setCSValue : ", e);
            } catch (FinderException e) {
                throw new BPELEngineException("FinderException CSCache setCSValue : ", e);
            }
        } else {
            synchronized (cslist) {
                for (Iterator it = cslist.iterator(); it.hasNext();) {
                    CSData csd = (CSData) it.next();
                    if (csd.getCSId() == csId) {
                        csd.setCSValue(val);
                        break;
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("CSCache setCSValue");
        }
    }

    private static Vector convert(Collection csLocalCol) {
        Vector result = new Vector();
        for (Iterator it = csLocalCol.iterator(); it.hasNext();) {
            CSLocal cs = (CSLocal) it.next();
            CSData tmp = convert(cs);
            if (tmp != null) {
                result.add(tmp);
            }
        }
        return result;
    }

    private static CSData convert(CSLocal csl) {
        if (csl !=null) {
            try {
                ScopeLocal s = csl.getScope();
                int scopeId;
                if (s == null) {
                    scopeId = -1;
                } else {
                    scopeId = s.getId().intValue();
                }
                return new CSData(
                        csl.getId().intValue(),
                        csl.getBPIId().intValue(),
                        csl.getCSName(),
                        csl.getCSValue(),
                        scopeId);
            } catch (NoSuchObjectLocalException e) {
                //this could happen if the cs is remove while the convert method
            }
        }
        return null;
    }
}
