/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * MessageBean.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: MessageBean.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 * @ejb.bean name="MessageBean"
 *           description="Deployment descriptor for the Message bean"
 *           type="CMP"
 *           view-type="both"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/MessageHome"
 *           local-jndi-name="org/objectweb/orchestra/data/MessageHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.MessageHomeLocal"
 *           remote-class="org.objectweb.orchestra.data.MessageHome"
 * @ejb.interface local-class="org.objectweb.orchestra.data.MessageLocal"
 *                remote-class="org.objectweb.orchestra.data.Message"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelMessage"
 *                     automatic-pk="true"
 *
 *
 * @ejb:finder signature= "java.util.Collection findAllMessage()"
 *             query ="SELECT OBJECT(o) FROM MessageBean o"
 *
 * @ejb:finder signature="org.objectweb.orchestra.data.MessageLocal findByType( java.lang.Class pClass )"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From MessageBean as u Where u.type = ?1"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.data.MessageLocal findByState( java.lang.String pstate )"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From MessageBean as u Where u.state = ?1"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByBPIAndStatementNb(int bpiId, int nb)"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From MessageBean as u Where u.bPI.id = ?1 and u.statementNb = ?2"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByBPIWithStatementNb(int bpiId)"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From MessageBean as u Where u.bPI.id = ?1 and u.statementNb <> -1"
 *             transaction-type="Supports"
 *
 *
 * @ejb:finder signature="java.util.Collection findByNoBpi()"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From MessageBean as u Where u.bPI IS NULL"
 *             transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 *
 * @jonas.max-cache-size @MessageBean.max-cache-size@
 */

public abstract class MessageBean implements EntityBean {

    /**
     * ejbCreate.
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    /**
     * ejbCreate.
     *
     * @param processName
     * @param targetNamespace
     * @param portType
     * @param operation
     * @param refPartner
     * @param value
     * @param statementNb
     * @param isFault
     * @param date
     * @param faultName
     * @param faultMessage
     * @return pk primary key set to null
     * @throws CreateException
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(String processName, String targetNamespace, String portType, String operation, String refPartner,
            Object value, int statementNb, boolean isFault, long date, String faultName, String faultMessage)
    throws CreateException {
        setPortType(portType);
        setOperation(operation);
        setRefPartner(refPartner);
        setValue(value);
        setStatementNb(statementNb);
        setProcessName(processName);
        setTargetNamespace(targetNamespace);
        setFault(isFault);
        setDate(date);
        setFaultName(faultName);
        setFaultMessage(faultMessage);
        setPartner(null);
        return null;
    }

    // persistent fields

    /**
     * @ejb.pk-field
     * @ejb:persistence column-name="id"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * @ejb:persistence column-name="processName"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getProcessName();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setProcessName(String processName);

    /**
     * @ejb:persistence column-name="targetNamespace"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getTargetNamespace();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setTargetNamespace(String targetNamespace);

    /**
     * @ejb:persistence column-name="value"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Object getValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setValue(Object value);

    /**
     * @ejb:persistence column-name="type"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Class getType();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setType(Class type);

    /**
     * @ejb:persistence column-name="state"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getState();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setState(String state);

    /**
     * @ejb:persistence column-name="namespace"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getNamespace();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setNamespace(String namespace);

    /**
     * @ejb:persistence column-name="statementNb"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract int getStatementNb();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setStatementNb(int statementNb);

    /**
     * @ejb:persistence column-name="portType"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getPortType();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setPortType(String portType);

    /**
     * @ejb:persistence column-name="operation"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getOperation();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setOperation(String operation);

    /**
     * @ejb:persistence column-name="partner"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getPartner();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setPartner(String partner);

    /**
     * @ejb:persistence column-name="refPartner"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getRefPartner();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setRefPartner(String refPartner);

    /**
     * @ejb:persistence column-name="fault"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract boolean getFault();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setFault(boolean fault);

    /**
     * @ejb:persistence column-name="faultName"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getFaultName();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setFaultName(String faultName);

    /**
     * @ejb:persistence column-name="faultMessage"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getFaultMessage();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setFaultMessage(String faultMessage);

    /**
     * @ejb:persistence column-name="date_"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract long getDate();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setDate(long date);

    /**
     *
     * Retrieve the BPI of this Message.
     *
     * @ejb:interface-method
     * @ejb:relation name="BPIMessage"
     *               role-name="is-bpi-of"
     *               target-multiple="yes"
     *               target-ejb="BPIEntityBean"
     *               cascade-delete="yes"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract org.objectweb.orchestra.data.BPIEntityLocal getBPI();

    /**
     * Set the BPI of this Message.
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPI(org.objectweb.orchestra.data.BPIEntityLocal pBPI);

    // standard call back methods

    public void setEntityContext(EntityContext ec) {
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbRemove() throws RemoveException {
        // Bean Method
    }

}
