/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.io.Serializable;

/**
 * MessageData.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: MessageData.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class MessageData implements Serializable{

    private int messageId;
    private String processName;
    private String targetNamespace;
    private String portType;
    private String operation;
    private String refPartner;
    private String partner;
    private Object value;
    private int bpiId;
    private int statementNb;
    private boolean isFault;
    private long date;
    private String faultName;
    private String faultMessage;

    public MessageData(int messageId, String processName, String targetNamespace, String portType, String operation,
            String refPartner, Object value, int statementNb, boolean isFault, long date, String faultName,
            String faultMessage, int bpiId) {
        this.messageId = messageId;
        this.processName = processName;
        this.targetNamespace = targetNamespace;
        this.portType = portType;
        this.operation = operation;
        this.refPartner = refPartner;
        this.value = value;
        this.statementNb = statementNb;
        this.isFault = isFault;
        this.date = date;
        this.faultName = faultName;
        this.faultMessage = faultMessage;
        this.bpiId = bpiId;
    }

    public MessageData(int messageId, String processName, String targetNamespace, String portType, String operation,
            String refPartner, Object value, int statementNb, boolean isFault, long date, String faultName,
            String faultMessage, int bpiId, String partner) {
        this(messageId, processName, targetNamespace, portType, operation, refPartner, value, statementNb, isFault,
                date, faultName, faultMessage, bpiId);
        this.partner = partner;
    }

    public int getMessageId() {
        return this.messageId;
    }

    public String getProcessName() {
        return this.processName;
    }

    public String getTargetNamespace() {
        return this.targetNamespace;
    }

    public String getPortType() {
        return this.portType;
    }

    public String getOperation() {
        return this.operation;
    }

    public String getRefPartner() {
        return this.refPartner;
    }

    public String getPartner() {
        return this.partner;
    }

    public Object getValue() {
        return this.value;
    }

    public int getStatementNb() {
        return this.statementNb;
    }

    public int getBPIId() {
        return this.bpiId;
    }

    public boolean isFault() {
        return this.isFault;
    }

    public long getDate() {
        return this.date;
    }

    public String getFaultName() {
        return this.faultName;
    }

    public String getFaultMessage() {
        return this.faultMessage;
    }

    // WARNING Setters should only be called from Cache Class
    public void setBPIId(int bpiId) {
        this.bpiId = bpiId;
    }

    public void setPartner(String partner) {
        this.partner = partner;
    }

    public void setStatementNb(int statementNb) {
        this.statementNb = statementNb;
    }
}
