/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.util.Collection;
import java.util.Date;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * ScopeBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ScopeBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 * 
 * @ejb.bean name="ScopeBean"
 *           description="Deployment descriptor for the Scope bean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/ScopeHome"
 *           local-jndi-name="org/objectweb/orchestra/data/ScopeHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.ScopeHomeLocal"
 *           remote-class="org.objectweb.orchestra.data.ScopeHome"
 * @ejb.interface local-class="org.objectweb.orchestra.data.ScopeLocal"
 *                remote-class="org.objectweb.orchestra.data.Scope"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelScope"
 *                     automatic-pk="true"
 *
 * @ejb.permission role-name="admin"
 *
 * @ejb:finder signature= "java.util.Collection findAllScope()"
 *             query ="SELECT OBJECT(o) FROM ScopeBean o"
 *
 * @ejb:finder signature="java.util.Collection findByNameOrderByDate(int pId, java.lang.String pName )"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From ScopeBean as u Where (u.id = ?1 and u.name = ?2) order by u.endDate"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByNb( java.lang.String myNb )"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From ScopeBean as u Where u.nb = ?1"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByNbAndFinished( java.lang.String myNb, java.lang.Boolean finished )"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From ScopeBean as u Where u.nb = ?1 and u.finished = ?2"
 *             transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 *
 * @jonas.max-cache-size @ScopeBean.max-cache-size@
 */

public abstract class ScopeBean implements EntityBean {

    /**
     * ejbCreate
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(String name) throws CreateException {
        setName(name);
        setFinished(new Boolean(true));
        return null;
    }

    // persistent fields

    /**
     * @ejb.pk-field
     * @ejb.interface-method
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * @ejb:persistence column-name="name"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getName();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setName(String name);

    /**
     * @ejb:persistence column-name="nb"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getNb();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setNb(String nb);

    /**
     * @ejb.interface-method
     * @ejb:persistence column-name="iter"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getIter();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setIter(Integer id);

    /**
     * @ejb.interface-method
     * @ejb:persistence column-name="finished"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Boolean getFinished();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setFinished(Boolean finished);

    /**
     * @ejb:persistence column-name="endDate"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Date getEndDate();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setEndDate(Date endDate);

    /**
     * Retrieve the variables of this Scope.
     *
     * @ejb:interface-method
     * @ejb:relation name="ScopeVariable"
     *               role-name="variables-of"
     *               target-ejb="VariableBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getVariables();

    /**
     * Set the variables of this Scope
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setVariables(Collection pVariables);

    /**
     *
     * Retrieve the BPI of this Scope.
     *
     * @ejb:interface-method
     * @ejb:relation name="BPIScope"
     *               role-name="is-bpi-of"
     *               cascade-delete="yes"
     *               target-multiple="yes"
     *               target-ejb="BPIEntityBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract org.objectweb.orchestra.data.BPIEntityLocal getBPI();

    /**
     * Set the BPI of this Scope.
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPI(org.objectweb.orchestra.data.BPIEntityLocal pBPI);

    /**
     * Retrieve the father of this Scope.
     *
     * @ejb:interface-method
     * @ejb:relation name="ScopeFather"
     *               role-name="father-of"
     *               target-ejb="ScopeBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract org.objectweb.orchestra.data.ScopeLocal getFather();

    /**
     * Set the Father of this Scope
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setFather(org.objectweb.orchestra.data.ScopeLocal pFather);

    /**
     * Retrieve the children of this Scope.
     *
     * @ejb:interface-method
     * @ejb:relation name="ScopeFather"
     *               role-name="children-of"
     *               target-multiple="yes"
     *               target-ejb="ScopeBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getChildren();

    /**
     * Set the children of this Scope
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setChildren(Collection pChildren);

    /**
     * @return the CSs of the Scope
     * @ejb:interface-method
     * @ejb:relation name="Scope-CS"
     *               role-name="Scope-has-CS"
     *               target-multiple="no"
     *               target-ejb="CSBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getCSs();

    /**
     * Set the CSs of the Scope
     *
     * @ejb:interface-method
     * @param CSs the CSs of the Scope
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setCSs(Collection CSs);

    // standard call back methods

    public void setEntityContext(EntityContext ec) {
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbRemove() throws RemoveException {
        // Bean Method
    }

}