/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELTrace;


/**
 * ScopeCache.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ScopeCache.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class ScopeCache {

    private static Vector scopeList = new Vector();
    private static int scopeNb = 0;
    private static Object mutex = new Object();

    public static int createScope(String scopeName) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache createScope");
        }
        int scopeId = -1;
        if (BPELEngine.isWithEjb()) {
            try {
                ScopeHomeLocal shl = ScopeBeanUtil.getLocalHome();
                ScopeLocal sl = shl.create(scopeName);
                scopeId = sl.getId().intValue();
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException ScopeCache createScope scopeName = " + scopeName + " :\n", e);
            } catch (CreateException e) {
                throw new BPELEngineException("CreateException ScopeCache createScope scopeName = " + scopeName + " :\n", e);
            }
        } else {
            synchronized (mutex) {
                scopeId = scopeNb++;
            }
            ScopeData sd  = new ScopeData(scopeId, scopeName);
            scopeList.add(sd);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end createScope");
        }
        return scopeId;
    }

    public static int createScope(String scopeName, String scopeNbName, int bpiId, int fatherScopeId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache createScope scopeName=" + scopeName
                    + ", scopeNbName=" + scopeNbName + ", bpiId=" + bpiId
                    + ", fatherScopeId=" + fatherScopeId);
        }
        int scopeId = -1;
        if (BPELEngine.isWithEjb()) {
            try {
                BPIEntityHomeLocal bpihl = BPIEntityBeanUtil.getLocalHome();
                BPIEntityLocal bpil = bpihl.findByPrimaryKey(new Integer(bpiId));
                ScopeHomeLocal sh = ScopeBeanUtil.getLocalHome();
                ScopeLocal sl = sh.create(scopeName);
                sl.setNb(scopeNbName);

                Collection col = sh.findByNb(scopeNbName);
                sl.setIter(new Integer(col.size()));
                Collection scopes = bpil.getScopes();
                scopes.add(sl);
                sl.setBPI(bpil);
                //if this scope has a father, associate them
                if (fatherScopeId != -1) {
                    ScopeLocal sFather = sh.findByPrimaryKey(new Integer(fatherScopeId));
                    sl.setFather(sFather);
                }
                scopeId = sl.getId().intValue();
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException ScopeCache createScope scopeName = " + scopeName + " :\n", e);
            } catch (FinderException e) {
                throw new BPELEngineException("FinderException ScopeCache createScope scopeName = " + scopeName + " :\n", e);
            } catch (CreateException e) {
                throw new BPELEngineException("CreateException ScopeCache createScope scopeName = " + scopeName + " :\n", e);
            }
        } else {
            synchronized (mutex) {
                scopeId = scopeNb++;
            }
            int iter = getIter(scopeNbName);
            ScopeData sd  = new ScopeData(scopeId, scopeName, bpiId, scopeNbName, iter, fatherScopeId);
            scopeList.add(sd);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end createScope scopeId=" + scopeId);
        }
        return scopeId;
    }

    public static ScopeData findScope(int scopeId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache findScope");
        }
        ScopeData result = null;
        if (BPELEngine.isWithEjb()) {
            ScopeLocal scope;
            try {
                ScopeHomeLocal sh = ScopeBeanUtil.getLocalHome();
                scope = sh.findByPrimaryKey(new Integer(scopeId));
                result = convert(scope);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException ScopeCache findScope scopeId = " + scopeId + " :\n", e);
            } catch (FinderException e) {
                // Nothing
                //throw new BPELEngineException("FinderException ScopeCache findScope scopeId = " + scopeId + " :\n", e);
            }
        } else {
            synchronized (scopeList) {
                for (Iterator it = scopeList.iterator(); it.hasNext();) {
                    ScopeData sd = (ScopeData) it.next();
                    if (sd.getScopeId() == scopeId) {
                        result = sd;
                        break;
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end findScope");
        }
        return result;
    }

    public static ScopeData getFather(int scopeId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache getFather scopeId=" + scopeId);
        }
        ScopeData result = null;
        if (BPELEngine.isWithEjb()) {
            ScopeLocal father;
            try {
                ScopeHomeLocal sh = ScopeBeanUtil.getLocalHome();
                ScopeLocal scope = sh.findByPrimaryKey(new Integer(scopeId));
                father = scope.getFather();
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException ScopeCache getFather scopeId = " + scopeId + " :\n", e);
            } catch (FinderException e) {
                throw new BPELEngineException("FinderException ScopeCache getFather scopeId = " + scopeId + " :\n", e);
            }
            result = convert(father);
        } else {
            int fatherId = -1;
            synchronized (scopeList) {
                for (Iterator it = scopeList.iterator(); it.hasNext();) {
                    ScopeData sd = (ScopeData) it.next();
                    if (sd.getScopeId() == scopeId) {
                        fatherId = sd.getFatherId();
                        break;
                    }
                }
                if (fatherId != -1) {
                    for (Iterator it = scopeList.iterator(); it.hasNext();) {
                        ScopeData sd = (ScopeData) it.next();
                        if (sd.getScopeId() == fatherId) {
                            result = sd;
                            break;
                        }
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end getFather result=" + result);
        }
        return result;
    }

    public static void setEndDate(int scopeId, Date d) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache setEndDate");
        }
        if (BPELEngine.isWithEjb()) {
            // Find Scope Bean
            ScopeHomeLocal sh = null;
            ScopeLocal scope = null;
            try {
                sh = ScopeBeanUtil.getLocalHome();
                scope = sh.findByPrimaryKey(new Integer(scopeId));
            } catch (javax.ejb.FinderException fe) {
                throw new BPELEngineException("ERROR (FinderException): Problem finding ScopeBean :\n" , fe);
            } catch (NamingException e) {
                throw new BPELEngineException("ERROR (NamingException): Problem finding ScopeBean :\n" , e);
            }
            scope.setEndDate(d);
        } else {
            synchronized (scopeList) {
                for (Iterator it = scopeList.iterator(); it.hasNext();) {
                    ScopeData sd = (ScopeData) it.next();
                    if (sd.getScopeId() == scopeId) {
                        sd.setEndDate(d);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end setEndDate");
        }
    }

    public static Collection getChildren(int scopeId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache getChildren");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            Collection children;
            try {
                ScopeHomeLocal sh = ScopeBeanUtil.getLocalHome();
                ScopeLocal scope = sh.findByPrimaryKey(new Integer(scopeId));
                children = scope.getChildren();
                result = convert(children);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException ScopeCache getChildren scopeId = " + scopeId + " :\n", e);
            } catch (FinderException e) {
                throw new BPELEngineException("FinderException ScopeCache getChildren scopeId = " + scopeId + " :\n", e);
            }
        } else {
            synchronized (scopeList) {
                for (Iterator it = scopeList.iterator(); it.hasNext();) {
                    ScopeData sd = (ScopeData) it.next();
                    if (sd.getFatherId() == scopeId) {
                        result.add(sd);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end getChildren");
        }
        return result;
    }

    public static Collection findByNameOrderByDate(int bpiId, String scopeName) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache findByNameOrderByDate");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            ScopeHomeLocal sh = null;
            Collection scopes = null;
            try {
                sh = ScopeBeanUtil.getLocalHome();
                scopes = sh.findByNameOrderByDate(bpiId, scopeName);
                result = convert(scopes);
            } catch (javax.ejb.FinderException e) {
                // Nothing
                // throw new BPELEngineException("FinderException ScopeCache findByNameOrderByDate bpiId = " + bpiId
                //+ ", scopeName = " + scopeName + " :\n", e);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException ScopeCache findByNameOrderByDate bpiId = " + bpiId
                        + ", scopeName = " + scopeName + " :\n", e);
            }
        } else {
            result = findByName(bpiId, scopeName);
            // TODO order result by date
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end findByNameOrderByDate");
        }
        return result;
    }

    // Only called when working without EJB
    private static Vector findByName(int bpiId, String scopeName) {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache findByName");
        }
        Vector result = new Vector();
        synchronized (scopeList) {
            for (Iterator it = scopeList.iterator(); it.hasNext();) {
                ScopeData sd = (ScopeData) it.next();
                if ((sd.getBPIId() == bpiId) && (sd.getScopeName().equals(scopeName))) {
                    result.add(sd);
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end findByName");
        }
        return result;
    }

    // Only called when working without EJB
    // With EJB relations are used
    public static Vector getBPIScopes(int bpiId) {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache getBPIScopes");
        }
        Vector result = new Vector();
        synchronized (scopeList) {
            for (Iterator it = scopeList.iterator(); it.hasNext();) {
                ScopeData sd = (ScopeData) it.next();
                if (sd.getBPIId() == bpiId) {
                    result.add(sd);
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end getBPIScopes");
        }
        return result;
    }

    // Only called when working without EJB
    // With EJB relations are used
    public static void removeBPIScopes(int bpiId) {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache removeBPIScopes");
        }
        Vector result = new Vector();
        synchronized (scopeList) {
            for (Iterator it = scopeList.iterator(); it.hasNext();) {
                ScopeData sd = (ScopeData) it.next();
                if (sd.getBPIId() == bpiId) {
                    result.add(sd);
                }
            }
            scopeList.removeAll(result);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end removeBPIScopes");
        }
    }

    public static Collection findByNb(String nb) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache findByNb");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            Collection scopes;
            try {
                ScopeHomeLocal sh = ScopeBeanUtil.getLocalHome();
                scopes = sh.findByNb(nb);
                result = convert(scopes);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException ScopeCache findByNb nb = " + nb + " :\n", e);
            } catch (FinderException e) {
                // Nothing
                //throw new BPELEngineException("FinderException ScopeCache findByNb nb = " + nb + " :\n", e);
            }
        } else {
            synchronized (scopeList) {
                for (Iterator it = scopeList.iterator(); it.hasNext();) {
                    ScopeData sd = (ScopeData) it.next();
                    if (sd.getNb().equals(nb)) {
                        result.add(sd);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end findByNb");
        }
        return result;
    }

    public static void setScopeFinished(int scopeId, boolean finished) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache setScopeFinished");
        }
        if (BPELEngine.isWithEjb()) {
            ScopeHomeLocal shl = null;
            ScopeLocal sl = null;
            try {
                shl = ScopeBeanUtil.getLocalHome();
                sl = shl.findByPrimaryKey(new Integer(scopeId));
                sl.setFinished(new Boolean(finished));
            } catch (javax.ejb.FinderException fe) {
                throw new BPELEngineException("FinderException ScopeCache setScopeFinished : \n", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException ScopeCache setScopeFinished :\n", e);
            }
        } else {
            synchronized (scopeList) {
                for (Iterator it = scopeList.iterator(); it.hasNext();) {
                    ScopeData sd = (ScopeData) it.next();
                    if (sd.getScopeId() == scopeId) {
                        sd.setFinished(finished);
                        break;
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("ScopeCache end setScopeFinished");
        }
    }

    public static Vector convert(Collection scopeLocalCol) {
        Vector result = new Vector();
        for (Iterator it = scopeLocalCol.iterator(); it.hasNext();) {
            ScopeLocal sl = (ScopeLocal) it.next();
            result.add(convert(sl));
        }
        return result;
    }

    private static int getIter(String nb) {
        int result = 0;
        synchronized (scopeList) {
            for (Iterator it = scopeList.iterator(); it.hasNext();) {
                ScopeData sd = (ScopeData) it.next();
                if (sd.getNb().equals(nb)) {
                    result++;
                }
            }
        }
        return result;
    }

    private static ScopeData convert(ScopeLocal sl) {
        if (sl != null) {
            ScopeLocal father = sl.getFather();
            int fatherId;
            if (father == null) {
                fatherId = -1;
            } else {
                fatherId = father.getId().intValue();
            }
            BPIEntityLocal bpi = sl.getBPI();
            int bpiId;
            if (bpi == null) {
                bpiId = -1;
            } else {
                bpiId = bpi.getId().intValue();
            }
            return new ScopeData(sl.getId().intValue(), sl.getName(), bpiId, sl.getNb(), sl.getIter().intValue(),
                    sl.getFinished().booleanValue(), sl.getEndDate(), fatherId);
        }
        return null;
    }
}
