/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * VariableBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: VariableBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb.bean name="VariableBean"
 *           description="Deployment descriptor for the Variable bean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/VariableHome"
 *           local-jndi-name="org/objectweb/orchestra/data/VariableHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.VariableHomeLocal"
 *           remote-class="org.objectweb.orchestra.data.VariableHome"
 * @ejb.interface local-class="org.objectweb.orchestra.data.VariableLocal"
 *                remote-class="org.objectweb.orchestra.data.Variable"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelVariable"
 *                     automatic-pk="true"
 *
 * @ejb:finder signature= "java.util.Collection findAllVariables()"
 *             query ="SELECT OBJECT(v) FROM VariableBean as v"
 *
 * @ejb:finder signature= "java.util.Collection findVariablesByBpiByNameAndByScope(java.lang.Integer id, java.lang.String name, java.lang.Integer sId)"
 *             query ="SELECT OBJECT(v) FROM VariableBean as v Where v.bPI.id = ?1 AND v.name = ?2 AND v.scope.id = ?3"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature= "java.util.Collection findVariablesByScope(java.lang.Integer id)"
 *             query ="SELECT OBJECT(v) FROM VariableBean as v Where v.scope.id = ?1"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 *
 * @jonas.cleanup create
 *
 * @jonas.max-cache-size @VariableBean.max-cache-size@
 */

public abstract class VariableBean implements javax.ejb.EntityBean {

    // Keep the reference on the EntityContext
    protected EntityContext entityContext;

    /* ========================= ejbCreate methods ============================ */

    /**
     * There must be one ejbCreate() method per create() method on the Home
     * interface, and with the same signature.
     *
     * @param
     *
     * @return
     * @ejb:create-method
     */
    public Integer ejbCreate(String name, String messageType, String type, String element, String BPELJType) throws CreateException, EJBException {
        setName(name);
        setMessageType(messageType);
        setType(type);
        setElement(element);
        setBPELJType(BPELJType);
        return null;
    }

    /*
     * ====================== javax.ejb.EntityBean implementation
     * =================
     */

    /**
     * A container invokes this method when the instance is taken out of the
     * pool of available instances to become associated with a specific EJB
     * object. This method transitions the instance to the ready state.
     *
     * This method executes in an unspecified transaction context.
     *
     * @exception EJBException
     *                Thrown by the method to indicate a failure caused by a
     *                system-level error.
     */
    public void ejbActivate() {
        // Nothing to be done for this simple example.
    }

    /**
     * A container invokes this method to instruct the instance to synchronize
     * its state by loading it state from the underlying database. This method
     * always executes in the proper transaction context.
     *
     * @exception EJBException
     *                Thrown by the method to indicate a failure caused by a
     *                system-level error.
     * @exception RemoteException -
     *                This exception is defined in the method signature to
     *                provide backward compatibility for enterprise beans
     *                written for the EJB 1.0 specification. Enterprise beans
     *                written for the EJB 1.1 and higher specification should
     *                throw the javax.ejb.EJBException instead of this
     *                exception.
     *
     */
    public void ejbLoad() {
        // Nothing to be done for this simple example, in implicit persistance.
    }

    /**
     * A container invokes this method on an instance before the instance
     * becomes disassociated with a specific EJB object. After this method
     * completes, the container will place the instance into the pool of
     * available instances.
     *
     * This method executes in an unspecified transaction context.
     *
     * @exception EJBException
     *                Thrown by the method to indicate a failure caused by a
     *                system-level error.
     */
    public void ejbPassivate() {
        // Nothing to be done for this simple example.
    }

    /**
     * A container invokes this method before it removes the EJB object that is
     * currently associated with the instance. This method is invoked when a
     * client invokes a remove operation on the enterprise Bean's home interface
     * or the EJB object's remote interface. This method transitions the
     * instance from the ready state to the pool of available instances.
     *
     * This method is called in the transaction context of the remove operation.
     *
     * @exception EJBException
     *                Thrown by the method to indicate a failure caused by a
     *                system-level error.
     * @exception RemoteException -
     *                This exception is defined in the method signature to
     *                provide backward compatibility for enterprise beans
     *                written for the EJB 1.0 specification. Enterprise beans
     *                written for the EJB 1.1 and higher specification should
     *                throw the javax.ejb.EJBException instead of this
     *                exception.
     * @exception RemoveException
     *                The enterprise Bean does not allow destruction of the
     *                object.
     *
     */
    public void ejbRemove() throws RemoveException {
        // Nothing to be done for this simple example, in implicit persistance.
    }

    /**
     * A container invokes this method to instruct the instance to synchronize
     * its state by storing it to the underlying database.
     *
     * This method always executes in the proper transaction context.
     *
     * @exception EJBException
     *                Thrown by the method to indicate a failure caused by a
     *                system-level error.
     * @exception RemoteException -
     *                This exception is defined in the method signature to
     *                provide backward compatibility for enterprise beans
     *                written for the EJB 1.0 specification. Enterprise beans
     *                written for the EJB 1.1 and higher specification should
     *                throw the javax.ejb.EJBException instead of this
     *                exception.
     */
    public void ejbStore() {
        // Nothing to be done for this simple example, in implicit persistance.
    }

    /**
     * Sets the associated entity context. The container invokes this method on
     * an instance after the instance has been created.
     *
     * This method is called in an unspecified transaction context.
     *
     * @param ctx -
     *            An EntityContext interface for the instance. The instance
     *            should store the reference to the context in an instance
     *            variable.
     * @exception EJBException
     *                Thrown by the method to indicate a failure caused by a
     *                system-level error.
     * @exception RemoteException -
     *                This exception is defined in the method signature to
     *                provide backward compatibility for enterprise beans
     *                written for the EJB 1.0 specification. Enterprise beans
     *                written for the EJB 1.1 and higher specification should
     *                throw the javax.ejb.EJBException instead of this
     *                exception.
     */
    public void setEntityContext(EntityContext ctx) {

        // Keep the entity context in object
        this.entityContext = ctx;
    }

    /**
     * Unsets the associated entity context. The container calls this method
     * before removing the instance. This is the last method that the container
     * invokes on the instance. The Java garbage collector will eventually
     * invoke the finalize() method on the instance.
     *
     * This method is called in an unspecified transaction context.
     *
     * @exception EJBException
     *                Thrown by the method to indicate a failure caused by a
     *                system-level error.
     * @exception RemoteException -
     *                This exception is defined in the method signature to
     *                provide backward compatibility for enterprise beans
     *                written for the EJB 1.0 specification. Enterprise beans
     *                written for the EJB 1.1 and higher specification should
     *                throw the javax.ejb.EJBException instead of this
     *                exception.
     */
    public void unsetEntityContext() {
        this.entityContext = null;
    }

    // persistent fields

    /**
     * @ejb:persistence column-name="id"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * @ejb:persistence column-name="name"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getName();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setName(String name);

    /**
     * @ejb:persistence column-name="value"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Object getValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setValue(Object value);

    /**
     * @ejb:persistence column-name="messageType"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getMessageType();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setMessageType(String messageType);

    /**
     * @ejb:persistence column-name="type"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getType();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setType(String type);

    /**
     * @ejb:persistence column-name="element"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getElement();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setElement(String element);

    /**
     * @ejb:persistence column-name="BPELJType"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getBPELJType();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPELJType(String BPELJType);

    /**
     *
     * Retrieve the BPI of this Variable.
     *
     * @ejb:interface-method
     * @ejb:relation name="BPIVariable"
     *               role-name="is-bpi-of"
     *               cascade-delete="yes"
     *               target-multiple="yes"
     *               target-ejb="BPIEntityBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract org.objectweb.orchestra.data.BPIEntityLocal getBPI();

    /**
     * Set the BPI of this Variable.
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPI(org.objectweb.orchestra.data.BPIEntityLocal pBPI);

    /**
     *
     * Retrieve the Scope of this Variable.
     *
     * @ejb:interface-method
     * @ejb:relation name="ScopeVariable"
     *               role-name="is-scope-of"
     *               cascade-delete="yes"
     *               target-multiple="yes"
     *               target-ejb="ScopeBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract org.objectweb.orchestra.data.ScopeLocal getScope();

    /**
     * Set the Scope of this Variable.
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setScope(org.objectweb.orchestra.data.ScopeLocal pScope);

}