/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import org.objectweb.orchestra.util.BPELConstants;

/**
 * BPELEngine.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPELEngine.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class BPELEngine {
    private static int mode = BPELConstants.WITH_EJB;
    private static String monitor = BPELConstants.MONITOR_ALL;
    private static int jbi = BPELConstants.JBI_OFF;

    public static int getMode() {
        //System.out.println("BPELEngine getMode : mode = " + mode);
        return mode;
    }

    public static void setMode(int m) {
        //System.out.println("BPELEngine setMode : new mode = " + m);
        mode = m;
    }

    public static boolean isWithEjb() {
        return (mode == BPELConstants.WITH_EJB);
    }

    public static String getMonitor() {
        return monitor;
    }

    public static void setMonitor(String m) {
        monitor = m;
    }

    public static boolean monitorRunning() {
        boolean result = false;
        result = result || monitor.equals(BPELConstants.MONITOR_ALL);
        result = result || monitor.equals(BPELConstants.MONITOR_RUNNING_AND_MESSAGES);
        result = result || monitor.equals(BPELConstants.MONITOR_RUNNING);
        return result;
    }

    public static boolean flushMessages() {
        boolean result = false;
        result = result || monitor.equals(BPELConstants.MONITOR_ALL);
        result = result || monitor.equals(BPELConstants.MONITOR_MESSAGES);
        result = result || monitor.equals(BPELConstants.MONITOR_RUNNING_AND_MESSAGES);
        return result;
    }

    public static boolean flushInstance() {
        return monitor.equals(BPELConstants.MONITOR_ALL);
    }

    /**
     * @return the jbi
     */
    public static int getJbi() {
        return jbi;
    }

    /**
     * @return true if jbi mode is on
     */
    public static boolean isJbi() {
        return jbi == BPELConstants.JBI_ON;
    }

    /**
     * @param jbi the jbi to set
     */
    public static void setJbi(int jbiMode) {
        jbi = jbiMode;
    }

}
