/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.ClassLoaderMap;


/**
 * BPELMessage.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPELMessage.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public abstract class BPELMessage implements java.io.Serializable, java.lang.Cloneable {

    private String messageName = null;

    public abstract Object getPart(String partName);

    public abstract void setPart(String partName, Object partValue);

    public abstract Object getPropertyValue(String prop);

    public abstract void setPropertyValue(String prop, Object value);

    public abstract boolean compare(String cs, Object corrValue);

    public abstract MessageStructure getStructure();

    public void setMessageName(String messageName) {
        this.messageName = messageName;
    }

    public String getMessageName() {
        return this.messageName;
    }

    /**
     * @return The Class of the BPELMessage
     */
    public Class getBPELMessageClass() {
        return this.getClass();
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#clone()
     */
    public Object clone() {
        BPELMessage jm = null;
        Object part = null;
        try {
            MessageStructure ms = getStructure();
            PartStructure ps[] = ms.getParts();
            jm = (BPELMessage) super.clone();
            for (int i = 0; i < ps.length; i++) {
                part = getPart(ps[i].getName());
                if (part != null) {
                    Class cl = part.getClass();
                    try {
                        Method met = cl.getDeclaredMethod("clone", (Class[]) null);
                        jm.setPart(ps[i].getName(), met.invoke(part, (Object[]) null));
                    } catch (Exception e) {
                        // Nothing
                    }
                }
            }
        } catch (CloneNotSupportedException cnse) {
            // This never happens
            return null;
        }
        return jm;
    }

    public void initializePart(String partname, String parttype, String processName) throws BPELEngineException {
        if (BPELTrace.isEngineDebugEnabled()) {
            BPELTrace.traceEngine("BPELMessage initializePart, partName = " + partname);
        }

        try {
            ClassLoader classL = ClassLoaderMap.getCL(processName);
            Class cl = Class.forName(parttype, true, classL);
            Constructor constr = cl.getDeclaredConstructor((Class[]) null);
            Object tmp = constr.newInstance((Object[]) null);
            this.setPart(partname, tmp);
        } catch (NoSuchMethodException e) {
            if (BPELTrace.isEngineDebugEnabled()) {
                BPELTrace.traceEngine("NoSuchMethodException BPELMessage initializePart :\n");
            }
        } catch (ClassNotFoundException ex) {
            throw new BPELEngineException("ClassNotFoundException BPELMessage initializePart :\n" , ex);
        } catch (InstantiationException ex) {
            throw new BPELEngineException("InstantiationException BPELMessage initializePart :\n" , ex);
        } catch (IllegalArgumentException ex) {
            throw new BPELEngineException("IllegalArgumentException BPELMessage initializePart :\n" , ex);
        } catch (IllegalAccessException ex) {
            throw new BPELEngineException("IllegalAccessException BPELMessage initializePart :\n" , ex);
        } catch (InvocationTargetException ex) {
            throw new BPELEngineException("InvocationTargetException BPELMessage initializePart :\n" , ex);
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException BPELMessage initializePart :\n" , e);
        }
        if (BPELTrace.isEngineDebugEnabled()) {
            BPELTrace.traceEngine("BPELMessage end initializePart");
        }
    }
}
