/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.MalformedObjectNameException;
import javax.management.NotificationFilter;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.TimerFilter;


/**
 * BPELTimerManager.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPELTimerManager.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public final class BPELTimerManager {

    private static BPELTimerManager tm = null; // Singleton pattern
    private static ObjectName timer;
    private static MBeanServer server;

    public void removeAlarms(int bpiId, PendingEvent pe) throws BPELEngineException {
        if (BPELTrace.isTimerDebugEnabled()) {
            BPELTrace.traceTimer("BPI " + bpiId + " removeAlarms");
        }
        Vector alarms = pe.getAlarms();
        for (int i=0; i<alarms.size(); i++) {
            String type = (String) alarms.get(i);
            Object[] params = new Object[]{type};
            String[] signature = new String[] {java.lang.String.class.getName()};
            invokeMBeanServer(bpiId, server, timer, "removeNotifications", params, signature);
            if (BPELTrace.isTimerDebugEnabled()) {
                BPELTrace.traceTimer("BPI " + bpiId + " removeAlarms with type = "+type);
            }
        }
        if (BPELTrace.isTimerDebugEnabled()) {
            BPELTrace.traceTimer("BPI " + bpiId + " End removeAlarms");
        }
    }

    public void launchTimer(int bpiId, Calendar c, int brancheNb, String type, int bpuId, int scopeId)
    throws BPELEngineException {
        if (BPELTrace.isTimerDebugEnabled()) {
            BPELTrace.traceTimer("BPI " + bpiId + " launchTimer : type + " + type + ", time : " + c.getTime());
        }
        Object[] params = new Object[4];
        params[0] = type; //type
        params[1] = bpiId + ":" + brancheNb + ":" + scopeId + ":" + bpuId; //message
        params[2] = "bpel"; //user object
        params[3] = c.getTime(); //delay time

        String[] signature = new String[] {"".getClass().getName(), "".getClass().getName(), "java.lang.Object",
                Date.class.getName()};
        invokeMBeanServer(bpiId, server, timer, "addNotification", params, signature);
        if (BPELTrace.isTimerDebugEnabled()) {
            BPELTrace.traceTimer("BPI " + bpiId + " Timer armed");
            BPELTrace.traceTimer("BPI " + bpiId + " End launchTimer");
        }
    }

    private Object invokeMBeanServer(int bpiId, MBeanServer server, ObjectName objName, String methodName,
            Object[] params, String[] signature) throws BPELEngineException {
        if (BPELTrace.isTimerDebugEnabled()) {
            BPELTrace.traceTimer("BPI " + bpiId + " invokeMBeanServer");
        }
        Object result = null;
        try {
            result = server.invoke(objName, methodName, params, signature);
        } catch (InstanceNotFoundException infe) {
            throw new BPELEngineException("InstanceNotFoundException BPIMdb invokeMBeanServer :\n" , infe);
        } catch (ReflectionException re) {
            throw new BPELEngineException("ReflectionException BPIMdb invokeMBeanServer :\n" , re);
        } catch (MBeanException mbe) {
            throw new BPELEngineException("MBeanException BPIMdb invokeMBeanServer :\n" , mbe);
        }
        if (BPELTrace.isTimerDebugEnabled()) {
            BPELTrace.traceTimer("BPI " + bpiId + " end invokeMBeanServer, result = "+result);
        }
        return result;
    }

    public static BPELTimerManager getInstance() throws BPELEngineException {
        if (tm == null) {
            tm = new BPELTimerManager();
        }
        return tm;
    }

    private BPELTimerManager() throws BPELEngineException {
        try{
            timer = new ObjectName(BPELConstants.BPELTimer);
            ObjectName receiver = new ObjectName(BPELConstants.BPELTimerReceiver);
            List list = MBeanServerFactory.findMBeanServer(null);
            server = (MBeanServer) list.iterator().next();
            NotificationFilter filter = new TimerFilter();
            server.addNotificationListener(timer, receiver, filter, null);
        } catch (MalformedObjectNameException e) {
            throw new BPELEngineException("MalformedObjectNameException caught in BPELTimerManager constructor : ", e);
        } catch (InstanceNotFoundException e) {
            throw new BPELEngineException("InstanceNotFoundException caught in BPELTimerManager constructor : ", e);
        }
    }
}


