/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.io.Serializable;

import org.objectweb.orchestra.tree.Invoke;

/**
 * InvokerObject.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: InvokerObject.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class InvokerObject implements Serializable {

    private String partnerLinkName = null;
    private String qnamePortType = null;
    private String invokeOperation = null;
    private BPELMessage inBM = null;
    private BPELMessage outBM = null;
    private String port = null;
    private String serviceLocatorClassName = null;
    private String address = null;
    private boolean requestResponse;
    private String processName = null;
    private int bpiId = -1;
    private int scopeId = -1;
    private WsdlBinding wsdlBinding = null;
    private org.objectweb.orchestra.tree.Invoke invoke = null;
    private String packag = null;
    private String targetNamespace = null;
    private boolean ret = false;
    private OrchestraServiceListener serviceListener = null;

    public InvokerObject(String partnerLinkName, String qnamePortType, String port, String serviceLocatorClassName,String invokeOperation, BPELMessage inBM, BPELMessage outBM, String address, boolean requestResponse, String processName, int bpiId, int scopeId, WsdlBinding wsdlBinding, Invoke invoke, String packag, String targetNamespace, OrchestraServiceListener serviceListener) {
        super();
        this.partnerLinkName = partnerLinkName;
        this.qnamePortType = qnamePortType;
        this.port = port;
        this.serviceLocatorClassName = serviceLocatorClassName;
        this.invokeOperation = invokeOperation;
        this.inBM = inBM;
        this.outBM = outBM;
        this.address = address;
        this.requestResponse = requestResponse;
        this.processName = processName;
        this.bpiId = bpiId;
        this.scopeId = scopeId;
        this.wsdlBinding = wsdlBinding;
        this.invoke = invoke;
        this.packag = packag;
        this.targetNamespace = targetNamespace;
        this.serviceListener = serviceListener;
    }
    /**
     * @return the address
     */
    public String getAddress() {
        return this.address;
    }
    /**
     * @param address the address to set
     */
    public void setAddress(String address) {
        this.address = address;
    }
    /**
     * @return the bpiId
     */
    public int getBpiId() {
        return this.bpiId;
    }
    /**
     * @param bpiId the bpiId to set
     */
    public void setBpiId(int bpiId) {
        this.bpiId = bpiId;
    }
    /**
     * @return the inBM
     */
    public BPELMessage getInBM() {
        return this.inBM;
    }
    /**
     * @param inBM the inBM to set
     */
    public void setInBM(BPELMessage inBM) {
        this.inBM = inBM;
    }
    /**
     * @return the invokeOperation
     */
    public String getInvokeOperation() {
        return this.invokeOperation;
    }
    /**
     * @param invokeOperation the invokeOperation to set
     */
    public void setInvokeOperation(String invokeOperation) {
        this.invokeOperation = invokeOperation;
    }
    /**
     * @return the outBM
     */
    public BPELMessage getOutBM() {
        return this.outBM;
    }
    /**
     * @param outBM the outBM to set
     */
    public void setOutBM(BPELMessage outBM) {
        this.outBM = outBM;
    }
    /**
     * @return the partnerLinkName
     */
    public String getPartnerLinkName() {
        return this.partnerLinkName;
    }
    /**
     * @param partLinkName the partnerLinkName to set
     */
    public void setParnertLinkName(String partnerLinkName) {
        this.partnerLinkName = partnerLinkName;
    }
    /**
     * @return the processName
     */
    public String getProcessName() {
        return this.processName;
    }
    /**
     * @param processName the processName to set
     */
    public void setProcessName(String processName) {
        this.processName = processName;
    }
    /**
     * @return the qnamePortType
     */
    public String getQnamePortType() {
        return this.qnamePortType;
    }
    /**
     * @param qnamePortType the qnamePortType to set
     */
    public void setQnamePortType(String qnamePortType) {
        this.qnamePortType = qnamePortType;
    }
    /**
     * @return the requestResponse
     */
    public boolean isRequestResponse() {
        return this.requestResponse;
    }
    /**
     * @param requestResponse the requestResponse to set
     */
    public void setRequestResponse(boolean requestResponse) {
        this.requestResponse = requestResponse;
    }
    /**
     * @return the scopeId
     */
    public int getScopeId() {
        return this.scopeId;
    }
    /**
     * @param scopeId the scopeId to set
     */
    public void setScopeId(int scopeId) {
        this.scopeId = scopeId;
    }
    /**
     * @return the port
     */
    public String getPort() {
        return this.port;
    }
    /**
     * @param port the port to set
     */
    public void setPort(String port) {
        this.port = port;
    }
    /**
     * @return the serviceLocatorClassName
     */
    public String getServiceLocatorClassName() {
        return this.serviceLocatorClassName;
    }
    /**
     * @param serviceLocatorClassName the serviceLocatorClassName to set
     */
    public void setServiceLocatorClassName(String serviceLocatorClassName) {
        this.serviceLocatorClassName = serviceLocatorClassName;
    }
    /**
     * @return the wsdlBinding
     */
    public WsdlBinding getWsdlBinding() {
        return this.wsdlBinding;
    }
    /**
     * @param wsdlBinding the wsdlBinding to set
     */
    public void setWsdlBinding(WsdlBinding wsdlBinding) {
        this.wsdlBinding = wsdlBinding;
    }
    /**
     * @return the invoke
     */
    public org.objectweb.orchestra.tree.Invoke getInvoke() {
        return this.invoke;
    }
    /**
     * @param invoke the invoke to set
     */
    public void setInvoke(org.objectweb.orchestra.tree.Invoke invoke) {
        this.invoke = invoke;
    }
    /**
     * @return the packag
     */
    public String getPackag() {
        return this.packag;
    }
    /**
     * @param packag the packag to set
     */
    public void setPackag(String packag) {
        this.packag = packag;
    }
    /**
     * @return the targetNamespace
     */
    public String getTargetNamespace() {
        return this.targetNamespace;
    }
    /**
     * @param targetNamespace the targetNamespace to set
     */
    public void setTargetNamespace(String targetNamespace) {
        this.targetNamespace = targetNamespace;
    }
    /**
     * @return the ret
     */
    public boolean isRet() {
        return this.ret;
    }
    /**
     * @param ret the ret to set
     */
    public void setRet(boolean ret) {
        this.ret = ret;
    }
    /**
     * @param partnerLinkName the partnerLinkName to set
     */
    public void setPartnerLinkName(String partnerLinkName) {
        this.partnerLinkName = partnerLinkName;
    }
    /**
     * @return the serviceListener
     */
    public OrchestraServiceListener getServiceListener() {
        return this.serviceListener;
    }
    /**
     * @param serviceListener the serviceListener to set
     */
    public void setServiceListener(OrchestraServiceListener serviceListener) {
        this.serviceListener = serviceListener;
    }

}


