/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.generator.PartnerLinkType;
import org.objectweb.orchestra.generator.Role;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;



/**
 * PLType.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: PLType.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class PLType implements Serializable {

    private static String myPath = BPELConstants.PLTYPES_DIR;
    private Vector plt = null;

    /**
     * Default Constructor
     *
     * @param processName
     * @throws FileNotFoundException
     * @throws IOException
     */
    public PLType(String processName) throws BPELEngineException {
        if (myPath.startsWith("null")) { throw new BPELEngineException("ProcessMap: "+BPELConstants.BPEL_HOME_PROPERTY+" property is not set",null); }
        try {
            File f = new File(myPath + File.separator + processName + ".txt");
            this.plt = new Vector();
            FileReader fr = new FileReader(f);
            BufferedReader br = new BufferedReader(fr);
            String line = br.readLine();
            while (line != null) {
                String[] elements = line.split(" ");

                PartnerLinkType p = new PartnerLinkType(Util.buildQname(elements[0]));
                for (int i = 1; (i + 1) < elements.length; i++) {
                    Role r = new Role(elements[i], Util.buildQname(elements[i + 1]));
                    p.addRole(r);
                }
                this.plt.add(p);

                line = br.readLine();
            }
            fr.close();
            br.close();
        } catch (FileNotFoundException fe) {
            throw new BPELEngineException("FileNotFoundException PLType PLType : \n" , fe);
        } catch (IOException e) {
            throw new BPELEngineException("IOException PLType PLType :\n" , e);
        }
    }

    /**
     * @param partnerLinkTypeName the partneLinkType name
     * @param roleName the role name
     * @return Vector a vector containing the portType associated to this
     *         partnerLinkType and this role and the endPoint associated to this
     *         portType
     */

    public QName getPortType(QName partnerLinkTypeName, String roleName) {
        QName result = null;
        PartnerLinkType p = null;
        Role r = null;
        Vector roles = null;
        root:
            for (int i = 0; i < this.plt.size(); i++) {
                p = ((PartnerLinkType) this.plt.get(i));
                if (p.getName().equals(partnerLinkTypeName)) {
                    roles = p.getRoles();
                    for (int j = 0; j < roles.size(); j++) {
                        r = (Role) roles.get(j);
                        if (r.getName().equals(roleName)) {
                            result = r.getPortType();
                            break root;
                        }
                    }
                }
            }
        return result;
    }

}