/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELConstants;



/**
 * ProcessMap.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ProcessMap.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class ProcessMap {

    private static String myPath = BPELConstants.PROCESSMAP_DIR;
    private static Vector processMapReceives = null;
    //private static Vector processLoaded = null;

    /**
     * @param processName
     *            the Name of Process
     * @throws FileNotFoundException
     * @throws IOException
     * @throws BPELEngineException
     */
    public static void loadProcess(String processName) throws BPELEngineException {
        if (myPath.startsWith("null")) { throw new BPELEngineException("ProcessMap: "+BPELConstants.BPEL_HOME_PROPERTY+" property is not set",null); }
        try {
            if (processMapReceives == null) {
                processMapReceives = new Vector();
                //processLoaded = new Vector();
                File f = new File(myPath);
                File[] processMapList = f.listFiles();
                for (int i = 0; i < processMapList.length; i++) {
                    FileReader fr = new FileReader(processMapList[i]);
                    BufferedReader br = new BufferedReader(fr);
                    String line = br.readLine();
                    while (line != null) {
                        String[] elements = line.split(" ");
                        ProcessMapReceive pmr = new ProcessMapReceive(elements);
                        //pmr.print();
                        processMapReceives.add(pmr);
                        //processLoaded.add(pmr.getProcessName());
                        line = br.readLine();
                    }
                    fr.close();
                    br.close();
                }
            } else if (!isLoaded(processName)) {
                File f = new File(myPath + File.separator + processName + ".txt");
                FileReader fr = new FileReader(f);
                BufferedReader br = new BufferedReader(fr);
                String line = br.readLine();
                while (line != null) {
                    //System.out.println("Line2 : " + line);
                    String[] elements = line.split(" ");
                    ProcessMapReceive pmr = new ProcessMapReceive(elements);
                    //pmr.print();
                    processMapReceives.add(pmr);
                    //processLoaded.add(pmr.getProcessName());
                    line = br.readLine();
                }
                fr.close();
                br.close();
            }
        } catch (FileNotFoundException fe) {
            throw new BPELEngineException("FileNotFoundException ProcessMap loadProcess : \n" , fe);
        } catch (IOException e) {
            throw new BPELEngineException("IOException ProcessMap loadProcess :\n" , e);
        }
        //print();
    }

    /**
     * @param processName
     *            the Name of the Process
     * @return
     */
    private static boolean isLoaded(String processName) {
        boolean found = false;
        if (processMapReceives != null) {
            synchronized(processMapReceives) {
                for (int i = 0; i < processMapReceives.size(); i++) {
                    ProcessMapReceive pmr = (ProcessMapReceive) processMapReceives.get(i);
                    if (pmr.compare(processName)) {
                        found = true;
                        break;
                    }
                }
            }
        }
        return found;
    }

    public static void removeLoadedProcess(String processName) {
        Vector tmp = new Vector();
        if (processMapReceives != null) {
            synchronized(processMapReceives) {
                for (int i = 0; i < processMapReceives.size(); i++) {
                    ProcessMapReceive pmr = (ProcessMapReceive) processMapReceives.get(i);
                    if (!pmr.compare(processName)) {
                        tmp.addElement(pmr);
                    }
                }
                processMapReceives = tmp;
            }
        }

    }
    /**
     * @param process
     *            the Process
     * @param portType
     *            the Port Type
     * @param operation
     *            the Operation
     * @return Vector of ProcessMap Receives
     */
    public static Vector getPMR(String process, String portType, String operation) {
        Vector result = new Vector();
        for (int i = 0; i < processMapReceives.size(); i++) {
            if (((ProcessMapReceive) processMapReceives.get(i)).compare(process, portType, operation)) {
                result.add(processMapReceives.get(i));
            }
        }
        return result;
    }

    /**
     * @param process
     * @return
     */
    public static Vector getPMR(String process) {

        Vector result = new Vector();
        for (int i = 0; i < processMapReceives.size(); i++) {
            if (((ProcessMapReceive) processMapReceives.get(i)).compare(process)) {
                result.add(processMapReceives.get(i));
            }
        }
        return result;
    }
}