/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import org.objectweb.orchestra.util.BPELTrace;

/**
 * ProcessMapReceive.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ProcessMapReceive.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class ProcessMapReceive {

    private String type = null;
    private String processName = null;
    private String portType = null;
    private String operation = null;
    private String partner = null;
    private String createInstance = null;
    private String correlationSet = null;
    private String[] correlationSetProperties = null;
    private String name = null;

    /**
     * @param properties
     * @throws Exception
     */
    public ProcessMapReceive(String[] properties) {
        this.type = properties[0];
        this.processName = properties[1];
        this.partner = properties[2];
        this.portType = properties[3];
        this.operation = properties[4];
        this.createInstance = properties[5];
        this.name = properties[6];
        if (this.name.equals("null")) {
            this.name = null;
        }
        if (properties.length > 7) { // With CS
            if (BPELTrace.isEngineDebugEnabled())
                BPELTrace.traceEngine("ProcessMapReceive new : properties.length : " + properties.length);
            this.correlationSet = properties[7];
            this.correlationSetProperties = new String[properties.length - 8];
            for (int i = 0; i < this.correlationSetProperties.length; i++)
                this.correlationSetProperties[i] = properties[8 + i];
        }
    }

    /**
     * @return an array of CS Properties
     */
    public String[] getCSProperties() {
        return this.correlationSetProperties;
    }

    /**
     * @return the Correlation Set
     */
    public String getCS() {
        return this.correlationSet;
    }

    /**
     * @return the Type
     */
    public String getType() {
        return this.type;
    }

    /**
     * @return the Partner
     */
    public String getPartner() {
        return this.partner;
    }

    /**
     * @return createInstance
     */
    public String getCreateInstance() {
        return this.createInstance;
    }

    /**
     * @return the process Name
     */
    public String getProcessName() {
        return this.processName;
    }

    /**
     * @return the portType
     */
    public String getPortType() {
        return this.portType;
    }

    /**
     * @return the operation
     */
    public String getOperation() {
        return this.operation;
    }

    /**
     * @return the name
     */
    public String getName() {
        return this.name;
    }

    /**
     * @param proc
     * @param port
     * @param op
     * @return
     */
    public boolean compare(String proc, String port, String op) {
        return ((this.processName.equals(proc)) && (this.portType.equals(port)) && (this.operation.equals(op)));
    }

    /**
     * @param proc
     * @return
     */
    public boolean compare(String proc) {
        return this.processName.equals(proc);
    }

    /**
     * Print ProcessMapReceive
     */
    public void print() {
        System.out.println("ProcessMapReceive :");
        System.out.println("   type : " + this.type);
        System.out.println("   processName : " + this.processName);
        System.out.println("   portType : " + this.portType);
        System.out.println("   operation : " + this.operation);
        System.out.println("   partner : " + this.partner);
        System.out.println("   createInstance : " + this.createInstance);
        System.out.println("   correlationSet : " + this.correlationSet);
        System.out.println("   correlationSetProperties : " + this.correlationSetProperties);
    }
}