/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import javax.xml.namespace.QName;

import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;



/**
 * WsdlBindingElement.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: WsdlBindingElement.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class WsdlBindingElement implements java.io.Serializable {
    private String processName = null;
    private String serviceName = null;
    private String portName = null;
    private String binding = null;
    private String addressLocation = null;
    private String bindingType = null;
    private String portTypeName = null;
    private String operationName = null;
    private String inputMessageName = null;
    private String outputMessageName = null;
    private FaultInfo[] faults = null;
    private int faultNb = 0;

    /**
     * Default Constructor
     *
     * @param properties
     * @throws Exception
     */
    public WsdlBindingElement(String[] properties) {
        String tmp = properties[0].substring(0, properties[0].indexOf("="));
        if (tmp.equals("processName")) {
            //service information
            this.processName = properties[0].substring(properties[0].indexOf("=") + 1);
            this.serviceName = properties[1].substring(properties[1].indexOf("=") + 1);
            this.portName = properties[2].substring(properties[2].indexOf("=") + 1);
            this.binding = properties[3].substring(properties[3].indexOf("=") + 1);
            this.addressLocation = properties[4].substring(properties[4].indexOf("=") + 1);
            this.bindingType = properties[5].substring(properties[5].indexOf("=") + 1);
        } else {
            //operation information
            this.portTypeName = properties[0].substring(properties[0].indexOf("=") + 1);
            this.operationName = properties[1].substring(properties[1].indexOf("=") + 1);
            this.inputMessageName = properties[2].substring(properties[2].indexOf("=") + 1);
            this.outputMessageName = properties[3].substring(properties[3].indexOf("=") + 1);
            String faultsInfo = properties[4].substring(properties[4].indexOf("=") + 1);
            if (!faultsInfo.equals("")) {
                //System.out.println("WSDLBINDINGELEMENT : FaultsInfo : " + faultsInfo);
                String[] faultsInfoArray = faultsInfo.split(BPELConstants.FAULT_SEPARATOR);
                this.faultNb = faultsInfoArray.length;
                this.faults = new FaultInfo[this.faultNb];
                for (int i=0 ; i < this.faultNb ; i++) {
                    String[] faultInfo = faultsInfoArray[i].split(BPELConstants.FAULTINFO_SEPARATOR);
                    this.faults[i] = new FaultInfo(faultInfo[0], faultInfo[1]);
                }
            }
        }
    }

    public String getFaultName(String faultMessage) {
        String faultName = null;
        if (this.faults != null) {
            for (int i=0 ;i < this.faultNb ; i++) {
                /*String fmNS = faults[i].getFaultMessage();
                System.out.println("la : fmNS : " + fmNS);
                System.out.println("la : BPELString.getNS(fmNS) : " + BPELString.getNS(fmNS));
                System.out.println("la : BPELString.removeNS(fmNS); : " + BPELString.removeNS(fmNS));
                System.out.println("la : Util.getPackageFromNamespace(BPELString.getNS(fmNS)) : " + Util.getPackageFromNamespace(BPELString.getNS(fmNS)));
                String fm = Util.getPackageFromNamespace(BPELString.getNS(fmNS)) + "." + BPELString.removeNS(fmNS);
                System.out.println("la : fm : " + fm);*/
                String fm = this.faults[i].getFaultMessagePackage();
                if (faultMessage.equals(fm)) {
                    faultName = this.faults[i].getFaultName();
                    break;
                }
            }
        }
        return faultName;
    }

    public String getProcessName() {
        return this.processName;
    }

    /**
     * @return
     */
    public String getServiceName() {
        return this.serviceName;
    }

    /**
     * @return
     */
    public String getPortName() {
        return this.portName;
    }

    /**
     * @return
     */
    public String getBinding() {
        return this.binding;
    }

    /**
     * @return
     */
    public String getAddressLocation() {
        return this.addressLocation;
    }

    /**
     * @return
     */
    public String getBindingType() {
        return this.bindingType;
    }

    /**
     * @return
     */
    public String getPortTypeName() {
        return this.portTypeName;
    }

    /**
     * @return
     */
    public String getOperationName() {
        return this.operationName;
    }

    /**
     * @return
     */
    public String getInputMessageName() {
        return this.inputMessageName;
    }

    /**
     * @return
     */
    public String getOutputMessageName() {
        return this.outputMessageName;
    }

    public FaultInfo[] getFaults() {
        return this.faults;
    }

    /**
     * @return
     */
    public boolean compare(String processName0, QName serviceName0, String portName0, QName binding0, String addressLocation0, QName bindingType0) {
        String s0 = processName0;
        String s1 = Util.buildStringFromQName(serviceName0);
        String s2 = portName0;
        String s3 = Util.buildStringFromQName(binding0);
        String s4 = addressLocation0;
        String s5 = Util.buildStringFromQName(bindingType0);

        return s0.equals(this.processName) && s1.equals(this.serviceName) && s2.equals(this.portName) && s3.equals(this.binding) && s4.equals(this.addressLocation) && s5.equals(this.bindingType);
    }

    public String toString() {
        String st = "";
        st += "processName = "+this.processName+"\n";
        st += "serviceName = "+this.serviceName+"\n";
        st += "portName = "+this.portName+"\n";
        st += "binding = "+this.binding+"\n";
        st += "addressLocation = "+this.addressLocation+"\n";
        st += "bindingType = "+this.bindingType+"\n";

        return st;
    }
}