/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.util.List;

import org.apache.commons.jxpath.ExpressionContext;
import org.jaxen.Context;
import org.jaxen.Function;
import org.jaxen.FunctionCallException;
import org.jaxen.UnresolvableException;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * XPathExtensionGetLinkStatus.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: XPathExtensionGetLinkStatus.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class XPathExtensionGetLinkStatus implements Function {

    public Object call(Context context, List args) throws FunctionCallException {
        Object result = null;
        if (args.size()!=1) {
            throw new FunctionCallException("There is not exactly 1 argument specified for the function : getLinkStatus");
        }
        String linkName = (String)args.get(0);
        try {
            PendingTarget pt = (PendingTarget)context.getVariableValue(BPELConstants.PRODUCTNAME,null,"pendingTarget");
            result = getLinkStatus(linkName,pt);
        } catch (UnresolvableException e) {
            throw new FunctionCallException("UnresolvableException GetVariableDataExtensionFuntion call : " , e);
        } catch (BPELEngineException e) {
            throw new FunctionCallException("BPELUtilException GetVariableDataExtensionFuntion call : " , e);
        }
        return result;
    }

    public Object getLinkStatus(ExpressionContext context, String linkName) throws BPELEngineException {
        PendingTarget pt = (PendingTarget)context.getJXPathContext().getVariables().getVariable("pendingTarget");
        return getLinkStatus(linkName,pt);
    }

    public Object getLinkStatus(String linkName, PendingTarget pt) throws BPELEngineException {
        Object result = null;
        try {
            result = pt.getTransitionCondition(linkName);
        } catch (BPELEngineException e) {
            throw new BPELEngineException ("BPELEngineException caught in XPathExtensionGetLinkStatus : ",e);
        }
        return result;
    }
}


