/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.util.List;

import org.apache.commons.jxpath.ExpressionContext;
import org.jaxen.Context;
import org.jaxen.Function;
import org.jaxen.FunctionCallException;
import org.jaxen.UnresolvableException;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;

import org.objectweb.orchestra.engine.EngineHomeLocal;
import org.objectweb.orchestra.engine.EngineLocal;
import org.objectweb.orchestra.engine.EngineUtil;

/**
 * XPathExtensionGetVariableData.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: XPathExtensionGetVariableData.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class XPathExtensionGetVariableData implements Function {

    public Object call(Context context, List args) throws FunctionCallException {
        Object result = null;
        if (args.isEmpty()) {
            throw new FunctionCallException("No arg specified for the function : getVariableData");
        }
        if (args.size()>3) {
            throw new FunctionCallException("More than 3 args specified for the function : getVariableData");
        }
        String variable = (String) args.get(0);
        String part = null;
        String query = null;
        if (args.size() >= 2) {
            part = (String) args.get(1);
        }
        if (args.size() >= 3) {
            query = (String) args.get(2);
        }
        try {
            Integer tmp = (Integer) context.getVariableValue(BPELConstants.PRODUCTNAME,null,"bpiId");
            int bpiId = tmp.intValue();
            tmp = (Integer) context.getVariableValue(BPELConstants.PRODUCTNAME,null,"scopeId");
            int scopeId = tmp.intValue();
            String processName = (String)context.getVariableValue(BPELConstants.PRODUCTNAME,null,"processName");
            result = getVariableData(variable, part, query, bpiId, scopeId, processName);
        } catch (UnresolvableException e) {
            throw new FunctionCallException("UnresolvableException GetVariableDataExtensionFuntion call : " , e);
        } catch (BPELEngineException e) {
            throw new FunctionCallException("BPELUtilException GetVariableDataExtensionFuntion call : " , e);
        }
        return result;
    }

    public Object getVariableData(ExpressionContext context, String variable) throws BPELEngineException {
        return getVariableData(context,variable,null,null);
    }

    public Object getVariableData(ExpressionContext context, String variable, String part) throws BPELEngineException {
        return getVariableData(context,variable,part,null);
    }

    public Object getVariableData(ExpressionContext context, String variable, String part, String query) throws BPELEngineException {
        int bpiId = ((Integer)context.getJXPathContext().getVariables().getVariable("bpiId")).intValue();
        int scopeId = ((Integer)context.getJXPathContext().getVariables().getVariable("scopeId")).intValue();
        String processName = (String)context.getJXPathContext().getVariables().getVariable("processName");
        return getVariableData(variable,part,query,bpiId, scopeId, processName);
    }


    public Object getVariableData(String variable, String part, String query, int bpiId, int scopeId, String processName) throws BPELEngineException {
        Object result = null;
        try {
            EngineHomeLocal ehl = EngineUtil.getLocalHome();
            EngineLocal engine = ehl.create();

            if (part == null) {
                result = engine.getVariableData(bpiId, scopeId, variable,processName);
            } else if (query == null) {
                result = engine.getVariableData(bpiId, scopeId, variable, part,processName);
            } else {
                result = engine.getVariableData(bpiId, scopeId, variable, part, query,processName);
            }
        } catch (javax.naming.NamingException e) {
            throw new BPELEngineException("NamingException GetVariableDataExtensionFuntion call : " , e);
        } catch (javax.ejb.CreateException e) {
            throw new BPELEngineException("CreateException GetVariableDataExtensionFuntion call : " , e);
        } catch (BPELEngineException e) {
            throw new BPELEngineException("BPELEngineException GetVariableDataExtensionFuntion call : " , e);
        }
        result = Util.getNumberAsDouble(result);
        return result;
    }
}


