/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;
import java.io.FileWriter;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.WSDLJavaMatching;



/**
 * PartnerLinkTypeFile.java, writes a file giving the mapping between a property
 * name and its value (declared in wsdl file)
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ComplexTypesFile.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class ComplexTypesFile {

    private static String myPath = BPELConstants.COMPLEXTYPES_DIR;
    private FileWriter fileWriter = null;
    private String fileName = null;
    private String processName = null;
    private String f = null;

    /**
     * constructor
     *
     * @param process
     *            the name of the process
     */
    public ComplexTypesFile(String process) throws BPELGeneratorException {

        this.processName = process;
        this.fileName = this.processName;

        try {
            this.f = myPath + File.separator + this.fileName + ".txt";
            //System.out.println("File : " + f);
            new File(this.f).getAbsoluteFile().getParentFile().mkdirs();
            this.fileWriter = new FileWriter(this.f);
        } catch (Exception e) {
            throw new BPELGeneratorException("ComplexTypesFile.init : ",e);
        }

    }

    /**
     * insert a line in the file
     *
     * @param tns
     *            the schema targetnamespace
     * @param complexTypeName
     *            the complex type name
     */
    public void insertComplexType(String xmlType, String javaType) throws BPELGeneratorException {
        try {
            String holderType = WSDLJavaMatching.getHolderType(javaType);
            String st = xmlType + " " + javaType + " " + holderType + "\n";
            this.fileWriter.write(st);
        } catch (Exception e) {
            throw new BPELGeneratorException("ComplexTypesFile.init : ",e);
        }
    }

    /**
     * write the last instructions of the file
     *
     */
    public void finish() throws BPELGeneratorException {
        try {
            this.fileWriter.close();
        } catch (Exception e) {
            throw new BPELGeneratorException("ComplexTypesFile.init : ",e);
        }
        System.out.println("File " + this.f + " created");
    }
}