/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

/**
 * ElementType.java, represents a wsdl element tag
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ElementType.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class ElementType {

    private String mElementName = null;
    private String mType = null;

    /**
     * Constructor
     *
     * @param name
     *            the name of this element, for example : urn:ns,el (urn:ns is
     *            the namespace of this element tag)
     * @param type
     *            the type of this element, for example, urn:ns,elType(urn:ns is
     *            the namespace of this element type)
     */
    public ElementType(String name, String type) {
        //for example
        //xmlns:ns="urn:ns"
        //<ns:element name="el" type="ns:elType"/>

        //mElementName="urn:ns,el"
        this.mElementName = name;
        //mType="urn:ns,elType"
        this.mType = type;
    }

    /**
     * get the elementName of this ElementType
     *
     * @return String the elementName, for example : urn:ns,el (urn:ns is the
     *         namespace of this element tag)
     */
    public String getElementName() {
        return this.mElementName;
    }

    /**
     * get the type of this ElementType
     *
     * @return String the type, for example : urn:ns,elType (urn:ns is the
     *         namespace of this element type)
     */
    public String getType() {
        return this.mType;
    }
}