/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import javax.xml.namespace.QName;

/**
 * MessagePart.java, represents a part of a message
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: MessagePart.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class MessagePart {

    private String mPartName = null;
    private String mPartType = null;
    private QName mPartElement = null;

    /**
     * Constructor
     *
     * @param name
     *            the name of the part
     * @param type
     *            the type of part : for example : urn:ns:p1,partType
     * @param element
     *            the element of the part : for example : urn:ns:p1,partElement
     */
    public MessagePart(String name, String type, QName element) {
        this.mPartName = name;
        this.mPartType = type;
        this.mPartElement = element;
    }

    /**
     * print this part
     *
     */
    public String toString() {
        return "Message Part, name=" + this.mPartName + ", type=" + this.mPartType + ", element=" + this.mPartElement;
    }

    /**
     * print this part
     *
     */
    public void printMessagePart() {
        System.out.println("  Print Message Part");
        System.out.println("    Message Part name : " + this.mPartName);
        if (this.mPartType != null) {
            System.out.println("    Message Part type : " + this.mPartType);
        } else {
            System.out.println("    Message Part element : " + this.mPartElement);
        }

    }

    /**
     * get the part name of this MessagePart
     *
     * @return String the part name
     */
    public String getPartName() {
        return this.mPartName;
    }

    /**
     * get the part type of this MessagePart
     *
     * @return String the part type
     */
    public String getPartType() {
        return this.mPartType;
    }

    /**
     * get the part element of this MessagePart
     *
     * @return String the part element
     */
    public QName getPartElement() {
        return this.mPartElement;
    }

}