/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

/**
 * NamespaceDef.java, represents a namespace definition : in definitions tag of
 * a wsdl file or in process tag of a bpel file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: NamespaceDef.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public abstract class NamespaceDef {

    protected String processName = null;

    /**
     * get a namespace uri based on a prefix
     *
     * @param alias
     *            the alias (prefix) of the namespace searched
     * @return String the namespaceURI corresponding to this alias
     */
    public abstract String getNS(String alias);

    /**
     * get the targetNamespace of this NamespaceDef
     *
     * @return String the namespaceURI corresponding to the targetNamespace
     */
    public abstract String getTargetNamespace();

    /**
     * get the name of this definitions tag (wsdl) or the name of the process
     * (bpel)
     *
     * @return String the name of this NamespaceDef
     */
    public abstract String getName();

    /**
     * get the name of the process
     *
     * @return String the name of the process
     */
    public String getProcessName() {
        return this.processName;
    }
}