/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import org.objectweb.orchestra.util.BPELString;

/**
 * PropertyAlias.java, represents a propertyAlias tag in wsdl file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: PropertyAlias.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class PropertyAlias {

    private String mPropertyName = null;
    private String mMessageType = null;
    private String mPart = null;
    private String mQuery = null;

    /**
     * constructor
     *
     * @param name
     *            the name of the property of this propertyAlias
     * @param type
     *            the type of the message of this propertyAlias
     * @param part
     *            the part of the message of this propertyAlias
     * @param query
     *            the query of the message of this propertyAlias
     */
    public PropertyAlias(String name, String type, String part, String query) {
        this.mPropertyName = BPELString.removePrefixUpper(name);
        this.mMessageType = BPELString.removePrefixUpper(type);
        this.mPart = part;
        this.mQuery = query;
    }

    /**
     * print a this propertyAlias
     */
    public void printPropertyAlias() {
        System.out.println("Print Property Alias");
        System.out.println("  Property Alias name : " + this.mPropertyName);
        System.out.println("  Property Alias Message Type : " + this.mMessageType);
        System.out.println("  Property Alias Part : " + this.mPart);
        System.out.println("  Property Alias Query : " + this.mQuery);
    }

    /**
     * get the property name of this propertyAlias
     */
    public String getPropertyName() {
        return this.mPropertyName;
    }

    /**
     * get the message type of this propertyAlias
     */
    public String getMessageType() {
        return this.mMessageType;
    }

    /**
     * get the part of the message of this propertyAlias
     */
    public String getPart() {
        return this.mPart;
    }

    /**
     * get the query of the message of this propertyAlias
     */
    public String getQuery() {
        return this.mQuery;
    }

}