package org.objectweb.orchestra.generator;


import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Entity resolver allowing to find schema within the classloader.
 * @author Florent Benoit
 */
public class SchemaEntityResolver implements EntityResolver {

    /**
     * Map where the schemas URLs are stored.
     */
    private Map schemasUrls = null;

    /**
     * Constructor. Finds the XSD with classloader.
     * @param schemas the name of the schemas to resolve.
     */
    public SchemaEntityResolver(final String[] schemas, final String[] ns) {
        schemasUrls = new HashMap();
        URL url = null;
        for (int i = 0; i < schemas.length; i++) {
            url = SchemaEntityResolver.class.getResource("/" + schemas[i]);
            if (url == null) {
                throw new IllegalStateException("'" + schemas[i] + "' was not found in the current classloader !");
            }
            String urlString = url.toString();
            String id = ns[i];
            schemasUrls.put(id, urlString);

        }
    }

    /**
     * The Parser will call this method before opening any external entity
     * except the top-level document entity.
     * @param publicId The public identifier of the external entity being
     *        referenced, or null if none was supplied.
     * @param systemId The system identifier of the external entity being
     *        referenced.
     * @return An InputSource object describing the new input source, or null to
     *         request that the parser open a regular URI connection to the
     *         system identifier.
     * @throws SAXException Any SAX exception, possibly wrapping another
     *         exception.
     * @throws IOException A Java-specific IO exception, possibly the result of
     *         creating a new InputStream or Reader for the InputSource.
     */
    public InputSource resolveEntity(final String publicId, final String systemId) throws IOException, SAXException {

        String localPath = null;

        if (systemId != null) {
            // Registred ?
            localPath = (String) schemasUrls.get(systemId);

        }

        // schema not found
        if (localPath == null) {
            throw new SAXException("No XSD found for '" + systemId + "'.");
        }

        // Return the local path source
        return (new InputSource(localPath));
    }

}



