/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELString;
import org.objectweb.orchestra.util.Util;



/**
 * StubServiceFile.java, generates a stub identical to the one generated by
 * wsdl2java but add some methods ans an easy way to get output params of a call
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: StubServiceFile.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class StubServiceFile {

    private static String myPathOut = BPELConstants.GENERATOR_DIR;

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------
    private String directoryName = null;
    private BufferedReader br = null;
    private BufferedWriter bw = null;
    private String stout = null;

    /**
     * constructor
     *
     * @param process
     *            the name of the process
     * @param w
     *            the wsdl object giving mapping between namespaces and prefixes
     */
    public StubServiceFile(String process, String namespace, String bindingName, String myPathIn) throws BPELGeneratorException {
        this.directoryName = process;
        try {
            String className = BPELString.toUpper(bindingName) + "Stub.java";
            String packag = Util.getPackageFromNamespace(namespace);
            String dir = Util.getDirectoryFromPackage(packag);
            String temp = myPathIn + File.separator + this.directoryName + File.separator + dir + File.separator;
            File f = new File(temp);
            File[] list = f.listFiles();
            for (int i = 0; i < list.length; i++) {
                String tmp = list[i].getName();
                if (tmp.equals(className)) {
                    String stin = temp + className;
                    //System.out.println("Executing StubServiceFile on file " + stin);
                    File fin = new File(stin);
                    FileReader fr = new FileReader(fin);
                    this.stout = myPathOut + File.separator + this.directoryName + File.separator + dir + File.separator + className;
                    File fout = new File(this.stout);
                    fout.getAbsoluteFile().getParentFile().mkdirs();
                    fout.createNewFile();
                    FileWriter fw = new FileWriter(fout);
                    this.br = new BufferedReader(fr);
                    this.bw = new BufferedWriter(fw);
                    String st = null;
                    while ((st = this.br.readLine()) != null) {
                        if (st.indexOf("protected org.apache.axis.client.Call createCall()") != -1) {
                            this.br.reset();
                            insertOperations();
                            st = this.br.readLine();
                        } else if (st.indexOf("static org.apache.axis.description.OperationDesc [] _operations;") != -1) {
                            this.br.reset();
                            this.bw.write("    private java.util.Vector calls = new java.util.Vector();");
                            this.bw.newLine();
                            st = this.br.readLine();
                        } else if (st.indexOf("extractAttachments(_call);") != -1) {
                            this.br.reset();
                            this.bw.write("          calls.addElement(_call.getOperationName());");
                            this.bw.newLine();
                            this.bw.write("          calls.addElement(_call);");
                            this.bw.newLine();
                            st = this.br.readLine();
                        } else if (st.indexOf("_call.setOperation(_operations") != -1) {
                            this.br.reset();
                            this.bw.write("        if (super.cachedTimeout == null) {");
                            this.bw.newLine();
                            this.bw.write("          _call.setTimeout(new Integer(0));");
                            this.bw.newLine();
                            this.bw.write("        }");
                            this.bw.newLine();
                            st = this.br.readLine();
                        }


                        this.br.mark(1000);
                        this.bw.write(st);
                        this.bw.newLine();
                    }
                    finish();
                }
            }
        } catch (Exception e) {
            throw new BPELGeneratorException("StubServiceFile.init : ",e);
        }
    }

    /**
     * insert setEndpoint(java.net.URL endpointURL), getEndpoint(),
     * getService(), getPort() and getMyCalls operations
     *
     */
    public void insertOperations() throws Exception {
        this.bw.write("    public void setEndpoint(java.net.URL endpointURL) {");
        this.bw.newLine();
        this.bw.write("        super.cachedEndpoint = endpointURL;");
        this.bw.newLine();
        this.bw.write("    }");
        this.bw.newLine();
        this.bw.newLine();

        this.bw.write("    public java.net.URL getEndpoint() {");
        this.bw.newLine();
        this.bw.write("        return super.cachedEndpoint;");
        this.bw.newLine();
        this.bw.write("    }");
        this.bw.newLine();
        this.bw.newLine();

        this.bw.write("    public org.apache.axis.client.Service getService() {");
        this.bw.newLine();
        this.bw.write("        return (org.apache.axis.client.Service) super.service;");
        this.bw.newLine();
        this.bw.write("    }");
        this.bw.newLine();
        this.bw.newLine();

        this.bw.write("    public javax.xml.namespace.QName getPort() {");
        this.bw.newLine();
        this.bw.write("        return super.cachedPortName;");
        this.bw.newLine();
        this.bw.write("    }");
        this.bw.newLine();
        this.bw.newLine();

        this.bw.write("    public java.util.Vector getMyCalls() {");
        this.bw.newLine();
        this.bw.write("        return calls;");
        this.bw.newLine();
        this.bw.write("    }");
        this.bw.newLine();
        this.bw.newLine();
    }

    /**
     * write the last instructions of the file
     *
     */
    public void finish() throws BPELGeneratorException {
        try {
            //fr.close();
            //fw.close();
            this.br.close();
            this.bw.close();
        } catch (Exception e) {
            throw new BPELGeneratorException("StubServiceFile.finish : ",e);
        }
        System.out.println("File " + this.stout + " created");
    }

}