/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;



/**
 * WSDDFile.java, generates a stub identical to the one generated by wsdl2java
 * but add some methods ans an easy way to get output params of a call
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: WSDDFile.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class WSDDFile {

    private static String myPathIn = BPELConstants.WSDL_DIR;

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------

    private String processName = null;
    private BufferedReader br = null;
    private BufferedWriter bw = null;
    private String fName = null;

    /**
     * constructor
     *
     * @param process
     *            the name of the process
     * @param w
     *            the wsdl object giving mapping between namespaces and prefixes
     */
    public WSDDFile(String process, org.objectweb.orchestra.generator.Wsdl w, Vector ports_wsdl) throws BPELGeneratorException {
        this.processName = process;
        try {
            QName qn = Util.buildQname("", w);
            String namespace = qn.getNamespaceURI();
            String packag = Util.getPackageFromNamespace(namespace);
            String dir = myPathIn + File.separator + process + File.separator + Util.getDirectoryFromPackage(packag) + File.separator;
            File f = new File(dir + "deploy.wsdd");
            System.out.println("Executing WSDDFile on file " + f);
            this.fName = f.getAbsolutePath();
            FileReader fr = new FileReader(f);
            String stout = dir + "deployTMP.wsdd";
            File fout = new File(stout);
            fout.getAbsoluteFile().getParentFile().mkdirs();
            fout.createNewFile();
            FileWriter fw = new FileWriter(fout);
            this.br = new BufferedReader(fr);
            this.bw = new BufferedWriter(fw);
            String st = null;
            String wsdlLocation = null;
            boolean paramInserted = false;
            while ((st = this.br.readLine()) != null) {
                if (st.indexOf("<service") != -1) {
                    this.br.reset();
                    int index1 = st.indexOf("name");
                    String tmp = st.substring(index1);
                    index1 = tmp.indexOf("\"") + 1;
                    int index2 = tmp.indexOf("\"", index1);
                    String name = tmp.substring(index1, index2);
                    wsdlLocation = getLoc(ports_wsdl, name);
                    replaceProvider(st);
                    this.br.readLine();
                    st = this.br.readLine();
                } else if (st.indexOf("</service>") != -1) {
                    this.br.reset();
                    insertWsdlFile(wsdlLocation);
                    st = this.br.readLine();
                    paramInserted = false;
                } else if (!paramInserted && st.indexOf("<parameter") != -1) {
                    this.br.reset();
                    insertParameter();
                    paramInserted = true;
                    st = this.br.readLine();
                }
                this.br.mark(1000);
                this.bw.write(st);
                this.bw.newLine();

            }
            finish(f, fout);
        } catch (Exception e) {
            throw new BPELGeneratorException("WSDDFile.init : ",e);
        }
    }

    /**
     * insert wsdlFile url
     *
     */
    public void insertWsdlFile(String wsdlLocation) throws Exception {
        /*
        int prefSize = wsdlLocation.indexOf(':')+1;
        if (System.getProperty("os.name").toLowerCase().indexOf("windows")!=-1) {
            prefSize++;
        }
         */
        String prefix = Util.getFilePrefix();
        if (wsdlLocation.startsWith(prefix)) {
            wsdlLocation = wsdlLocation.substring(prefix.length());
        }
        if (System.getProperty("os.name").toLowerCase().indexOf("windows") != -1) {
            wsdlLocation = removeProtocol(wsdlLocation);
        }
        this.bw.write("      <wsdlFile>" + wsdlLocation + "</wsdlFile>");
        this.bw.newLine();
    }

    private String removeProtocol(String wsdlLocation) {
        String result = new String(wsdlLocation);
        if (result.startsWith("file:")) {
            result = result.substring(5);
            while(result.charAt(0) == '\\') {
                result = result.substring(1);
            }
        }
        return result;
    }
    /**
     * insert bpelProcessName parameter
     *
     */
    public void insertParameter() throws Exception {
        this.bw.write("      <parameter name=\"bpelProcessName\" value=\""+this.processName+"\"/>");
        this.bw.newLine();
    }

    /**
     * insert wsdlFile url
     *
     */
    public String getLoc(Vector v, String name) throws Exception {
        String result = null;
        for (int i = 0; i < v.size(); i++) {
            PortWsdl pw = (PortWsdl) v.get(i);
            if (pw.getPortName().equals(name)) {
                result = pw.getWsdlLocation();
                v.removeElement(pw);
                break;
            }
        }
        return result;
    }

    public void replaceProvider(String st) throws Exception {
        int beforeProvider = st.indexOf("provider");
        String tmp = st.substring(beforeProvider);
        tmp = tmp.substring(tmp.indexOf("\"")+1);
        String end = tmp.substring(tmp.indexOf("\"")+1);
        String start = st.substring(0,beforeProvider);
        String provider = "provider=\"java:BPEL\"";
        String result = start + provider + end;
        this.bw.write(result);
        this.bw.newLine();
    }

    /**
     * write the last instructions of the file
     *
     */
    public void finish(File f, File fout) throws BPELGeneratorException {
        try {
            //fr.close();
            //fw.close();
            this.br.close();
            this.bw.close();
            String name = f.getAbsolutePath();
            f.delete();
            fout.renameTo(new File(name));
        } catch (Exception e) {
            throw new BPELGeneratorException("WSDDFile.finish : ",e);
        }
        System.out.println("File " + this.fName + " created");
    }

}