/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Input;
import javax.wsdl.Message;
import javax.wsdl.Output;
import javax.wsdl.PortType;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.axis.utils.WSDLUtils;
import org.apache.axis.wsdl.symbolTable.SymbolTable;
import org.apache.axis.wsdl.symbolTable.TypeEntry;
import org.apache.axis.wsdl.toJava.Emitter;
import org.apache.axis.wsdl.toJava.Utils;
import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELString;
import org.objectweb.orchestra.util.UDDIDirectory;
import org.objectweb.orchestra.util.Util;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.ibm.wsdl.ImportImpl;
import com.ibm.wsdl.MessageImpl;
import com.ibm.wsdl.OperationImpl;
import com.ibm.wsdl.PartImpl;
import com.ibm.wsdl.PortImpl;
import com.ibm.wsdl.ServiceImpl;
import com.ibm.wsdl.factory.WSDLFactoryImpl;

/**
 * WSDLCompilo.java, generates all needed files based on the wsdl file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: WSDLCompilo.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class WSDLCompilo {

    // -------------------------------------------------------------------------
    // Variables
    // -------------------------------------------------------------------------
    private String processName = null;
    private WebServiceFile f = null;
    private Document document = null;
    private Vector messageTypes = null;
    private PartnerLinkTypeFile partnerLinkTypeFile = null;
    private ComplexTypesFile complexTypesFile = null;
    private Wsdl wsdl = null;
    private Definition def = null;
    private DocumentBuilder docBuilder = null;
    private int wsdlFilesNb = 0;
    private Vector propertiesNS = null;
    private Map plTypes = null;
    private Vector portsWsdl = null;
    private Vector messExamined = null;
    private Vector serviceExamined = null;
    private Vector bindingExamined = null;
    private String processTargetNS = null;
    private SymbolTable symbolTable = null;

    /**
     * constructor
     * @param process the process name
     * @throws ParserConfigurationException
     */
    public WSDLCompilo(String process) throws BPELGeneratorException  {
        this.processName = process;
        this.serviceExamined = new Vector();
        this.bindingExamined = new Vector();
        this.messExamined = new Vector();
        //the file giving all partnerLinkTypes of this process
        this.partnerLinkTypeFile = new PartnerLinkTypeFile(process);
        //the file giving all complexTypes of this process
        this.complexTypesFile = new ComplexTypesFile(process);
        //a vector containing all messages of the wsdl
        this.messageTypes = new Vector();
        //a vector containing all partner link types of the wsdl
        this.plTypes = new HashMap();
        //a vector containing association between all ports and its declaring wsdl file
        this.portsWsdl = new Vector();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            this.docBuilder = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new BPELGeneratorException ("ParserConfigurationException caught in WSDLCompilo.init : ",e);
        }
    }

    public void analyse(String file, String processTargetNS) throws BPELGeneratorException {
        this.processTargetNS = processTargetNS;
        try {
            Emitter emit = Util.buildEmitter(file);
            emit.setVerbose(false);
            this.symbolTable = emit.getSymbolTable();
        } catch (BPELUtilException ex) {
            throw new BPELGeneratorException("" , ex);
        }
        analyseFile(file);
    }

    /**
     * analyse the wsdl file
     * @param file the wsdl file uri
     * @throws BPELGeneratorException,ParserConfigurationException,SAXException,IOException
     */
    public void analyseFile(String file) throws BPELGeneratorException {
        //the number of wsdl file opened (>1 if there is (are) import(s))
        this.wsdlFilesNb++;
        WSDLFactoryImpl wsdlFactory = new WSDLFactoryImpl();
        WSDLReader wsdlReader = wsdlFactory.newWSDLReader();
        try {
            this.def = wsdlReader.readWSDL(file);
        } catch (javax.wsdl.WSDLException e) {
            throw new BPELGeneratorException("WSDLException WSDLCompilo analyseFile :\n" , e);
        }

        //imports management
        Definition firstDef = this.def;
        Map imports = this.def.getImports();
        Set importsSet = imports.entrySet();

        for (Iterator i = importsSet.iterator(); i.hasNext();) {
            Map.Entry entry = (Map.Entry) i.next();
            Vector vect = (Vector) entry.getValue();
            for (int z = 0; z < vect.size(); z++) {
                ImportImpl imp = (ImportImpl) vect.get(z);
                this.def = imp.getDefinition();
                analyseFile(this.def.getDocumentBaseURI());
                this.def = firstDef;
            }
        }
        this.def = firstDef;
        buildNSProperties();
        try {
            this.document = this.docBuilder.parse(this.def.getDocumentBaseURI());
        } catch (IOException e) {
            throw new BPELGeneratorException("IOException WSDLCompilo analyseFile :\n" , e);
        } catch (SAXException e) {
            throw new BPELGeneratorException("SAXException WSDLCompilo analyseFile :\n" , e);
        }

        //wsdl object gives all informations of the definition tag in wsdl file
        this.wsdl = new Wsdl(this.processName);
        this.wsdl.addProperty("xmlns:xsd", BPELConstants.NS_XSD);

        analyseDefinitions();
        analysePartnerLinkTypes(this.document);
        analyseServices();
        analyseMessages();
        analysePropertyAlias(this.document);
        analyseComplexTypes();
        printBPELMessages();
        this.wsdlFilesNb--;
        if (this.wsdlFilesNb == 0) {
            finish();
        }

    }

    public SymbolTable getSymbolTable() {
        return this.symbolTable;
    }

    public Map getPartnerLinkTypes() {
        return this.plTypes;
    }

    public Wsdl getWsdlNSdef() {
        return this.wsdl;
    }

    /**
     * write the last instructions of the file
     */
    public void finish() throws BPELGeneratorException {
        for (Iterator it = this.plTypes.keySet().iterator(); it.hasNext();) {
            this.partnerLinkTypeFile.insertLine((PartnerLinkType) this.plTypes.get(it.next()));
        }
        this.partnerLinkTypeFile.finish();
        new WSDDFile(this.processName, this.wsdl, this.portsWsdl);
        this.complexTypesFile.finish();
    }

    /**
     * Analyse 'definitions' tag of wsdl file
     */
    private void analyseDefinitions() throws BPELGeneratorException {
        String tns = this.def.getTargetNamespace();
        if (tns != null) {
            this.wsdl.addProperty("targetNamespace", tns);
        }
        QName name = this.def.getQName();
        if (name != null) {
            this.wsdl.addProperty("name", name.getLocalPart());
        }
        Set ns = this.def.getNamespaces().entrySet();
        for (Iterator i = ns.iterator(); i.hasNext();) {
            Map.Entry entry = (Map.Entry) i.next();
            String sep = ":";
            if (entry.getKey().toString().equals("")) {
                sep = "";
            }
            this.wsdl.addProperty("xmlns" + sep + entry.getKey().toString(), entry.getValue().toString());
        }
    }

    /**
     * Analyse 'service' tag of wsdl file
     */
    private void analyseServices() throws BPELGeneratorException {
        /*
        try {
            this.emitter = Util.buildEmitter(this.def.getDocumentBaseURI());
        } catch (BPELUtilException ex) {
            throw new BPELGeneratorException("BPELUtilException WSDLCompilo analysePortTypes :\n" , ex);
        }
        Vector parameters = Util.fillParameters(this.emitter);
         */
        Map services = this.def.getServices();
        Set servSet = services.entrySet();
        for (Iterator i = servSet.iterator(); i.hasNext();) {
            Map.Entry entry0 = (Map.Entry) i.next();
            ServiceImpl s = (ServiceImpl) entry0.getValue();
            QName serv = s.getQName();
            //duplicate examination could occur due to import function
            if (!isAlreadyExamined(this.serviceExamined, serv.toString())) {
                Map ports = s.getPorts();
                Set p = ports.entrySet();
                for (Iterator j = p.iterator(); j.hasNext();) {
                    Map.Entry entry1 = (Map.Entry) j.next();
                    PortImpl port = (PortImpl) entry1.getValue();
                    String portName = port.getName();
                    this.portsWsdl.addElement(new PortWsdl(portName, this.def.getDocumentBaseURI()));
                    Binding binding = port.getBinding();
                    QName bindingQN = binding.getQName();
                    PortType pt = binding.getPortType();
                    QName ptQN = pt.getQName();
                    //List extL = port.getExtensibilityElements();
                    String addressLocation = WSDLUtils.getAddressFromPort(port);
                    if (addressLocation != null) {
                        try {
                            UDDIDirectory.addService(this.processName,serv, portName, bindingQN, addressLocation, ptQN);
                        } catch (BPELUtilException e) {
                            throw new BPELGeneratorException("BPELUtilException caught in WSDLCompilo.analyseServices : ",e);
                        }
                    }
                    HashMap signatures = null;
                    try {
                        signatures = Util.getSignatures(binding,this.processName);
                    } catch (BPELUtilException e) {
                        throw new BPELGeneratorException("BPELUtilException caught in WSDLCompilo.analyseServices : ",e);
                    }
                    analyseBinding(pt, binding);
                    analysePortType(pt,signatures);
                }
                this.serviceExamined.addElement(serv.toString());
            }
        }
    }

    /**
     * Analyse 'binding' tag of wsdl file
     */
    private void analyseBinding(PortType pt, Binding binding) throws BPELGeneratorException {
        String ptName = Util.buildStringFromQName(pt.getQName());
        QName bindingQN = binding.getQName();
        String bindingName = bindingQN.getLocalPart();
        //duplicate examination could occur due to import function
        if (!isAlreadyExamined(this.bindingExamined, bindingQN.toString())) {
            this.f = new WebServiceFile(this.processName, ptName, bindingName, this.wsdl, this.processTargetNS);
            new StubServiceFile(this.processName, this.wsdl.getTargetNamespace(), bindingName,BPELConstants.WSDL_DIR);
            this.bindingExamined.addElement(bindingQN.toString());
        }
    }

    /**
     * Analyse 'portType' tag of wsdl file
     */
    private void analysePortType(PortType pt, HashMap signatures) throws BPELGeneratorException {
        //String ptName = Util.buildStringFromQName(pt.getQName());
        //QName bindingQN = binding.getQName();
        //String bindingName = bindingQN.getLocalPart();
        List operationsL = pt.getOperations();
        for (Iterator j = operationsL.iterator(); j.hasNext();) {
            OperationImpl op = (OperationImpl) j.next();
            //String operationName = op.getName();

            //Parameters opParameters = Util.getOperationParameters(parameters, op, pt);

            Input in = op.getInput();
            String input = null;
            if (in != null) {
                Message mIn = in.getMessage();
                analyseMessage(mIn);
                input = Util.buildStringFromQName(mIn.getQName());
            }
            Output out = op.getOutput();
            String output = null;
            if (out != null) {
                Message mOut = out.getMessage();
                analyseMessage(mOut);
                output = Util.buildStringFromQName(mOut.getQName());
            }
            //Map faultsMap = op.getFaults();

            Vector inputParts = new Vector();
            Vector outputParts = new Vector();
            MessageType mt = null;
            String mName = null;
            for (int w = 0; w < this.messageTypes.size(); w++) {
                mt = (MessageType) this.messageTypes.get(w);
                mName = mt.getMessageName();
                if (input != null && input.equals(mName)) {
                    inputParts = mt.getParts();
                }
                if (output != null && output.equals(mName)) {
                    outputParts = mt.getParts();
                }
            }
            //String signature = opParameters.signature + " {";
            String signature = (String)signatures.get(Utils.xmlNameToJava(op.getName()));
            this.f.insertOperation(pt, op, input, output, inputParts, outputParts, signature);
        }
        this.f.finish();
    }

    /**
     * Build a vector containing in the right order all namespaces uri of
     * property tags
     */
    private void buildNSProperties() {
        this.propertiesNS = new Vector();
        List ext = this.def.getExtensibilityElements();
        for (Iterator j = ext.iterator(); j.hasNext();) {
            ExtensibilityElement ext0 = (ExtensibilityElement) j.next();
            QName qnExt = ext0.getElementType();
            if (qnExt.getLocalPart().equals(BPELConstants.PROPERTY)) {
                this.propertiesNS.addElement(qnExt.getNamespaceURI());
            }
        }
    }

    /**
     * Analyse 'partnerLinkType' tags of wsdl file
     */
    private void analysePartnerLinkTypes(Node n) {
        int type = n.getNodeType();
        switch (type) {
        case Node.ELEMENT_NODE:
            if (BPELConstants.PARTNERLINKTYPE.equals(BPELString.removePrefix(n.getNodeName()))) {
                analysePartnerLinkType(n);
            } else {
                analyseChildsPartnerLinkTypes(n);
            }
            break;
        case Node.TEXT_NODE:
            break;
        default:
            analyseChildsPartnerLinkTypes(n);
        break;
        }

    }

    /**
     * Analyse 'partnerLinkType' tag of wsdl file
     */
    private void analysePartnerLinkType(Node n) {
        String ns = this.wsdl.getTargetNamespace();
        String name = null;
        NamedNodeMap atts = n.getAttributes();
        for (int i = 0; i < atts.getLength(); i++) {
            Node att = atts.item(i);
            if (att.getNodeName().equals("name")) {
                name = att.getNodeValue();
            }
        }
        QName nameQN = new QName(ns, name);
        PartnerLinkType plt = new PartnerLinkType(nameQN);
        analysePartnerLinkTypeRoles(n, plt);
        this.plTypes.put(nameQN, plt);
    }

    /**
     * Analyse 'Role' tag of wsdl file
     */
    private void analysePartnerLinkTypeRoles(Node n, PartnerLinkType plt) {
        for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {
            if (child.getNodeType() == Node.ELEMENT_NODE
                    && BPELString.removePrefix(child.getNodeName()).equals("role") ) {
                String name = null;
                String portType = null;
                NamedNodeMap childAtts = child.getAttributes();
                for (int i = 0; i < childAtts.getLength(); i++) {
                    Node att = childAtts.item(i);
                    if (att.getNodeName().equals("name")) {
                        name = att.getNodeValue();
                        portType = analysePartnerLinkTypeRolePortType(child);
                        break;
                    }
                }
                QName ptQN = Util.buildQname(portType, this.wsdl);
                Role r = new Role(name, ptQN);
                plt.addRole(r);
            }
        }
    }

    /**
     * Analyse 'Role' tag of wsdl file
     */
    private String analysePartnerLinkTypeRolePortType(Node n) {
        for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {
            if (child.getNodeType() == Node.ELEMENT_NODE
                    && BPELString.removePrefix(child.getNodeName()).equals("portType")) {
                NamedNodeMap childAtts = child.getAttributes();
                for (int i = 0; i < childAtts.getLength(); i++) {
                    Node att = childAtts.item(i);
                    if (att.getNodeName().equals("name")) {
                        return att.getNodeValue();
                    }
                }
            }
        }
        return null;
    }

    /**
     * Analyse 'PropertyAlias' tag of wsdl file
     */
    private void analysePropertyAlias(Node n) {
        String propertyName = null;
        String messageType = null;
        String part = null;
        String query = null;
        int type = n.getNodeType();

        switch (type) {
        case Node.ELEMENT_NODE:
            if (n.getNodeName().endsWith(BPELConstants.PROPERTYALIAS)) {
                NamedNodeMap atts = n.getAttributes();
                for (int i = 0; i < atts.getLength(); i++) {
                    Node att = atts.item(i);
                    if (att.getNodeName().equals("propertyName"))
                        propertyName = new String(att.getNodeValue());
                    else if (att.getNodeName().equals("messageType"))
                        messageType = new String(att.getNodeValue());
                    else if (att.getNodeName().equals("part"))
                        part = new String(att.getNodeValue());
                    else if (att.getNodeName().equals("query"))
                        query = new String(att.getNodeValue());
                }
                QName mtQN = Util.buildQname(messageType, this.wsdl);
                QName pnQN = Util.buildQname(propertyName, this.wsdl);
                String mt = Util.buildStringFromQName(mtQN);
                String pn = Util.buildStringFromQName(pnQN);

                MessageType mess = findMessageType(mt);
                Vector mapping = new Vector();
                if (query != null) {
                    Vector prefixs = Util.getPrefixFromQuery(query);
                    for (int i=0;i<prefixs.size();i++) {
                        String prefix = (String)prefixs.get(i);
                        String ns = this.def.getNamespace(prefix);
                        mapping.addElement(prefix);
                        mapping.addElement(ns);
                    }
                }

                mess.addCS(pn, part, query,mapping);

            } else {
                analyseChildsPropertyAlias(n);
            }
            break;
        case Node.TEXT_NODE:
            break;
        default:
            analyseChildsPropertyAlias(n);
        break;
        }

    }

    /**
     * Analyse 'message' tag of wsdl file
     */
    private void analyseMessage(Message m) {
        String messageName = Util.buildStringFromQName(m.getQName());
        //duplicate examination could occur due to import function
        if (!isAlreadyExamined(this.messExamined, messageName)) {
            MessageType mess = new MessageType(this.processName, messageName);
            List parts = m.getOrderedParts(null);
            for (Iterator j = parts.iterator(); j.hasNext();) {
                PartImpl part = (PartImpl) j.next();
                String partName = part.getName();
                String partType = null;
                QName partElementQN = part.getElementName();
                partType = Util.getJavaType(this.symbolTable, part.getTypeName(), part.getElementName());
                mess.addPart(partName, partType, partElementQN);
            }
            this.messageTypes.add(mess);
            this.messExamined.addElement(messageName);
        }

    }

    /**
     * Analyse 'message' tag of wsdl file
     */
    private void analyseMessages() {
        Map messages = this.def.getMessages();
        Set messSet = messages.entrySet();
        for (Iterator i = messSet.iterator(); i.hasNext();) {
            Map.Entry entry0 = (Map.Entry) i.next();
            MessageImpl m = (MessageImpl) entry0.getValue();
            analyseMessage(m);
        }
    }

    /**
     * Analyse 'complexType' tag of wsdl file
     */
    private void analyseComplexTypes() throws BPELGeneratorException {
        insertTypeEntries(this.symbolTable.getElementIndex());
        insertTypeEntries(this.symbolTable.getTypeIndex());
    }

    /**
     * Writes the type entries into the complexTypesFile
     * @param typeEntries a Map of the typeEntries to browse
     * @throws BPELGeneratorException
     */
    private final void insertTypeEntries(Map typeEntries) throws BPELGeneratorException {
        Set typeEntrySet = typeEntries.entrySet();
        for (Iterator i = typeEntrySet.iterator(); i.hasNext();) {
            Map.Entry entry = (Map.Entry) i.next();
            TypeEntry type = (TypeEntry) entry.getValue();

            String xmlType = Util.buildStringFromQName(type.getQName());
            String javaType = type.getName();
            this.complexTypesFile.insertComplexType(xmlType, javaType);
        }
    }

    /**
     * Say if the given property is in the given vector
     * @param v the vector to analyse
     * @param property the searched property
     * @return boolean true if the given property is in the given vector
     */
    private boolean isAlreadyExamined(Vector v, String property) {
        boolean found = false;
        for (int i = 0; i < v.size(); i++) {
            if (((String) v.get(i)).equals(property)) {
                found = true;
                break;
            }
        }
        return found;
    }

    /**
     * Find a message based on its type
     * @param type the type of the message
     */
    private MessageType findMessageType(String type) {
        int i = 0;
        while ((i < this.messageTypes.size()) && !(((MessageType) this.messageTypes.get(i)).getMessageName().equals(type))) {
            i++;
        }
        if (i < this.messageTypes.size()) {
            return (MessageType) this.messageTypes.get(i);
        }
        return null;
    }

    /**
     * Print a BPELMessage for each message of the wsdl file
     */
    private void printBPELMessages() throws BPELGeneratorException {
        for (int i = 0; i < this.messageTypes.size(); i++) {
            ((MessageType) this.messageTypes.get(i)).printBPELFile();
            ((MessageType) this.messageTypes.get(i)).printXPathFactory();
        }
    }

    /**
     * recursive way to analyse children of n
     * @param n the parent node
     */
    private void analyseChildsPropertyAlias(Node n) {
        for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {
            analysePropertyAlias(child);
        }
    }

    /**
     * recursive way to analyse children of n
     * @param n the parent node
     */
    private void analyseChildsPartnerLinkTypes(Node n) {
        for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {
            analysePartnerLinkTypes(child);
        }
    }

}
