/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.Serializable;
import java.util.Vector;

import org.objectweb.orchestra.exception.BPELGeneratorException;


/**
 * Wsdl.java, represents the definitions tag at the beginning of the wsdl file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: Wsdl.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class Wsdl extends NamespaceDef implements Serializable {

    private Vector properties = null;

    /**
     * Default Constructor
     *
     * @param processName
     *            the process name
     */
    public Wsdl(String processName) {
        this.processName = processName;
        this.properties = new Vector();
    }

    /**
     * Add a property
     *
     * @param property
     *            the property to add
     * @param value
     *            the value of the property
     */
    public void addProperty(String property, String value) throws BPELGeneratorException {
        String[] elements = new String[1];
        elements[0] = property + "=" + value;
        WsdlElement we = null;
        try {
            we = new WsdlElement(elements);
        } catch (Exception e) {
            throw new BPELGeneratorException("Wsdl.addProperty",e);
        }
        this.properties.add(we);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.orchestra.engine.NamespaceDef#getNS(java.lang.String)
     */
    public String getNS(String alias) {
        String ns = null;
        WsdlElement prop = null;
        String prefix = null;
        for (int i = 0; i < this.properties.size(); i++) {
            prop = (WsdlElement) this.properties.get(i);
            if (prop.getType().equals("xmlns")) {
                prefix = prop.getPrefix();
                if (prefix.indexOf(":") != -1 && prefix.substring(prefix.indexOf(":") + 1).equals(alias)) {
                    ns = prop.getNamespaceURI();
                }
            }
        }
        return ns;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.orchestra.engine.NamespaceDef#getTargetNamespace()
     */
    public String getTargetNamespace() {
        String result = null;
        WsdlElement prop = null;
        for (int i = 0; i < this.properties.size(); i++) {
            prop = (WsdlElement) this.properties.get(i);
            if (prop.getType().equals("targetNamespace")) {
                result = prop.getTargetNamespace();
            }
        }
        return result;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.orchestra.engine.NamespaceDef#getName()
     */
    public String getName() {
        String result = null;
        WsdlElement prop = null;
        for (int i = 0; i < this.properties.size(); i++) {
            prop = (WsdlElement) this.properties.get(i);
            if (prop.getType().equals("name")) {
                result = prop.getName();
            }
        }
        return result;
    }

}