/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;

import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Jar;
import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.taskdefs.Javac;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.Environment;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * AnalyseProcessFilesTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: AnalyseProcessFilesTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class AnalyseProcessFilesTask extends StandardDeployTask {

    private File bpelFile = null;

    public AnalyseProcessFilesTask() {
        setTaskName("analyseProcessFiles");
    }

    public void execute() {
        log("Starting AnalyseProcessFilesTask...");
        deleteDirs();
        createDirs();
        compileToTmpJXPath();
        bpelAnalyser();
        manageFiles();
        log("Finishing AnalyseProcessFilesTask...");
    }

    private void createDirs() {
        String st = getProp("build_JXPath_dir") + this.sep + getProcessName();
        createDirName(st);
    }

    private void compileToTmpJXPath() {
        Javac j = new Javac();
        j.setProject(getProject());
        Path srcDir = new Path(getProject());
        srcDir.setLocation(new File(getProp("generate_wsdl_dir") + this.sep + getProcessName()));
        j.setSrcdir(srcDir);
        j.setDestdir(new File(getProp("build_JXPath_dir") + this.sep + getProcessName()));
        Path compileCP = new Path(getProject());
        compileCP.setPath(getProp("generate_wsdl_dir") + this.sep + getProcessName() + this.sep + "**" + this.sep + "*.java");
        j.setSourcepath(compileCP);
        j.setTaskName("javac");
        j.setSource(getProp("jdkVersion"));
        j.init();
        j.setClasspath((Path) getProject().getReference("compile.classpath"));
        j.execute();

        Jar jar = new Jar();
        jar.setTaskName("jar");
        jar.setProject(getProject());
        String st = getProp("build_JXPath_dir") + this.sep + getProcessName() + this.sep + getProcessName()+".jar";
        File f = new File(st);
        jar.setDestFile(f);

        FileSet fs1 = new FileSet();
        fs1.setDir(new File(getProp("build_JXPath_dir") + this.sep + getProcessName()));
        fs1.setIncludes("**");
        fs1.setExcludes("**/*.jar");
        jar.addFileset(fs1);

        jar.execute();
    }

    public void bpelAnalyser() {
        Java j = new Java();
        j.setTaskName("java");
        j.setProject(getProject());
        j.setClassname("org.objectweb.orchestra.generator.BPELAnalyser");
        j.setFork(true);
        Commandline.Argument wsdlFile = j.createArg();
        wsdlFile.setLine(getProp(getProcessName() + "_wsdl_file"));
        //if (!getProp("mode").equals("testsUtil")) {
        this.bpelFile = new File(getProp(getProcessName() + "_bpel_file"));
        //System.out.println(getProcessdir() + sep + getProcessname() + sep + getProcessname() + ".bpel");
        if (this.bpelFile.exists()) {
            Commandline.Argument bpelFile = j.createArg();
            bpelFile.setLine(getProp(getProcessName() + "_bpel_file"));
        }
        Environment.Variable bpelHome = new Environment.Variable();
        bpelHome.setKey(BPELConstants.BPEL_HOME_PROPERTY);
        bpelHome.setPath(new Path(getProject(), BPELConstants.BPEL_HOME));
        j.addSysproperty(bpelHome);
        Environment.Variable axisHandlers = new Environment.Variable();
        axisHandlers.setKey("java.protocol.handler.pkgs");
        axisHandlers.setValue(getProp("handler_pkgs"));
        j.addSysproperty(axisHandlers);
        j.init();
        j.setClasspath((Path) getProject().getReference("base.classpath"));
        j.execute();
    }

    public void manageFiles() {
        Copy c1 = new Copy();
        c1.setProject(getProject());
        String st = getProp("build_tmp_dir") + this.sep + getProcessName();
        c1.setTodir(new File(st));

        FileSet fs1 = new FileSet();
        st = getProp("generate_wsdl_dir") + this.sep + getProcessName();
        fs1.setDir(new File(st));
        fs1.setIncludes("**/*.java,**/*.wsdd");
        //it is an external WS, so an implementation is given by the user, so the default one is unused
        if (!this.bpelFile.exists()) {
            fs1.setExcludes("**/*Impl.java");
        }

        c1.addFileset(fs1);
        c1.setTaskName("copy");
        c1.execute();

        Copy c2 = new Copy();
        c2.setProject(getProject());
        st = getProp("build_tmp_dir") + this.sep + getProcessName();
        c2.setTodir(new File(st));
        c2.setOverwrite(true);

        FileSet fs2 = new FileSet();
        st = getProp("generate_generator_dir") + this.sep + getProcessName();
        fs2.setDir(new File(st));
        fs2.setIncludes("**/*.java");
        //it is not a process, so impl generated by generator is unused
        if (!this.bpelFile.exists()) {
            fs2.setExcludes("**/*Impl.java");
        }
        c2.addFileset(fs2);

        c2.setTaskName("copy");
        c2.execute();

        File srcDir = new File(getProp("srcDir"));
        if (srcDir.exists()) {
            Copy c3 = new Copy();
            c3.setProject(getProject());
            st = getProp("build_tmp_dir") + this.sep + getProcessName();
            c3.setTodir(new File(st));
            c3.setOverwrite(true);

            FileSet fs3 = new FileSet();
            fs3.setDir(srcDir);
            fs3.setIncludes("**/*.java");
            c3.addFileset(fs3);

            c3.setTaskName("copy");
            c3.execute();
        }
    }

    private void deleteDirs() {
        if (getProcessDir("build_JXPath_dir").exists()) {
            deleteProcessDir("build_JXPath_dir");
        }
    }

}