/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;

import org.apache.tools.ant.taskdefs.Jar;
import org.apache.tools.ant.taskdefs.Javac;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.PatternSet;


/**
 * AnalyseProcessFilesTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: CompileClientTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class CompileClientTask extends StandardDeployTask {

    private String dir = null;
    private String includes = null;
    private String excludes = null;
    private String classpath = null;
    private String classesDir = null;
    private String jarName = null;

    public void setDir(String dir) {
        this.dir = dir;
    }

    public String getDir() {
        return this.dir;
    }

    public void setIncludes(String includes) {
        this.includes = includes;
    }

    public String getIncludes() {
        return this.includes;
    }

    public void setExcludes(String excludes) {
        this.excludes = excludes;
    }

    public String getExcludes() {
        return this.excludes;
    }

    public void setClasspath(String classpath) {
        this.classpath = classpath;
    }

    public String getClasspath() {
        return this.classpath;
    }
    public CompileClientTask() {
        setTaskName("compileClient");
    }

    public void execute() {
        log("Starting CompileClientTask...");
        compile();
        jar();
        deleteClasses();
        log("Finishing CompileClientTask...");
    }

    public void compile() {
        this.classesDir = getProp("build_classes_dir") + this.sep + getProcessName()+ this.sep + "clientFiles";

        Path cp = new Path(getProject());
        FileSet fs1 = new FileSet();
        fs1.setDir(new File(getProp("build_jars_dir")));
        PatternSet.NameEntry ps1 = fs1.createInclude();
        ps1.setName(getProcessName()+".jar");

        FileSet fs2 = new FileSet();
        fs2.setDir(new File(getProp("lib_ext_dir")));
        PatternSet.NameEntry ps2 = fs2.createInclude();
        ps2.setName("**/*.jar");

        FileSet fs3 = new FileSet();
        fs3.setDir(new File(getProp("jonas_root_lib")));
        PatternSet.NameEntry ps3 = fs3.createInclude();
        ps3.setName("**/*.jar");

        FileSet fs4 = new FileSet();
        fs4.setDir(new File(getProp("lib_dir")));
        PatternSet.NameEntry ps4 = fs4.createInclude();
        ps4.setName("**/*.jar");

        FileSet fs5 = new FileSet();
        fs5.setDir(new File(getProp("jonas_base_lib_ext")));
        PatternSet.NameEntry ps5 = fs5.createInclude();
        ps5.setName("**/*.jar");

        cp.addFileset(fs1);
        cp.addFileset(fs2);
        cp.addFileset(fs3);
        cp.addFileset(fs4);
        cp.addFileset(fs5);

        Path srcDir = new Path(getProject());
        srcDir.setLocation(new File(this.dir));

        if (getClasspath() != null) {
            Path p = new Path(getProject());
            p.setPath(this.classpath);
            cp.add(p);
        }

        createDirName(this.classesDir);

        Javac j = new Javac();
        j.setProject(getProject());
        j.setDestdir(new File(this.classesDir));
        j.setSrcdir(srcDir);
        if (getIncludes() != null) {
            j.setIncludes(getIncludes());
        }
        if (getExcludes() != null) {
            j.setExcludes(getExcludes());
        }

        j.setTaskName("javac");
        j.setSource(getProp("jdkVersion"));
        j.init();
        j.setClasspath(cp);
        j.execute();
    }

    public void jar() {
        String jarDir = getProp("build_clientJars_dir");
        createDirName(jarDir);
        this.jarName = jarDir + this.sep + getProcessName() + ".jar";

        Jar j = new Jar();
        j.setTaskName("jar");
        j.setProject(getProject());

        File f = new File(this.jarName);
        j.setDestFile(f);

        FileSet fs1 = new FileSet();
        fs1.setDir(new File(this.classesDir));
        j.addFileset(fs1);

        j.execute();
    }

    public void deleteClasses() {
        deleteDir(this.classesDir);
    }
}