/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;
import java.util.Vector;

import javax.wsdl.Definition;
import javax.wsdl.xml.WSDLReader;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Copy;
import org.objectweb.orchestra.util.Util;

import com.ibm.wsdl.factory.WSDLFactoryImpl;

/**
 * DeployProcessTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: DeployProcessTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class DeployProcessTask extends StandardDeployTask {

    private Vector tasks = null;

    private String bpelFile = null;
    private String wsdlFile = null;
    private String srcDir = null;
    private boolean skipwsdl = false;

    public void setBpelFile(String bpelFile) {
        this.bpelFile = bpelFile;
    }

    public String getBpelFile() {
        return this.bpelFile;
    }

    public void setWsdlFile(String wsdlFile) {
        this.wsdlFile = wsdlFile;
    }

    public String getWsdlFile() {
        return this.wsdlFile;
    }

    public void setSrcDir(String srcDir) {
        this.srcDir = srcDir;
    }

    public String getSrcDir() {
        return this.srcDir;
    }

    public void setSkipwsdl(boolean skipwsdl) {
        this.skipwsdl = skipwsdl;
    }

    public boolean getSkipwsdl() {
        return this.skipwsdl;
    }

    public DeployProcessTask(String[] outServices, boolean skipwsdl) {
        this.tasks = new Vector();
        setTaskName("deployProcess");
        this.skipwsdl = skipwsdl;
        if (getTaskWithName("initProcess") == null) {
            //here we are sure that this task is called from the java may
            //not by ant
            addTask(this.tasks, new InitTask());
            addTask(this.tasks, new OutServicesTask(outServices));
            addTask(this.tasks, new CompileProcessTask(skipwsdl,outServices));
            addTask(this.tasks, new ArchiveProcessTask());
            addTask(this.tasks, new LoadProcessTask());
            addTask(this.tasks, new LoadProcessWSTask());
        }
    }

    public void execute() {
        log("Starting DeployProcessTask...");
        createProperties();
        copyFiles();
        this.tasks = organizeTasks();
        for (int i = 0; i < this.tasks.size(); i++) {
            StandardDeployTask task = (StandardDeployTask) this.tasks.get(i);
            task.setProcessName(getProcessName());
            ((Task) task).setProject(getProject());
            ((Task) task).execute();
        }
        log("Finishing DeployProcessTask...");
    }

    private void createProperties() {
        try {
            WSDLFactoryImpl wsdlFactory = new WSDLFactoryImpl();
            WSDLReader wsdlReader = wsdlFactory.newWSDLReader();
            Definition def = wsdlReader.readWSDL(getWsdlFile());
            String tns = def.getTargetNamespace();
            String directory = Util.getDirectoryFromPackage(Util.getPackageFromNamespace(tns));
            String wsddDir = getProp("generate_wsdl_dir") + this.sep + getProcessName() + this.sep + directory;
            createProperty("wsddProcessDir",wsddDir);
        } catch (Exception e) {
            throw new BuildException("Exception caught in DeployProcessTask.createProperties : ",e);
        }
    }

    private void copyFiles() {
        String base = getProp("build_process_files_dir")+this.sep+getProcessName()+this.sep+getProcessName();
        Copy c = new Copy();
        c.setProject(getProject());
        c.setTaskName("copy");
        c.setOverwrite(true);

        if (getBpelFile() != null) {
            c.setTofile(new File(base+".bpel"));
            c.setFile(new File(getBpelFile()));
            c.execute();
        }
        if (getWsdlFile() != null) {
            c.setTofile(new File(base+".wsdl"));
            c.setFile(new File(getWsdlFile()));
            c.execute();
        }

        if (getSrcDir() != null) {
            createProperty("srcDir",getSrcDir());
        } else {
            createProperty("srcDir","null");
        }
    }

    public Task getTaskWithName(String name) {
        Task t = null;
        for (int i = 0; i < this.tasks.size(); i++) {
            Task task = (Task) this.tasks.get(i);
            if (task.getTaskName().equals(name)) {
                t = task;
                break;
            }
        }
        return t;
    }

    public Vector organizeTasks() {
        Vector newV = new Vector();
        newV.addElement(getTaskWithName("initProcess"));
        newV.addElement(getTaskWithName("outServicesProcess"));
        newV.addElement(getTaskWithName("compileProcess"));
        newV.addElement(getTaskWithName("archiveProcess"));
        newV.addElement(getTaskWithName("loadProcessJar"));
        newV.addElement(getTaskWithName("loadProcessWS"));
        return newV;
    }

    public void addConfiguredInitProcess(InitTask task) {
        addTask(this.tasks, task);
    }

    public void addConfiguredOutServicesProcess(OutServicesTask task) {
        addTask(this.tasks, task);
    }

    public void addConfiguredCompileProcess(CompileProcessTask task) {
        addTask(this.tasks, task);
    }

    public void addConfiguredArchiveProcess(ArchiveProcessTask task) {
        addTask(this.tasks, task);
    }

    public void addConfiguredLoadProcessJar(LoadProcessTask task) {
        addTask(this.tasks, task);
    }

    public void addConfiguredLoadProcessWS(LoadProcessWSTask task) {
        addTask(this.tasks, task);
    }

}