/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;

import org.apache.tools.ant.taskdefs.Property;
import org.apache.tools.ant.taskdefs.condition.IsSet;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.PatternSet;
import org.objectweb.orchestra.util.BPELConstants;



/**
 * InitPropertiesTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version  $Id: InitPropertiesTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class InitPropertiesTask extends StandardDeployTask {

    private String sep = File.separator;

    public InitPropertiesTask() {
        setTaskName("initProperties");
    }

    public void execute() {
        log("Starting InitPropertiesTask...");
        createProperties();
        definePaths();
        echo();
        log("Finishing InitPropertiesTask...");
    }

    public void createProperties() {
        createProperty("opt_javac_debug", "true");
        createProperty("jdkVersion", "1.4");
        Property myenv = createEnvProperty("myenv");
        createProperty("java_home", getProp(BPELConstants.JAVA_HOME_ENV));
        createProperty("cp", getProp(BPELConstants.CLASSPATH_ENV));
        createProperty("jonas_root", getProp(BPELConstants.JONAS_ROOT_ENV));
        createProperty("jonas_base", getProp(BPELConstants.JONAS_BASE_ENV));
        createProperty("ant_home", getProp(BPELConstants.ANT_HOME_ENV));
        createProperty("bpel_home", getProp(BPELConstants.BPEL_HOME_ENV));
        createProperty("ant_home_lib", getProp("ant_home") + this.sep + "lib");
        createProperty("jonas_root_lib", getProp("jonas_root") + this.sep + "lib");
        createProperty("client.jar", getProp("jonas_root_lib") + this.sep + "client.jar");
        createProperty("jonas_base_ejbjars", getProp("jonas_base") + this.sep + "ejbjars");
        createProperty("jonas_base_conf", getProp("jonas_base") + this.sep + "conf");
        createProperty("jonas_base_lib", getProp("jonas_base") + this.sep + "lib");
        createProperty("jonas_base_lib_ext", getProp("jonas_base_lib") + this.sep + "ext");
        createProperty("jaas_config", getProp("jonas_base_conf") + this.sep + "jaas.config");

        createPropertyFile(BPELConstants.BUILD_PROPERTIES_DIR);
        createProperty("lib_dir", BPELConstants.LIB_DIR);
        createProperty("lib_ext_dir", BPELConstants.LIB_EXT_DIR);
        createProperty("axis_dir", BPELConstants.LIB_AXIS_DIR);
        createProperty("axis-ant", getProp("axis_dir") + this.sep + "axis-ant.jar");
        createProperty("support_dir", BPELConstants.SUPPORT_DIR);
        createProperty("build_dir", BPELConstants.BUILD_DIR);
        createProperty("build_ejbjars_dir", BPELConstants.BUILD_EJBJARS_DIR);
        createProperty("expand_dir", BPELConstants.EXPAND_DIR);
        createProperty("build_classes_dir", BPELConstants.BUILD_CLASSES_DIR);
        createProperty("build_jars_dir", BPELConstants.BUILD_JARS_DIR);
        createProperty("build_wsdd_dir", BPELConstants.BUILD_WSDD_DIR);
        createProperty("build_jbi_dir", BPELConstants.BUILD_JBI_DIR);
        createProperty("build_tmp_dir", BPELConstants.BUILD_TMP_DIR);
        createProperty("build_test_dir", BPELConstants.BUILD_TEST_DIR);
        createProperty("build_process_files_dir", BPELConstants.BUILD_PROCESSFILES_DIR);
        createProperty("build_JXPath_dir", BPELConstants.BUILD_JXPATH_DIR);
        createProperty("build_xml_dir", BPELConstants.BUILD_XML_DIR);

        createProperty("bpel.jar", BPELConstants.BPEL_JAR);
        createProperty("generate_dir", BPELConstants.GENERATE_DIR);
        createProperty("generate_wsdl_dir", BPELConstants.WSDL_DIR);
        createProperty("generate_process_dir", BPELConstants.PROCESS_DIR);
        createProperty("generate_wsdlfile_dir", BPELConstants.WSDLFILE_DIR);
        createProperty("generate_wsdlbindingfile_dir", BPELConstants.WSDLBINDING_DIR);
        createProperty("generate_processmap_dir", BPELConstants.PROCESSMAP_DIR);
        createProperty("generate_generator_dir", BPELConstants.GENERATOR_DIR);
        createProperty("generate_xdoclet_dir", BPELConstants.XDOCLET_DIR);
        createProperty("generate_outServices_dir", BPELConstants.OUTSERVICES_DIR);
        createProperty("generate_plTypes_dir", BPELConstants.PLTYPES_DIR);
        createProperty("generate_complexTypes_dir", BPELConstants.COMPLEXTYPES_DIR);

        createProperty("tests_dir", BPELConstants.TESTS_DIR);
        createProperty("samples_dir", BPELConstants.SAMPLES_DIR);
        createProperty("reports_dir", BPELConstants.REPORTS_DIR);
        createProperty("test_util_dir", BPELConstants.TESTS_UTIL_DIR);
        createProperty("test_util_pack_dir", BPELConstants.TESTS_UTIL_PACK_DIR);
        createProperty("build_clientJars_dir", BPELConstants.BUILD_CLIENTJARS_DIR);
        createProperty("handler_pkgs", BPELConstants.HANDLER_PKGS);
        createProperty("wsddExtJbi", BPELConstants.BUILD_EXT_WSDD_DIR);

        IsSet i = new IsSet();
        i.setProperty("myenv.JAVA_OPTS");
        i.setProject(getProject());
        if (i.eval()) {
            createProperty("java_opts", getProp(myenv.getEnvironment() + ".JAVA_OPTS"));
        } else {
            createProperty("java_opts", "");
        }
    }

    public void definePaths() {
        String s = null;

        Path compile_classpath = new Path(getProject());
        compile_classpath.setProject(getProject());

        Path.PathElement pe1 = compile_classpath.createPathElement();
        s = getProp("bpel.jar");
        pe1.setLocation(new File(s));

        FileSet fs2 = new FileSet();
        fs2.setDir(new File(getProp("lib_dir")));
        PatternSet.NameEntry ps2 = fs2.createInclude();
        ps2.setName("**/*.jar");
        compile_classpath.addFileset(fs2);

        FileSet fs3 = new FileSet();
        fs3.setDir(new File(getProp("jonas_root_lib")));
        PatternSet.NameEntry ps3 = fs3.createInclude();
        ps3.setName("**/*.jar");
        compile_classpath.addFileset(fs3);

        FileSet fs4 = new FileSet();
        fs4.setDir(new File(getProp("build_jars_dir")));
        PatternSet.NameEntry ps4 = fs4.createInclude();
        ps4.setName("*.jar");
        compile_classpath.addFileset(fs4);

        FileSet fs5 = new FileSet();
        fs5.setDir(new File(getProp("ant_home_lib")));
        PatternSet.NameEntry ps5 = fs5.createInclude();
        ps5.setName("*.jar");
        compile_classpath.addFileset(fs5);

        FileSet fs6 = new FileSet();
        fs6.setDir(new File(getProp("lib_ext_dir")));
        PatternSet.NameEntry ps6 = fs6.createInclude();
        ps6.setName("**/*.jar");
        compile_classpath.addFileset(fs6);

        FileSet fs7 = new FileSet();
        fs7.setDir(new File(getProp("support_dir")));
        PatternSet.NameEntry ps7 = fs7.createInclude();
        ps7.setName("**/*.jar");
        compile_classpath.addFileset(fs7);

        getProject().addReference("compile.classpath", compile_classpath);

        Path base_classpath = new Path(getProject());
        base_classpath.setProject(getProject());
        base_classpath.add(compile_classpath);

        FileSet fs8 = new FileSet();
        fs8.setDir(new File(getProp("bpel_home")));
        PatternSet.NameEntry ps8 = fs8.createInclude();
        ps8.setName("client-config.wsdd");
        base_classpath.addFileset(fs8);

        getProject().addReference("base.classpath", base_classpath);
    }

    public void echo() {
        echoProperty("bpel_home");
    }
}