/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import org.apache.tools.ant.taskdefs.ConditionTask;
import org.apache.tools.ant.taskdefs.Exit;
import org.apache.tools.ant.taskdefs.condition.And;
import org.apache.tools.ant.taskdefs.condition.Equals;
import org.apache.tools.ant.taskdefs.condition.IsSet;
import org.apache.tools.ant.taskdefs.condition.Not;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * InitTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: InitTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class InitTask extends StandardDeployTask {

    public InitTask() {
        setTaskName("initProcess");
    }

    public void execute() {
        log("Starting InitTask...");
        createDirs();
        checkEnv();
        checkUsage();
        log("Finishing InitTask...");
    }

    public void createDirs() {
        createDir("build_classes_dir");
        createDir("build_jars_dir");
        createDir("build_wsdd_dir");
        createDir("build_tmp_dir");
        createDir("generate_xdoclet_dir");
        createDir("generate_generator_dir");
        createDir("generate_wsdl_dir");
        createDir("generate_processmap_dir");
        createDir("generate_wsdlfile_dir");
        createDir("generate_wsdlbindingfile_dir");
    }

    public void checkEnv() {
        ConditionTask ct = new ConditionTask();
        ct.setProperty("environment_ok");
        And and = new And();
        IsSet i1 = new IsSet();
        i1.setProperty(BPELConstants.JONAS_ROOT_ENV);
        i1.setProject(getProject());
        and.addIsSet(i1);
        IsSet i2 = new IsSet();
        i2.setProperty(BPELConstants.JONAS_BASE_ENV);
        i2.setProject(getProject());
        and.addIsSet(i2);
        IsSet i3 = new IsSet();
        i3.setProperty(BPELConstants.JAVA_HOME_ENV);
        i3.setProject(getProject());
        and.addIsSet(i3);
        IsSet i4 = new IsSet();
        i4.setProperty(BPELConstants.BPEL_HOME_ENV);
        i4.setProject(getProject());
        and.addIsSet(i4);
        IsSet i5 = new IsSet();
        i5.setProperty(BPELConstants.ANT_HOME_ENV);
        i5.setProject(getProject());
        and.addIsSet(i5);
        and.setProject(getProject());
        ct.addAnd(and);
        ct.setProject(getProject());
        ct.execute();

        Exit exit = new Exit();
        String st = "\n********** E R R O R ******************************";
        st += "\nBPEL Generator";
        st += "\nAn error occurred when compiling the process.";
        st += "\nPlease set your JONAS_ROOT, JONAS_BASE, BPEL_HOME, JAVA_HOME, ANT_HOME environment variables before continuing.";
        exit.setMessage(st);
        exit.setUnless("environment_ok");
        exit.setProject(getProject());
        exit.setTaskName("exit");
        exit.execute();
    }

    public void checkUsage() {
        ConditionTask ct = new ConditionTask();
        ct.setProperty("usage_ok");
        And and = new And();
        and.setProject(getProject());
        IsSet i1 = new IsSet();
        i1.setProperty("bpel.host");
        i1.setProject(getProject());
        and.addIsSet(i1);

        Not not1 = new Not();
        not1.setProject(getProject());
        Equals eq1 = new Equals();
        eq1.setArg1(getProp("bpel.host"));
        eq1.setArg2("");
        eq1.setTrim(true);
        not1.addEquals(eq1);
        and.addNot(not1);

        IsSet i2 = new IsSet();
        i2.setProperty("bpel.port");
        i2.setProject(getProject());
        and.addIsSet(i2);

        Not not2 = new Not();
        not2.setProject(getProject());
        Equals eq2 = new Equals();
        eq2.setArg1(getProp("bpel.port"));
        eq2.setArg2("");
        eq2.setTrim(true);
        not2.addEquals(eq2);
        and.addNot(not2);

        ct.addAnd(and);
        ct.setProject(getProject());
        ct.execute();

        Exit exit = new Exit();
        String st = "\n********** E R R O R ******************************";
        st += "\nYour server host or/and your server name is/are not set";
        st += "\nUpdate BPEL_HOME/build.properties file to configure server properties";
        exit.setMessage(st);
        exit.setUnless("usage_ok");
        exit.setProject(getProject());
        exit.setTaskName("exit");
        exit.execute();
    }
}