/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Javac;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.PatternSet;
import org.objectweb.jonas.ant.EjbJar;
import org.objectweb.jonas.ant.JonasDeploymentTool;
import org.objectweb.jonas.container.EJBService;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * JmsTestTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version  $Id: JmsTestTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class JmsTestTask extends StandardDeployTask {

    public JmsTestTask() {
        setTaskName("jmsTest");
    }

    public void execute() {
        log("Starting JmsTestTask...");
        createProperties();
        clean();
        createPaths();
        initTask();
        compile();
        jonasejbjar();
        install();
        log("Finishing JmsTestTask...");
    }

    public void createProperties() {
        createPropertyFile(BPELConstants.BUILD_PROPERTIES_DIR);
        createEnvProperty(BPELConstants.ANT_ENV);
        createProperty("jonas_root", BPELConstants.JONAS_ROOT);
        createProperty("jonas_base", BPELConstants.JONAS_BASE);
        createProperty("jonas_base_ejbjars", getProp("jonas_base") + this.sep + "ejbjars");
        createProperty("verbose", "true");
        createProperty("test_util_pack_dir", BPELConstants.TESTS_UTIL_PACK_DIR);
        createProperty("current_dir", getProp("test_util_pack_dir") + this.sep + getProcessName());
        createProperty("dist_dir", getProp("current_dir"));
        createProperty("src_dir", getProp("current_dir"));
        createProperty("classes_dir", getProp("dist_dir") + this.sep + "classes");
        createProperty("dist_ejbjars_dir", getProp("dist_dir") + this.sep + "ejbjars");
        createProperty("jonas_root_lib", getProp("jonas_root") + this.sep + "lib");
    }

    public void clean() {
        deleteDir(getProp("classes_dir"));
        deleteDir(getProp("dist_ejbjars_dir"));
        adminServer("remove");
    }

    public void createPaths() {
        String s = null;

        Path base_classpath = new Path(getProject());
        base_classpath.setProject(getProject());

        Path.PathElement pe1 = base_classpath.createPathElement();
        s = getProp("classes_dir");
        pe1.setLocation(new File(s));

        FileSet fs1 = new FileSet();
        fs1.setDir(new File(getProp("jonas_root_lib") + this.sep + "commons"));
        PatternSet.NameEntry ps1 = fs1.createInclude();
        ps1.setName("j2ee" + this.sep + "*.jar");
        base_classpath.addFileset(fs1);

        getProject().addReference("base.classpath", base_classpath);
    }

    public void initTask() {
        createDir("dist_ejbjars_dir");
        createDir("classes_dir");
    }

    public void compile() {
        Javac j = new Javac();
        j.setProject(getProject());
        Path srcDir = new Path(getProject());
        srcDir.setLocation(new File(getProp("src_dir")));
        j.setSrcdir(srcDir);
        j.setDestdir(new File(getProp("classes_dir")));
        j.setDebug(new Boolean(getProp("opt.javac.debug")).booleanValue());
        j.setOptimize(new Boolean(getProp("opt.javac.optimize")).booleanValue());
        j.setIncludes("**/" + getProcessName() + "MDB.java");
        j.setTaskName("javac");
        j.setSource(getProp("jdkVersion"));
        j.init();
        j.setClasspath((Path) getProject().getReference("compile.classpath"));
        j.execute();
    }

    public void jonasejbjar() {
        EjbJar ejbjar = new EjbJar();
        ejbjar.setProject(getProject());
        ejbjar.setBasejarname(getProcessName() + "MDB");
        ejbjar.setSrcdir(new File(getProp("classes_dir")));
        ejbjar.setDescriptordir(new File(getProp("dist_dir")));
        ejbjar.setDependency("full");

        JonasDeploymentTool jonas = ejbjar.createJonas();
        jonas.setDestdir(new File(getProp("dist_ejbjars_dir")));
        jonas.setJonasroot(new File(getProp("jonas_root")));
        jonas.setProtocols("jrmp,jeremie,iiop");
        jonas.setKeepgenerated(true);
        jonas.setVerbose(new Boolean(getProp("verbose")).booleanValue());
        //jonas.setMappernames(getProp("mapper.names"));
        jonas.setAdditionalargs(getProp("genicargs"));

        PatternSet.NameEntry p1 = ejbjar.createInclude();
        p1.setName("**/*.xml");

        PatternSet.NameEntry p2 = ejbjar.createExclude();
        p2.setName("**/jonas-*.xml");

        PatternSet.NameEntry p3 = ejbjar.createExclude();
        p3.setName("**/build.xml");

        ejbjar.setTaskName("ejbjar");
        ejbjar.execute();
    }

    public void install() {
        Copy c = new Copy();
        c.setProject(getProject());
        c.setTaskName("copy");
        c.setTodir(new File(getProp("jonas_base_ejbjars")));

        FileSet fs1 = new FileSet();
        fs1.setDir(new File(getProp("dist_ejbjars_dir")));
        PatternSet.NameEntry ps1 = fs1.createInclude();
        ps1.setName("*.jar");

        c.addFileset(fs1);
        c.execute();

        adminServer("load");
    }

    public void adminServer(String action) throws BuildException {
        String jarFile = getProcessName() + "MDB.jar";
        try {
            EJBService ejb = (EJBService) ServiceManager.getInstance().getEjbService();
            if (action.equals("load")) {
                ejb.createContainer(jarFile);
            } else {
                ejb.removeContainer(jarFile);
            }
        } catch (Exception e) {
            throw new BuildException("Exception when performing action : " + action + " on file " + jarFile+" : \n"+e,e);
        }
    }
}