/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.optional.junit.AggregateTransformer;
import org.apache.tools.ant.taskdefs.optional.junit.FormatterElement;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTask;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTest;
import org.apache.tools.ant.taskdefs.optional.junit.XMLResultAggregator;
import org.apache.tools.ant.types.Environment;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.PatternSet;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * JunitTestsTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: JunitTestsTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class JunitTestsTask extends StandardDeployTask {

    public JunitTestsTask() {
        setTaskName("junitTests");
    }

    public void execute() {
        log("Starting JunitTestsTask...");
        createDirs();
        junit();
        junitReport();
        log("Finishing JunitTestsTask...");
    }

    public void createDirs() {
        createDir("reports_dir");
    }

    public void junit() {
        try {
            JUnitTask junit = new JUnitTask();
            junit.setProject(getProject());
            junit.setTaskName("junit");
            junit.init();
            junit.setFork(true);
            junit.setShowOutput(true);

            JUnitTask.SummaryAttribute sum = new JUnitTask.SummaryAttribute();
            sum.setValue("on");
            junit.setPrintsummary(sum);

            junit.setHaltonfailure(false);
            junit.setHaltonerror(false);

            Path classpath = new Path(getProject());
            Path.PathElement jbConf = classpath.createPathElement();
            jbConf.setPath(getProp("jonas_base_conf"));
            Path.PathElement client = classpath.createPathElement();
            client.setPath(getProp("client.jar"));
            Path.PathElement bpel = classpath.createPathElement();
            bpel.setPath(getProp("bpel.jar"));
            Path.PathElement cp = classpath.createPathElement();
            cp.setPath(getProp("cp"));
            /*
            FileSet jrlib = new FileSet();
            jrlib.setDir(new File(getProp("jonas_root_lib")));
            PatternSet.NameEntry jrlibps = jrlib.createInclude();
            jrlibps.setName("**" + this.sep + "*.jar");
            classpath.addFileset(jrlib);
             */
            Path.PathElement pe5 = classpath.createPathElement();
            String mail = getProp("jonas_root_lib") + this.sep + "commons" + this.sep + "j2ee" + this.sep + "mail.jar";
            pe5.setLocation(new File(mail));
            Path.PathElement pe6 = classpath.createPathElement();
            String activation = getProp("jonas_root_lib") + this.sep + "commons" + this.sep + "j2ee" + this.sep + "activation.jar";
            pe6.setLocation(new File(activation));
            Path.PathElement pe8 = classpath.createPathElement();
            String junitJar = getProp("lib_dir") + this.sep + "junit" + this.sep + "junit.jar";
            pe8.setLocation(new File(junitJar));

            FileSet jars = new FileSet();
            jars.setDir(new File(getProp("build_jars_dir")));
            PatternSet.NameEntry jarsps = jars.createInclude();
            jarsps.setName("*.jar");
            classpath.addFileset(jars);

            FileSet unitSamplesJar = new FileSet();
            unitSamplesJar.setDir(new File(getProp("build_clientJars_dir")));
            PatternSet.NameEntry unitSamplesJarPs = unitSamplesJar.createInclude();
            unitSamplesJarPs.setName("unitSamples.jar");
            classpath.addFileset(unitSamplesJar);

            FormatterElement form = new FormatterElement();
            FormatterElement.TypeAttribute typeAttr = new FormatterElement.TypeAttribute();
            typeAttr.setValue("xml");
            form.setType(typeAttr);
            junit.addFormatter(form);

            Vector tests = new Vector();
            tests.addElement("unitSamples.TestSamples");
            tests.addElement("unitCopy.TestCopy");
            tests.addElement("unitExpr.TestExpr");
            tests.addElement("unitInvoke.TestInvoke");
            tests.addElement("unitScope.TestScope");
            tests.addElement("unitFlow.TestFlow");
            tests.addElement("unitSwitch.TestSwitch");
            tests.addElement("unitWhile.TestWhile");
            tests.addElement("unitCondition.TestCondition");
            tests.addElement("unitPick.TestPick");
            tests.addElement("unitCopyJAXRPC.TestCopyJAXRPC");
            tests.addElement("unitCopyJAXRPC2.TestCopyJAXRPC2");
            tests.addElement("unitCopyJAXRPC3.TestCopyJAXRPC3");
            tests.addElement("unitCopyJAXRPC4.TestCopyJAXRPC4");
            tests.addElement("unitWait.TestWait");
            tests.addElement("unitCS.TestCS");
            tests.addElement("unitEvent.TestEvent");
            tests.addElement("unitSourceTarget.TestSourceTarget");
            tests.addElement("unitNamespace.TestNamespace");
            tests.addElement("unitFault.TestFault");
            tests.addElement("unitInvokeFault.TestInvokeFault");
            tests.addElement("unitPartnerLink.TestPartnerLink");
            tests.addElement("unitWsdl.TestWsdl");
            tests.addElement("unitXPath.TestXPath");
            tests.addElement("unitXPath2.TestXPath2");
            tests.addElement("unitGenerator.TestGenerator");
            tests.addElement("unitBPELJCode.TestBPELJCode");
            tests.addElement("unitForEach.TestForEach");
            tests.addElement("unitRepeatUntil.TestRepeatUntil");
            tests.addElement("unitIf.TestIf");
            tests.addElement("unitWait2_0.TestWait2_0");
            tests.addElement("unitWhile2_0.TestWhile2_0");
            tests.addElement("unitBPELJFault.TestBPELJFault");
            tests.addElement("unitBPELJValue.TestBPELJValue");
            tests.addElement("unitBPELJPartnerLink.TestBPELJPartnerLink");

            String pack = BPELConstants.TESTS_UNIT_PACK.replace(File.separatorChar,'.');
            for (int i=0;i<tests.size();i++) {
                JUnitTest test = new JUnitTest();
                test.setFork(true);
                test.setName(pack+"."+(String)tests.get(i));
                test.setTodir(new File(getProp("reports_dir")));
                junit.addTest(test);
            }

            Environment.Variable bh = new Environment.Variable();
            bh.setKey(BPELConstants.BPEL_HOME_PROPERTY);
            bh.setValue(BPELConstants.BPEL_HOME);
            junit.addEnv(bh);
            junit.addConfiguredSysproperty(bh);

            Environment.Variable jaas = new Environment.Variable();
            jaas.setKey("java.security.auth.login.config");
            jaas.setValue(getProp("jaas_config"));
            junit.addEnv(jaas);
            junit.addConfiguredSysproperty(jaas);

            Environment.Variable corba = new Environment.Variable();
            corba.setKey("javax.rmi.CORBA.PortableRemoteObjectClass");
            corba.setValue("org.objectweb.carol.rmi.multi.MultiPRODelegate");
            junit.addEnv(corba);
            junit.addConfiguredSysproperty(corba);

            Environment.Variable naming = new Environment.Variable();
            naming.setKey("java.naming.factory.initial");
            naming.setValue("org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory");
            junit.addEnv(naming);
            junit.addConfiguredSysproperty(naming);

            Environment.Variable otsOrb = new Environment.Variable();
            otsOrb.setKey("org.omg.PortableInterceptor.ORBInitializerClass.org.objectweb.jonas_tm.ots.OTSORBInitializer");
            otsOrb.setValue("");
            junit.addEnv(otsOrb);
            junit.addConfiguredSysproperty(otsOrb);

            Environment.Variable secuInit = new Environment.Variable();
            secuInit.setKey("org.omg.PortableInterceptor.ORBInitializerClass.org.objectweb.jonas.security.interceptors.iiop.SecurityInitializer");
            secuInit.setValue("");
            junit.addEnv(secuInit);
            junit.addConfiguredSysproperty(secuInit);

            Path junitCP = junit.createClasspath();
            junitCP.add(classpath);

            junit.execute();
        } catch (Exception e) {
            throw new BuildException("There was an exception when launching junit tests",e);
        }
    }

    public void junitReport() {
        File reportsDir = new File(getProp("reports_dir"));
        XMLResultAggregator report = new XMLResultAggregator();
        report.setProject(getProject());
        report.init();
        report.setTodir(reportsDir);

        FileSet fs1 = new FileSet();
        fs1.setDir(reportsDir);
        PatternSet.NameEntry ps = fs1.createInclude();
        ps.setName("TEST-*.xml");
        report.addFileSet(fs1);

        AggregateTransformer rep = report.createReport();
        AggregateTransformer.Format format = new AggregateTransformer.Format();
        format.setValue("frames");
        rep.setFormat(format);
        rep.setTodir(reportsDir);

        report.setTaskName("junitReport");
        report.execute();
    }
}