/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.axis.utils.ClassUtils;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Copy;
import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.engine.FileMap;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.util.ClassLoaderMap;
import org.objectweb.orchestra.util.Util;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;



/**
 * LoadProcessTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: LoadProcessWSTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class LoadProcessWSTask extends StandardDeployTask {

    public LoadProcessWSTask() {
        setTaskName("loadProcessWS");
    }

    public void execute() {
        log("Starting LoadProcessWSTask...");
        undeployService();
        loadProcessWSInAxisServlet();
        log("Finishing LoadProcessWSTask...");
    }

    public void undeployService() {
        FileMap.removeAll(getProcessName());
    }

    public void loadProcessWSInAxisServlet() {
        String wsdd = getProp("build_wsdd_dir") + this.sep + getProcessName()+"deploy.wsdd";
        ClassLoader classloader = null;
        try {
            classloader = ClassLoaderMap.getCL(getProcessName());
        } catch (BPELUtilException e) {
            throw new BuildException("BPELUtilException in LoadProcessWSTask loadProcessWSInAxisServlet : \n"+e,e);
        }
        callAdminClientTask(new File(wsdd), classloader);
        if (BPELEngine.isJbi()) {
            loadJbiWS(classloader);
        }
    }

    public void callAdminClientTask(File file, ClassLoader classloader) {
        ClassLoader old = ClassUtils.getDefaultClassLoader();
        try {

            ClassUtils.setDefaultClassLoader(classloader);

            loadArrayClasses(file, classloader);

            String l = "-lhttp://";
            l += getProp("bpel.host") + ":" + getProp("bpel.port");
            l += "/axis/servlet/AxisServlet";

            Method m = null;
            Class cl = null;
            URL[] urlTab = new URL[1];
            String url = Util.getFilePrefix();
            urlTab[0] = new URL(url+getProp("axis-ant"));
            URLClassLoader classL = new URLClassLoader(urlTab,classloader);
            cl = Class.forName("org.apache.axis.tools.ant.axis.AdminClientTask",true, classL);
            Object adminClientTask = cl.newInstance();
            m = cl.getMethod("setProject", new Class[]{Project.class});
            m.invoke(adminClientTask,new Object[]{getProject()});
            m = cl.getMethod("init", (Class[])null);
            m.invoke(adminClientTask,(Object[])null);
            m = cl.getMethod("setUrl", new Class[]{java.lang.String.class});
            m.invoke(adminClientTask,new Object[]{l});
            m = cl.getMethod("setXmlFile", new Class[]{java.io.File.class});
            m.invoke(adminClientTask,new Object[]{file});
            m = cl.getMethod("setTaskName", new Class[]{java.lang.String.class});
            m.invoke(adminClientTask,new Object[]{"AdminClientTask"});
            m = cl.getMethod("execute", (Class[])null);
            m.invoke(adminClientTask,(Object[])null);
        } catch (Exception e) {
            throw new BuildException("Exception in LoadProcessWSTask CallAdminClientTask : \n"+e,e);
        } finally {
            ClassUtils.setDefaultClassLoader(old);
        }
    }

    //Bypass an axis bug.
    //When arrays are used, we are falling down in a problem of classloading.
    //There is a CNFE in org.apache.axis.utils.ClassUtils.loadClass(String _className)
    //In fact with our ClassLoader hierachy, processes classes are in a separate ClassLoader we are giving
    //to ClassUtils by calling the static method setDefaultClassLoader (see above in method callAdminClientTask)
    //In this case, in the ClassUtils.loadClass method, a defaultClassLoader.loadClass(className) is performed
    //This is different from doing Class.forName. If we do not perform a Class.forName before the defaultClassLoader.loadClass
    //we get a CNFE
    public void loadArrayClasses(File wsddFile, ClassLoader processClassLoader) {
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = dbf.newDocumentBuilder();
            Document wsdd = docBuilder.parse(wsddFile);
            NodeList arrays = wsdd.getElementsByTagName("arrayMapping");
            int length = arrays.getLength();
            for (int i = 0;i<length ; i++) {
                Node arrayMapping = arrays.item(i);
                NamedNodeMap attributes = arrayMapping.getAttributes();
                String type = attributes.getNamedItem("type").getNodeValue();
                type = type.substring(type.indexOf(":")+1);
                //each time there is a [], it means there is a dimension more to the array
                //each time we add a [ at the beginning of the internal class representation
                String internalClass = "L" + type.substring(0,type.indexOf("[")) + ";";
                int index = type.indexOf("[]");
                while (index != -1) {
                    internalClass = "[" + internalClass;
                    type = type.substring(index+2);
                    index = type.indexOf("[]");
                }
                log("Preloading class : "+internalClass+"...");
                Class.forName(internalClass,true, processClassLoader);
                log("Preloading of class : "+internalClass+" successfull");
            }

        } catch (Exception e) {
            throw new BuildException("Exception in LoadProcessWSTask loadArrayClasses when building the DOM Document : \n"+e,e);
        }
    }

    public void loadJbiWS(ClassLoader classL) {
        String destDir = getProp("wsddExtJbi")+this.sep+getProcessName();
        createDirName(destDir);

        String wsddToLoad = getProp("wsddToLoad");
        if (wsddToLoad != null) {
            String[] wsdds = wsddToLoad.split(";");
            if (wsdds.length == 1 && wsdds[0].equals(wsddToLoad)) {
                //it means the property is empty => nothing to do
            } else {
                for (int i=0; i<wsdds.length;i++) {
                    String wsddFile = wsdds[i];
                    log("loadProcessWS, loading wsddFile : "+wsddFile);
                    callAdminClientTask(new File(wsddFile), classL);
                    copyWsdd(wsddFile,i,destDir);
                }
            }
        }

        File wsddToDeployDir = new File(destDir);
        File[] ext = wsddToDeployDir.listFiles();
        for (int i=0; i<ext.length;i++) {
            File f = ext[i];
            if (f.getName().endsWith("_deploy.wsdd")) {
                log("loadProcessWS, loading wsddFile : "+f);
                callAdminClientTask(f, classL);
            }
        }
    }

    public void copyWsdd(String wsddFile, int nb, String destDir) {
        File deploywsddSource = new File(wsddFile);
        String sourceDir = deploywsddSource.getParent();

        File undeployWsddSource = new File(sourceDir+this.sep+"undeploy.wsdd");

        Copy c = new Copy();
        c.setProject(getProject());
        c.setTaskName("copyExtDeployWsdd");
        c.setTofile(new File(destDir+this.sep+Integer.toString(nb)+"_"+deploywsddSource.getName()));
        c.setFile(deploywsddSource);
        c.execute();

        Copy c2 = new Copy();
        c2.setProject(getProject());
        c2.setTaskName("copyExtUndeployWsdd");
        c2.setTofile(new File(destDir+this.sep+Integer.toString(nb)+"_"+undeployWsddSource.getName()));
        c2.setFile(undeployWsddSource);
        c2.execute();
    }
}