/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.Jar;
import org.apache.tools.ant.taskdefs.Javac;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.generator.OutServicesAnalyser;



/**
 * DeployProcessTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: OutServicesTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class OutServicesTask extends StandardDeployTask {

    private String[] outServices = null;

    public String[] getOutServices() {
        return this.outServices;
    }

    public void setOutServices(String[] outServices) {
        this.outServices = outServices;
    }

    public OutServicesTask(String[] outServices) {
        setTaskName("outServicesProcess");
        setOutServices(outServices);
    }

    public void execute() {
        log("Starting OutServicesTask...");
        if (this.outServices != null && this.outServices.length>0) {
            createDirs();
            wsdl2JavaAll();
            compileToTmpJXPath();
            outServicesAnalyser();
        }
        log("Finishing OutServicesTask...");
    }

    public void outServicesAnalyser() {
        try {
            OutServicesAnalyser.analyse(getProcessName(),this.outServices);
        } catch (BPELGeneratorException e) {
            throw new BuildException("BPELGeneratorException in OutServicesTask outServicesAnalyser : \n"+e,e);
        }
    }

    private void createDirs() {
        String st = getProp("build_JXPath_dir") + this.sep + getProcessName();
        createDirName(st);
        String st2 = getProp("generate_wsdl_dir") + this.sep + getProcessName();
        log("OutServicesTask, st2 = "+st2);
        createDirName(st2);
    }

    private void compileToTmpJXPath() {
        Javac j = new Javac();
        j.setProject(getProject());
        Path srcDir = new Path(getProject());
        srcDir.setLocation(new File(getProp("generate_wsdl_dir") + this.sep + getProcessName()));
        j.setSrcdir(srcDir);
        j.setDestdir(new File(getProp("build_JXPath_dir") + this.sep + getProcessName()));
        Path compileCP = new Path(getProject());
        compileCP.setPath(getProp("generate_wsdl_dir") + this.sep + getProcessName() + this.sep + "**" + this.sep + "*.java");
        j.setSourcepath(compileCP);
        j.setTaskName("javac");
        j.setSource(getProp("jdkVersion"));
        j.init();
        j.setClasspath((Path) getProject().getReference("compile.classpath"));
        j.execute();

        Jar jar = new Jar();
        jar.setTaskName("jar");
        jar.setProject(getProject());
        String st = getProp("build_JXPath_dir") + this.sep + getProcessName() + this.sep + getProcessName()+".jar";
        File f = new File(st);
        jar.setDestFile(f);

        FileSet fs1 = new FileSet();
        fs1.setDir(new File(getProp("build_JXPath_dir") + this.sep + getProcessName()));
        fs1.setIncludes("**");
        fs1.setExcludes("**/*.jar");
        jar.addFileset(fs1);

        jar.execute();
    }

    public void wsdl2JavaAll() {
        boolean serverSide = false;
        String output = getProp("generate_wsdl_dir") + this.sep + getProcessName();
        for (int i=0; i<this.outServices.length;i++) {
            String wsdlFile = this.outServices[i];
            wsdl2Java(wsdlFile, serverSide,output);
            removeWSDD();
        }
    }

    public void wsdl2Java(String wsdlFile, boolean serverSide, String output) {
        Wsdl2JavaProcessTask wsdl2java = new Wsdl2JavaProcessTask(serverSide, wsdlFile,output);
        wsdl2java.setProject(getProject());
        wsdl2java.setProcessName(getProcessName());
        wsdl2java.execute();
    }

    public void removeWSDD() {
        String out = getProp("generate_wsdl_dir") + this.sep + getProcessName();
        deleteIncludes(out,"**/*.wsdd");
    }

}