/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Delete;
import org.apache.tools.ant.taskdefs.Echo;
import org.apache.tools.ant.taskdefs.Mkdir;
import org.apache.tools.ant.taskdefs.Property;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;

/**
 * StandardDeployTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: StandardDeployTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class StandardDeployTask extends Task {

    public String sep = File.separator;

    private String processName = null;

    public String getProcessName() {
        return this.processName;
    }

    public void setProcessName(String processName) {
        this.processName = processName;
    }

    //utility methods

    public void addTask(Vector tasks, StandardDeployTask task) {
        if (task != null) {
            tasks.addElement(task);
        }
    }

    public String getProp(String prop) {
        return getProject().getProperty(prop);
    }

    public File getProcessDir(String dirProperty) {
        String s = getProject().getProperty(dirProperty) + this.sep + getProcessName();
        return new File(s);
    }

    public File getJarFile(String dirProperty) {
        String s = getProject().getProperty(dirProperty) + this.sep + getProcessName() + ".jar";
        return new File(s);
    }

    public File getTxtFile(String dirProperty) {
        String s = getProject().getProperty(dirProperty) + this.sep + getProcessName() + ".txt";
        return new File(s);
    }

    public void createDirName(String dirName) {
        Mkdir dir = new Mkdir();
        dir.setTaskName("mkdir");
        dir.setDir(new File(dirName));
        dir.setProject(getProject());
        dir.execute();
    }

    public void createDir(String dirProp) {
        Mkdir dir = new Mkdir();
        dir.setTaskName("mkdir");
        dir.setDir(new File(getProp(dirProp)));
        dir.setProject(getProject());
        dir.execute();
    }

    public Property createProperty(String name, String value) {
        Property p = new Property();
        p.setName(name);
        p.setValue(value);
        p.setProject(getProject());
        p.execute();
        return p;
    }

    public Property createEnvProperty(String envName) {
        Property p = new Property();
        p.setEnvironment(envName);
        p.setProject(getProject());
        p.execute();
        return p;
    }

    public void createPropertyFile(String fileName) {
        Property p = new Property();
        p.setFile(new File(fileName));
        p.setProject(getProject());
        p.execute();
    }

    public File getFile(String propDir, String fileName) {
        String s = getProject().getProperty(propDir) + this.sep + fileName;
        return new File(s);
    }

    public void deleteProcessDir(String dir) {
        deleteDir(getProcessDir(dir));
    }

    public void deleteDir(File dir) {
        try {
            Delete d = new Delete();
            d.setTaskName("delete");
            d.setDir(dir);
            d.setProject(getProject());
            d.execute();
        } catch (Exception e) {
            System.err.println("Unable to delete directory : " + dir);
            System.err.println(e.getMessage());
        }
    }

    public void deleteDir(String dir) {
        deleteDir(new File(dir));
    }

    public void deleteFile(File f) {
        try {
            Delete d = new Delete();
            d.setTaskName("delete");
            d.setFile(f);
            d.setProject(getProject());
            d.execute();
        } catch (Exception e) {
            System.err.println("Unable to delete file : " + f);
        }
    }

    public void deleteFile(String file) {
        deleteFile(new File(file));
    }

    public void echoProperty(String prop) {
        Echo e = new Echo();
        e.setMessage(prop + " = " + getProject().getProperty(prop));
        e.setProject(getProject());
        e.setTaskName("echo");
        e.execute();
    }

    public Path removeBootStrapJar() {
        Path cp = new Path(getProject(), getProp("cp"));
        Path result = new Path(getProject());
        StringTokenizer st = new StringTokenizer(cp.toString(), File.pathSeparator);
        while (st.hasMoreTokens()) {
            String tmp = st.nextToken();
            if (!tmp.endsWith("bootstrap.jar")) {
                result.add(new Path(getProject(), tmp));
            }
        }
        return result;
    }

    public void deleteIncludes(String dirName, String includes) {
        File dir = new File(dirName);
        Delete d = new Delete();
        d.setTaskName("delete");
        FileSet fs = new FileSet();
        fs.setDir(dir);
        fs.setIncludes(includes);
        d.addFileset(fs);
        d.setProject(getProject());
        d.execute();
    }
}