/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tree;

/**
 * BPELJCode.java represents the <code>bpelj:code</code> statement in the java tree.
 * A <code>bpelj:code</code> statement is a java statement block that does not return a value.
 *
 * @author Jean-Noel Rouvignac
 * @version $Id: BPELJCode.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class BPELJCode extends Statement {

    /**
     * The snippet corresponding to this object
     */
    private JavaSnippet snippet;

    /**
     * Constructor for the <code>bpelj:code</code> statement.
     * The java code of this statement is encapsulated into a static method of the java
     * class representing the BPELJ process. This class is generated by the BPEL generator.
     * @param nb the statement number in the process
     * @param snippet the Java snippet associated to this BPELJCode
     */
    public BPELJCode(Integer nb, JavaSnippet snippet) {
        super();
        this.statementNb = nb;
        this.snippet = snippet;
    }

    /*
     * (non-Javadoc)
     * @see org.objectweb.orchestra.tree.Statement#print()
     */
    public void print() {
        System.out.println(" BPELJCode :");
        System.out.println("   snippet into " + this.snippet.getClassName() + "." + this.snippet.getMethodName() + "()");
    }

    /*
     * (non-Javadoc)
     * @see org.objectweb.orchestra.tree.Statement#getType()
     */
    public String getType() {
        return "BPELJCode";
    }

    public JavaSnippet getJavaSnippet() {
        return this.snippet;
    }

}
