/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tree;

/**
 * BPELJValue.java represents the <code>bpelj:value</code> statement in the java tree.
 * A <code>bpelj:value</code> statement is a java statement that is used to initialize a
 * BPEL variable. It can be a "block" or inline snippet.
 *
 * @author Jean-Noel Rouvignac
 * @version $Id: BPELJValue.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class BPELJValue extends Statement {

    /**
     * The snippet corresponding to this object
     */
    private JavaSnippet snippet;

    /**
     * Whether or not this statement is a "block" statement (it must use "return"
     * Java keyword to return a value) or an inline one (the expression is the
     * returned value).
     */
    private boolean isBlock;

    /**
     * Constructor for the <code>bpelj:value</code> statement.
     * The java code of this statement is encapsulated into a static method of the java
     * class representing the BPELJ process. This class is generated by the BPEL generator.
     * @param nb the statement number in the process
     * @param snippet the Java snippet associated to this BPELJValue
     * @param isBlock Whether or not this statement is a "block" statement
     */
    public BPELJValue(Integer nb, JavaSnippet snippet, boolean isBlock) {
        super();
        this.statementNb = nb;
        this.snippet = snippet;
        this.isBlock = isBlock;
    }

    /*
     * (non-Javadoc)
     * @see org.objectweb.orchestra.tree.Statement#print()
     */
    public void print() {
        System.out.println(" BPELJValue :");
        System.out.println("   snippet into " + this.snippet.getClassName() + "." + this.snippet.getMethodName() + "()");
    }

    /*
     * (non-Javadoc)
     * @see org.objectweb.orchestra.tree.Statement#getType()
     */
    public String getType() {
        return "BPELJValue";
    }

    public JavaSnippet getJavaSnippet() {
        return this.snippet;
    }

    public boolean isBlock() {
        return this.isBlock;
    }

}
