/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tree;

/**
 * BPELProcess.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPELProcess.java 8 2006-10-24 13:52:21Z lejeuneg $
 *
 */
public class BPELProcess {

    private String processName = null;

    private Statement child = null;
    private int totalStatementNb = -1;

    public BPELProcess(String name) {
        this.processName = name;
    }

    public String getName() {
        return this.processName;
    }

    public Statement getChild() {
        return this.child;
    }

    public void addChild(Statement s) {
        this.child = s;
    }

    public void setTotalStatementNb(int nb) {
        this.totalStatementNb = nb;
    }

    public int getTotalStatementNb() {
        return this.totalStatementNb;
    }

    public void print() {
        System.out.println("Process name : " + this.processName);
        this.child.print();
        System.out.println("End Process");
    }

    public Statement findStatement(int statementNb) {
        if (statementNb >= 1) {
            if (this.child != null) {
                return findStatement(statementNb, this.child);
            }
            System.out.println("BPELProcess.findStatement, statementNb = "+statementNb+", child = null");
        }
        return null;
    }

    private Statement findStatement(int statementNb, Statement s) {
        Statement result = null;
        if (s.getNb().intValue() == statementNb) {
            result = s;
        } else {
            Statement[] tab = s.getChilds();
            int taille = s.getChildNb();
            int i = 0;
            while ((i + 1) < taille && tab[i + 1].getNb().intValue() <= statementNb) {
                i++;
            }

            while (i < taille && result == null) {
                result = findStatement(statementNb, tab[i]);
                i++;
            }
        }
        return result;
    }

    public Statement[] getTabStatement() {
        Statement[] result = new Statement[this.totalStatementNb+1];
        if (this.totalStatementNb >= 1) {
            fillAllStatements(result, this.child);
        }
        return result;
    }

    private void fillAllStatements(Statement[] tab, Statement s) {
        int nb = s.getNb().intValue();
        tab[nb] = s;
        Statement[] children = s.getChilds();
        int taille = s.getChildNb();
        for (int i=0;i<taille; i++) {
            fillAllStatements(tab, children[i]);
        }
    }

    public String getSuppressJoinFailure(Statement s) {
        if (s != null) {
            if (s.isActivity()) {
                Activity a = (Activity) s;
                String sjf =a.getSuppressJoinFailure();
                if (sjf!=null) {
                    return sjf;
                }
                return getSuppressJoinFailure(s.getFather());
            }
            return getSuppressJoinFailure(s.getFather());
        }
        return null;
    }

    public int findStatementOfType (String type, Statement s) {
        //find the first statement of the given type in all father of the given statement
        int result = -1;
        while (result == -1 && s != null) {
            if (s.getType().equals(type)) {
                result = s.getNb().intValue();
            } else {
                s = s.getFather();
            }
        }
        return result;
    }
}