/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tree;

/**
 * ExpressionImpl.java represents an expression of a BPEL process.
 * This abstract class must be subclassed in order to support other <code>expressionLanguage</code>s
 *
 * @author Jean-Noel Rouvignac
 * @version $Id: ExpressionImpl.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public abstract class ExpressionImpl {

    /**
     * the text of the expression as it appears inside the BPEL process file.
     */
    private String expression;

    /**
     * Constructor for an ExpressionImpl object.
     * @param expression the literal expression as it appears inside the BPEL process file.
     */
    protected ExpressionImpl(String expression) {
        this.expression = expression;
    }

    /**
     * Returns the literal expression as it appears inside the BPEL process file.
     * @return the literal expression as it appears inside the BPEL process file.
     */
    public String getExpression() {
        return this.expression;
    }

}
