/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tree;

import java.io.Serializable;

/**
 * Statement.java The <code>Statement</code> class represents a statement in a BPEL process file.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: Statement.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public abstract class Statement implements Serializable {

    /**
     * The statement number of this statement in the BPEL process file
     */
    protected java.lang.Integer statementNb;
    /**
     * The number of child of this statement.
     */
    protected int childNb;
    /**
     * The array containing the childs of this statement.
     * At the end of the initialization of this process,
     * this array should be full and <code>this.cmpt</code> should
     * be equal to <code>this.childNb</code>.
     */
    protected Statement[] childs = null;
    /**
     * The next statement of this statement in the BPEL process file
     */
    protected Statement next = null;
    /**
     * The father statement of this statement in the BPEL process file
     */
    protected Statement father = null;
    /**
     * The current number of childs that have already been added to this statement.
     * It holds the index of the first free position in the childs array.
     */
    protected int cmpt = 0;

    public java.lang.Integer getNb() {
        return this.statementNb;
    }

    public void setChildNb(java.lang.Integer nb) {
        this.childs = new Statement[nb.intValue()];
        this.childNb = nb.intValue();
    }

    public void addChild(Statement s) {
        this.childs[this.cmpt++] = s;
    }

    public void setNext(Statement s) {
        this.next = s;
    }

    public Statement getNext() {
        return this.next;
    }

    public void setFather(Statement s) {
        this.father = s;
    }

    public Statement getFather() {
        return this.father;
    }

    public Statement[] getChilds() {
        return this.childs;
    }

    public int getChildNb() {
        return this.cmpt;
    }

    public Statement getChild(java.lang.Integer i) {
        return this.childs[i.intValue()];
    }

    public boolean isNull(String s) {
        if (s == null) {
            return true;
        }
        return s.equals("null");
    }

    public abstract void print();

    public abstract String getType();

    public boolean isActivity() {
        return false;
    }
}
