/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.util;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * BPELClassLoader.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPELClassLoader.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class BPELClassLoader extends URLClassLoader {

    private String name = null;

    public BPELClassLoader (String name,URL[] urls, ClassLoader parent) {
        super(urls, parent);
        this.name = name;
    }

    public BPELClassLoader (String name,URL[] urls) {
        super(urls);
        this.name = name;
    }

    public void addURL(URL url) {
        if (url != null) {
            super.addURL(url);
        }
    }

    public void addURLs(URL[] urls) {
        if (urls != null) {
            for (int i = 0; i < urls.length; i++) {
                if (urls[i] != null) {
                    super.addURL(urls[i]);
                }
            }
        }
    }

    public void printURLs() {
        System.out.println(this.name  + " ClassLoader :");
        URL[] urls = super.getURLs();
        for (int i = 0; i < urls.length; i++) {
            System.out.println("url=" + (new File(urls[i].getFile())).getAbsolutePath());
        }
        // display parent classloader
        if (getParent() != null && getParent() instanceof BPELClassLoader) {
            System.out.println("parent :");
            ((BPELClassLoader) getParent()).printURLs();
        }
    }

    public String getClassPath() {
        String cp = "";
        // add parent classpath before
        if (getParent() != null && getParent() instanceof BPELClassLoader) {
            cp += ((BPELClassLoader) getParent()).getClassPath();
        }
        URL[] urls = super.getURLs();
        for (int i = 0; i < urls.length; i++) {
            cp = cp + File.pathSeparator + (new File(urls[i].getFile())).getAbsolutePath();
        }
        return cp;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(this.getClass().getName());
        sb.append("[");
        sb.append(this.name);
        sb.append(", urls=");
        URL[] urls = getURLs();
        for (int u = 0; u < urls.length; u++) {
            sb.append(urls[u]);
            if (u != urls.length - 1) {
                sb.append(";");
            }
        }
        sb.append("]");
        return sb.toString();
    }
}
