/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.OutputStream;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;
import javax.xml.namespace.QName;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.axis.wsdl.symbolTable.Element;
import org.apache.axis.wsdl.symbolTable.SymbolTable;
import org.apache.axis.wsdl.symbolTable.Type;
import org.apache.axis.wsdl.toJava.Emitter;
import org.apache.axis.wsdl.toJava.Utils;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Delete;
import org.objectweb.orchestra.engine.BPELMessage;
import org.objectweb.orchestra.engine.MessageStructure;
import org.objectweb.orchestra.engine.PartStructure;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.ibm.wsdl.factory.WSDLFactoryImpl;


/**
 * Util.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: Util.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class Util {

    private static long id = 0;


    /**
     * Return an array of byte from a given object
     *
     * @param obj
     *            the object from which we must extract the bytes.
     * @return the byte[] from an object
     */
    public static byte[] getBytesFromObject(Object obj) {
        if (obj == null) { return null; }

        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        ObjectOutputStream oos = null;
        byte[] bytes = null;

        try {
            oos = new ObjectOutputStream(baos);
            oos.writeObject(obj);
            bytes = baos.toByteArray();
        } catch (IOException e) {
            System.out.println("Util.getBytesFromObject, catch IOE1 = "+e);
            e.printStackTrace();
            //Can't tranform
            return null;
        } finally {
            try {
                oos.close();
                baos.close();
            } catch (IOException e) {
                System.out.println("Util.getBytesFromObject, catch IOE2 = "+e);
                e.printStackTrace();
                //Not a problem
            }
        }
        return bytes;
    }

    // TODO delete?
    protected byte[] getLengthBytesFromObject(Object obj) {
        if (obj == null) return null;
        byte[] b = getBytesFromObject(obj);
        int l = b.length;
        byte[] inter = new byte[l + 1];
        inter[0] = (new Integer(l)).byteValue();
        for (int i = 0; i < b.length; i++) {
            inter[i + 1] = b[i];
        }
        return inter;
    }

    public static Object getObjectFromBytes(byte[] bytes) {
        //Declaration
        ByteArrayInputStream bis = null;
        ObjectInputStream ois = null;
        Object obj = null;

        if (bytes == null) { return null; }

        bis = new ByteArrayInputStream(bytes);
        try {
            ois = new ObjectInputStream(bis);
            obj = ois.readObject();

        } catch (ClassNotFoundException cfe) {
            //obj is null
        } catch (OptionalDataException ode) {
            //obj is null
        } catch (IOException ioe) {
            //obj is null
        } finally {
            try {
                bis.close();
                ois.close();
            } catch (IOException e) {
                //Ignore it
            }
        }
        return obj;
    }

    public static Object correlationQuery(String query, Object correlationSet) throws Exception {
        char[] c = query.toCharArray();
        if (c.length == 0) return correlationSet;
        if (c[0] == '/') {
            String s = query.substring(1, query.length());
            return correlationQuery(s, correlationSet);
        }
        int i = 0;
        while (i < c.length && c[i] != '/') {
            i++;
        }
        char[] t = new char[i];
        for (int j = 0; j < i; j++) {
            t[j] = c[i];
        }

        if (!Character.isUpperCase(t[0])) t[0] = Character.toUpperCase(t[0]);

        Class cl = correlationSet.getClass();
        Method method = cl.getMethod("get" + (new String(t)), (Class[])null);
        Object r = method.invoke(correlationSet, (Object[])null);
        return correlationQuery(query.substring(i + 1, query.length()), r);

    }

    public static Object getPartValue(Object obj, String partName) throws BPELUtilException {
        Object r = null;
        try {
            char[] c = partName.toCharArray();

            if (!Character.isUpperCase(c[0])) c[0] = Character.toUpperCase(c[1]);
            Class cl = obj.getClass();
            Method method = cl.getDeclaredMethod("get" + (new String(c)), (Class[])null);
            r = method.invoke(obj, (Object[])null);
        } catch (IllegalAccessException e) {
            throw new BPELUtilException("IllegalAccessException Util getPartValue : \n" , e);
        } catch (NoSuchMethodException e) {
            throw new BPELUtilException("NoSuchMethodException Util getPartValue : \n" , e);
        } catch (InvocationTargetException e) {
            throw new BPELUtilException("InvocationTargetException Util getPartValue :\n" , e);
        }
        return r;
    }

    /**
     * Deserializes an Object from a byte array. The process name helps identify which class loader must be used.
     * @param varValue the byte array representing the serialized variable value
     * @param processName the name of the process for which the method is called
     * @return Object the deserialized Object
     * @throws BPELUtilException if any exception occurs during the deserialization
     */
    public static synchronized Object getValueObject(byte[] varValue,String processName) throws BPELUtilException {
        Object result = null;
        if (varValue != null) {
            try {
                ByteArrayInputStream bais = new ByteArrayInputStream(varValue);
                ObjectInputStream ois = new Specialized_OIS(bais,processName);
                result = ois.readObject();
            }  catch (Exception e) {
                //Thread.dumpStack();
                throw new BPELUtilException("Exception Util getValueObject(byte[] varValue,String processName) : processName = "+processName+", varValue = "+varValue+"\n" , e);
            }
        }
        return result;
    }

    public static Object getValueObject(Object value,String processName) throws BPELUtilException {
        return getValueObject((byte[]) value,processName);
    }

    public static Object putValueObject(Object value) throws BPELUtilException {
        Object result = null;
        try {
            ByteArrayOutputStream x = new ByteArrayOutputStream();
            new ObjectOutputStream(x).writeObject(value);
            result = x.toByteArray();
        } catch (IOException e) {
            throw new BPELUtilException("IOException Util putValueObject :\n" , e);
        }
        return result;
    }

    public static QName buildQname(String expr) {
        //expr is like : ns,localpart
        QName qn = new QName(BPELString.getNS(expr), BPELString.removeNS(expr));
        return qn;
    }

    public static QName buildQname(String expr, org.objectweb.orchestra.generator.NamespaceDef p) {
        //expr is like : prefix:localpart
        String prefix = BPELString.getPrefix(expr);
        String localPart = BPELString.removePrefix(expr);
        String namespaceURI = null;
        if ("".equals(prefix)) {
            //get the targetNamespace
            namespaceURI = p.getTargetNamespace();
        } else {
            //get the nameSpaceURI corresponding
            namespaceURI = p.getNS(prefix);
        }
        return new QName(namespaceURI, localPart, prefix);
    }

    public static QName buildProperty(String prop) {
        QName qn = null;
        if (prop.indexOf(BPELConstants.NS_SEPARATOR) == -1) {
            //expr is like : prefix:localpart
            String localPart = BPELString.removePrefix(prop);
            // Something to do about BPEL 2.0?
            String namespaceURI = BPELConstants.NS_BPEL_V1_1;
            qn = new QName(namespaceURI, localPart);
        } else {
            //else the property has been converted at generation time and is
            // like : ns,localpart
            qn = buildQname(prop);
        }
        return qn;
    }

    public static String buildStringFromQName(QName qn) {
        String result = qn.getNamespaceURI() + BPELConstants.NS_SEPARATOR + qn.getLocalPart();
        return result;
    }

    public static String getPackageFromNamespace(String ns) {
        String packag = Utils.makePackageName(ns);
        return packag;
    }

    public static String getDirectoryFromPackage(String packag) {
        String dir = packag.replace('.', File.separatorChar);
        return dir;
    }

    /**
     * generate the BPEL signature to be inserted into generated java
     * source files.
     *
     */
    public static String autograph(String filename) {
        String nl = "\n";
        return ("/**" + nl + " * " + filename + nl + " *" + nl + " * This file was generated by the BPEL engine" + nl + " * on " + new java.util.Date() + nl + " */" + nl + nl);
    }

    public static Emitter buildEmitter(String file) throws BPELUtilException {
        Emitter emitter = new Emitter();
        String outputDir = BPELConstants.BUILD_EMITTER_DIR;
        File output = new File(outputDir);
        output.getAbsoluteFile().getParentFile().mkdirs();
        emitter.setOutputDir(outputDir);
        emitter.setTypeMappingVersion(BPELConstants.AXIS_TYPE_MAPPING_VERSION);
        emitter.setWrapArrays(true);
        try {
            emitter.run(file);
        } catch (Exception e) {
            throw new BPELUtilException("Exception Util buildEmitter : \n" , e);
        }
        Delete d = new Delete();
        d.setProject(new Project());
        d.setDir(output);
        d.execute();
        return emitter;
    }

    public static boolean isBPELNS(String ns) {
        return (BPELConstants.NS_BPEL_V1_1.equals(ns) || BPELConstants.NS_BPEL_V2_0.equals(ns));
    }

    public static String replacePackageByNS(String myType) {
        String result = "";
        StringTokenizer parts = new StringTokenizer(myType,".");
        //System.out.println("ici : myType : " + myType + ", parts : " + parts + ", parts.length : " + parts.countTokens());
        while (parts.hasMoreTokens()) {
            String part = parts.nextToken();
            if (parts.hasMoreTokens()) {
                result = ":" + part + result;
            } else {
                result = "urn" + result + "," + BPELString.toLower(part);
            }
        }
        //System.out.println("end : result : " + result);
        return result;
    }

    public static boolean namespacesEquals(String ns1, String ns2) {
        String pack1 = getPackageFromNamespace(ns1);
        String pack2 = getPackageFromNamespace(ns2);
        return pack1.equals(pack2);
    }
    /**
     * Returns the java type corresponding to the given type QName
     * following the provided SymbolTable
     * @param symbolTable the SymbolTable where to search for the given type
     * @param typeQN the type for which to search for the java type
     * @return String the corresponding java type
     */
    private static String getTypeJavaType(SymbolTable symbolTable, QName typeQN) {
        String javaType = null;
        if (typeQN != null) {
            javaType = WSDLJavaMatching.getJavaType(typeQN);
            if (javaType == null) {
                Type t = symbolTable.getType(typeQN);
                if (t != null) {
                    // Disabled because BPEL 2.0 authorizes ComplexTypes
                    // if ( t.isBaseType() || t.isSimpleType() ) {
                    javaType = t.getName();
                }
            }
        }
        return javaType;
    }

    /**
     * Returns the java type corresponding to the given element QName
     * following the provided SymbolTable
     * @param symbolTable the SymbolTable where to search for the given type
     * @param elementQN the element for which to search for the java type
     * @return String the corresponding java type
     */
    private static String getElementJavaType(SymbolTable symbolTable, QName elementQN) {
        String javaType = null;
        if (elementQN != null) {
            Element e = symbolTable.getElement(elementQN);
            if (e != null) {
                javaType = e.getName();
            }
        }
        return javaType;
    }

    /**
     * Returns the java type corresponding to the given type or element QName
     * following the provided SymbolTable
     * @param symbolTable the SymbolTable where to search for the given type
     * @param typeQN the type for which to search for the java type
     * @param elementQN the element for which to search for the java type
     * @return String the corresponding java type
     */
    public static String getJavaType(SymbolTable symbolTable, QName typeQN, QName elementQN) {
        String javaType = null;
        if (typeQN != null) {
            javaType = getTypeJavaType(symbolTable, typeQN);
        } else if (elementQN != null) {
            javaType = getElementJavaType(symbolTable, elementQN);
        }
        return javaType;
    }

    public static void printNodeAsString (Node n) throws BPELUtilException {
        String st = getNodeAsString(n);
        st = st.replaceAll(">",">\n");
        System.out.println("Begin printNodeAsString");
        System.out.println( st);
        System.out.println("End printNodeAsString");
    }

    public static String getNodeAsString (Node n) throws BPELUtilException {
        String st = null;
        try {
            Source source = new DOMSource(n);
            StringWriter stringWriter = new StringWriter();
            Result res = new StreamResult(stringWriter);
            TransformerFactory fact = TransformerFactory.newInstance();
            Transformer transformer = fact.newTransformer();
            transformer.transform(source, res);
            st = stringWriter.getBuffer().toString();
        } catch (TransformerConfigurationException e) {
            throw new BPELUtilException("Util TransformerConfigurationException caught when printing",e);
        } catch (TransformerException e) {
            throw new BPELUtilException("Util TransformerException caught when printing",e);
        }
        return st;
    }

    public static BPELMessage buildJM (String processName, QName messageQN) throws BPELUtilException {
        BPELMessage jm = null;
        try {
            String messageName = BPELConstants.BM+processName+messageQN.getLocalPart();
            String ns = messageQN.getNamespaceURI();
            String pack = Util.getPackageFromNamespace(ns);
            ClassLoader classL = ClassLoaderMap.getCL(processName);
            Class cl = Class.forName(pack + "." + messageName, true, classL);
            Constructor constr = cl.getConstructor((Class[])null);
            jm = (BPELMessage) constr.newInstance((Object[])null);
        } catch (ClassNotFoundException e) {
            throw new BPELUtilException("ClassNotFoundException Util buildJM :\n" , e);
        } catch (NoSuchMethodException e) {
            throw new BPELUtilException("NoSuchMethodException Util buildJM :\n" , e);
        } catch (IllegalAccessException e) {
            throw new BPELUtilException("IllegalAccessException Util buildJM :\n" , e);
        } catch (InstantiationException e) {
            throw new BPELUtilException("InstantiationException Util buildJM :\n" , e);
        } catch (InvocationTargetException e) {
            throw new BPELUtilException("InvocationTargetException Util buildJM :\n" , e);
        }
        return jm;
    }

    public static String getPartType(BPELMessage bpelmsg, String partName) {
        String partType = null;
        MessageStructure ms = bpelmsg.getStructure();
        PartStructure[] ps = ms.getParts();
        for (int i=0;i<ps.length;i++) {
            if (ps[i].getName().equals(partName)) {
                partType = ps[i].getType();
            }
        }
        return partType;
    }

    public static Object convertNumber(Class destClass, Object fromVal) throws BPELUtilException {
        Object result = fromVal;
        if (java.lang.Number.class.isAssignableFrom(destClass)) {
            String toType = WSDLJavaMatching.getPrimitiveType(destClass.getName());
            result = convertNumber(fromVal,toType);
        }
        return result;
    }

    public static Object convertNumber(BPELMessage bpelmsg, String partName, Object fromVal) throws BPELUtilException {
        String partType = Util.getPartType(bpelmsg,partName);
        return convertNumber(fromVal,partType);
    }

    public static Object convertNumber(Object fromVal, String toType) throws BPELUtilException {
        Object result = null;
        try {
            if (toType.equals("java.math.BigDecimal")) {
                result = new BigDecimal(((Double)convertNumber(fromVal, "double")).doubleValue());
            } else if (toType.equals("java.math.BigInteger")) {
                result = new BigInteger(((Integer)convertNumber(fromVal, "int")).toString());
            } else {
                Method m = fromVal.getClass().getMethod(toType+"Value",(Class[])null);
                result = m.invoke(fromVal,(Object[])null);
            }
        } catch (NoSuchMethodException ex) {
            throw new BPELUtilException("NoSuchMethodException Util convertNumber :\n" , ex);
        } catch (IllegalArgumentException ex) {
            throw new BPELUtilException("IllegalArgumentException Util convertNumber :\n" , ex);
        } catch (IllegalAccessException ex) {
            throw new BPELUtilException("IllegalAccessException Util convertNumber :\n" , ex);
        } catch (InvocationTargetException ex) {
            throw new BPELUtilException("InvocationTargetException Util convertNumber :\n" , ex);
        }
        return result;
    }

    public static Vector getFields(Class cl) {
        Vector result = new Vector();
        Field[] allfields = cl.getDeclaredFields();
        for (int i = 0; i < allfields.length; i++) {
            Field f1 = allfields[i];
            Class f1Class = f1.getType();
            String f1Name = f1.getName();
            //if (!f1Name.startsWith("_")) {
            try {
                cl.getDeclaredMethod("set"+BPELString.toUpper(f1Name),new Class[]{f1Class});
                String getStart = "get";
                if (f1Class.equals(boolean.class)) {
                    getStart="is";
                }
                cl.getDeclaredMethod(getStart+BPELString.toUpper(f1Name),(Class[])null);
                result.addElement(f1);
            } catch (NoSuchMethodException ex) {
                //if one of the get or set/is method does not exist,
                //the field is not a standard field : getter / setter
            }
            //}
        }
        return result;
    }

    public static String getFilePrefix() {
        String prefix = BPELConstants.LINUXPREFIXFILE;
        if (System.getProperty("os.name").toLowerCase().indexOf("windows") != -1) {
            prefix = BPELConstants.WINDOWSPREFIXFILE;
        }
        return prefix;
    }

    public static String getPathSeparator() {
        String result = BPELConstants.LINUX_PATH_SEPARATOR;
        if (System.getProperty("os.name").toLowerCase().indexOf("windows") != -1) {
            result = BPELConstants.WINDOWS_PATH_SEPARATOR;
        }
        return result;
    }

    public static Vector getPrefixFromQuery(String query) {
        Vector result = new Vector();
        String[] parts = query.split("/");
        for (int i=0;i<parts.length;i++) {
            String part = parts[i];
            if (part.length()>0) {
                int j = part.indexOf(":");
                if (j!=-1) {
                    String prefix = part.substring(0,j);
                    if (!result.contains(prefix)) {
                        result.addElement(prefix);
                    }
                }
            }
        }
        return result;
    }

    //  This method writes a DOM document to a file
    public static void writeXmlFile(Document doc, String filename) throws BPELUtilException {
        try {
            // Prepare the DOM document for writing
            Source source = new DOMSource(doc);
            // Prepare the output file
            File file = new File(filename);
            file.getAbsoluteFile().getParentFile().mkdirs();
            FileWriter fileWriter = new FileWriter(file);
            Result result = new StreamResult(fileWriter);
            // Write the DOM document to the file
            Transformer xformer = TransformerFactory.newInstance().newTransformer();
            xformer.transform(source, result);
            fileWriter.close();
        } catch (TransformerConfigurationException e) {
            throw new BPELUtilException("TransformerConfigurationException Util writeXmlFile :\n" , e);
        } catch (TransformerException e) {
            throw new BPELUtilException("TransformerException Util writeXmlFile :\n" , e);
        } catch (IOException e) {
            throw new BPELUtilException("IOException Util writeXmlFile :\n" , e);
        }
    }

    public static String getPrimitiveIdentifier(String type) {
        String result = type;
        String lowerCaseType = BPELString.toLower(type);
        if (WSDLJavaMatching.isJavaPrimitiveType(lowerCaseType)) {
            result = "_"+lowerCaseType;
        }
        return result;
    }

    public static HashMap getSignatures(Binding b, String processName) throws BPELUtilException {
        HashMap hm = new HashMap();
        QName qn = b.getQName();
        String pack = getPackageFromNamespace(qn.getNamespaceURI());
        String dir = getDirectoryFromPackage(pack);
        String finalDir = BPELConstants.WSDL_DIR + File.separator + processName + File.separator + dir;
        String className = BPELString.toUpper(qn.getLocalPart())+"Impl.java";
        String path = finalDir + File.separator + className;
        if (path.startsWith("null")) { throw new BPELUtilException("Util.getSignatures: "+BPELConstants.BPEL_HOME_PROPERTY+" property is not set",null); }
        try {
            File f = new File(path);
            FileReader fr = new FileReader(f);
            BufferedReader br = new BufferedReader(fr);
            String line = br.readLine();
            while (line != null) {
                String signature = new String(line);
                if (line.endsWith("{") && line.indexOf("public ")!=-1 && line.indexOf(" class ") == -1) {
                    //it is a method declaration
                    String methodName = line.substring(line.indexOf("public "));
                    //here line = public <void> or <type> methodName(<parameters>) throws RMIException {
                    methodName = methodName.substring(methodName.indexOf(" ")+1);
                    //here line = <void> or <type> methodName(<parameters>) throws RMIException {
                    methodName = methodName.substring(methodName.indexOf(" ")+1);
                    //here line = methodName(<parameters>) throws RMIException {
                    methodName = methodName.substring(0,methodName.indexOf("("));
                    //here line = methodName
                    hm.put(methodName,signature);
                    //System.out.println("************ methodName = "+methodName+", signature = "+signature);
                }
                line = br.readLine();
            }
            fr.close();
            br.close();
        } catch (FileNotFoundException e) {
            throw new BPELUtilException("FileNotFoundException Util getSignatures : \n" , e);
        } catch (IOException e) {
            throw new BPELUtilException("IOException Util getSignatures : \n" , e);
        }
        return hm;
    }

    public static synchronized long getUniqueRefpartner() {
        return ++id;
    }

    /**
     * Returns a String representation of a Throwable, like would a printStackTrace() do.
     * @param ex the Throwable to represent as a string
     * @return String the representation of the Throwable
     */
    public static String getExceptionAsString(Throwable ex) {
        String excep = "\n";
        while (ex != null) {
            excep += ex.getClass()+" : "+ex.getMessage()+"\n";
            StackTraceElement[] steTab = ex.getStackTrace();
            for (int j=0;j<steTab.length;j++) {
                StackTraceElement ste = steTab[j];
                excep += "  at " + ste.toString() + "\n";
            }
            ex = ex.getCause();
            if (ex != null) {
                excep += "caused by ";
            }
        }
        return excep;
    }

    public static Method getMethodByName(Class cl, String operation) throws BPELUtilException {
        //find a method based on his name
        Method[] tabMet = null;
        tabMet = cl.getDeclaredMethods();
        int i = 0;
        while (i < tabMet.length && !tabMet[i].getName().equals(operation)) {
            i++;
        }
        if (i==tabMet.length) {
            throw new BPELUtilException("Util, getMethodByName, no method called '"+operation+"' found in class '"+cl+"'",null);
        }
        if (BPELTrace.isUtilDebugEnabled())
            BPELTrace.traceUtil("Util end getMethodByName  result = " + tabMet[i].getName());
        return tabMet[i];
    }

    public static Object getNumberAsDouble(Object o) {
        Object result = o;
        if (o != null && o instanceof Number) {
            result = new Double(o.toString());
        }
        return result;
    }

    /**
     * Returns a String representation of this String object suitable for printing into a Stream.
     * If the String object is null, it returns <code>"null"</code>,
     * otherwise, it returns a String representing the String between double quotes.
     * @param s the String to transform into a printable String
     * @return String If the String object is null, it returns <code>"null"</code>,
     * otherwise, it returns a String representing the String between double quotes
     */
    public static final String toQuotedString(String s) {
        return (s == null) ?   "null"  :  "\"" + s + "\"";
    }

    public static void copyFile(File src, File dst) throws BPELUtilException {
        try {
            InputStream in = new FileInputStream(src);
            OutputStream out = new FileOutputStream(dst);

            // Transfer bytes from in to out
            byte[] buf = new byte[1024];
            int len;
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
            in.close();
            out.close();
        } catch (IOException e) {
            throw new BPELUtilException("IOException caught in Util.copyFile : ", e);
        }
    }

    public static String addMessagePort(String wsdlFile, String processName, String host, String port) throws BPELUtilException {
        File wsdl = new File(wsdlFile);
        String ExtProcessJbiDir = BPELConstants.BUILD_JBI_DIR + File.separator + processName;
        File newWsdlDir = new File(ExtProcessJbiDir);
        if (newWsdlDir.exists()) {
            int i = 2;
            while (true) {
                newWsdlDir = new File(ExtProcessJbiDir+new Integer(i));
                if (!newWsdlDir.exists()) {
                    break;
                }
                i++;
            }
        }
        File newWsdl = new File(newWsdlDir.getAbsolutePath() + File.separator + wsdl.getName());
        newWsdlDir.mkdirs();


        try {
            copyFile(wsdl, newWsdl);

            WSDLFactoryImpl wsdlFactory = new WSDLFactoryImpl();
            WSDLReader wsdlReader = wsdlFactory.newWSDLReader();
            Definition def = wsdlReader.readWSDL(newWsdl.getAbsolutePath());
            Map services = def.getServices();
            Set servicesSet = services.entrySet();
            for (Iterator i = servicesSet.iterator(); i.hasNext();) {
                Map.Entry entry0 = (Map.Entry) i.next();
                Service s = (Service) entry0.getValue();
                QName serviceQN = s.getQName();
                QName newServiceQN = new QName(serviceQN.getNamespaceURI(),serviceQN.getLocalPart()+BPELConstants.MESSAGE_PORT_SUFFIX);
                s.setQName(newServiceQN);
                Map ports = s.getPorts();
                Set portsSet = ports.entrySet();
                for (Iterator j = portsSet.iterator(); j.hasNext();) {
                    Map.Entry entry1 = (Map.Entry) j.next();
                    Port p = (Port) entry1.getValue();
                    p.setName(p.getName()+BPELConstants.MESSAGE_PORT_SUFFIX);
                    List extEl = p.getExtensibilityElements();
                    for (int k = 0;k<extEl.size();k++) {
                        ExtensibilityElement el = (ExtensibilityElement)extEl.get(k);
                        if (el instanceof SOAPAddress) {
                            SOAPAddress sa = (SOAPAddress) el;
                            URL locationURI = new URL(sa.getLocationURI());
                            //change host and port to thise defined in BPEL build.properties
                            String newLocation = "http://"+host + ":" + port+locationURI.getPath()+BPELConstants.MESSAGE_PORT_SUFFIX;
                            sa.setLocationURI(newLocation);
                            el = sa;
                        }
                    }
                    Binding b = p.getBinding();
                    QName bindingQN = b.getQName();
                    QName newBindingQN = new QName(bindingQN.getNamespaceURI(),bindingQN.getLocalPart()+BPELConstants.MESSAGE_PORT_SUFFIX);
                    b.setQName(newBindingQN);
                }
            }
            WSDLWriter writer = wsdlFactory.newWSDLWriter();
            Document doc = writer.getDocument(def);
            String toWrite = Util.getNodeAsString(doc);
            FileWriter fw = new FileWriter(newWsdl);
            BufferedWriter bw = new BufferedWriter(fw);
            bw.write(toWrite);
            bw.close();
            fw.close();
        } catch (IOException e) {
            throw new BPELUtilException("IOException Util addMessagePort, processName = "+processName+", wsdlFile =  "+wsdlFile+" :\n" , e);
        } catch (WSDLException e) {
            throw new BPELUtilException("WSDLException Util addMessagePort, processName = "+processName+", wsdlFile =  "+wsdlFile+" :\n" , e);
        } catch (BPELUtilException e) {
            throw new BPELUtilException("BPELUtilException Util addMessagePort, processName = "+processName+", wsdlFile =  "+wsdlFile+" :\n" , e);
        }
        return newWsdl.getAbsolutePath();
    }

}