/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.util;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Operation;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;

import org.objectweb.orchestra.exception.BPELUtilException;

import com.ibm.wsdl.factory.WSDLFactoryImpl;


/**
 * WSDLUtil.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: WSDLUtil.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class WSDLUtil {

    private Definition definition = null;

    public WSDLUtil(String wsdlUrl) throws BPELUtilException {
        WSDLFactoryImpl wsdlFactory = new WSDLFactoryImpl();
        WSDLReader wsdlReader = wsdlFactory.newWSDLReader();
        try {
            this.definition = wsdlReader.readWSDL(wsdlUrl);
        } catch (javax.wsdl.WSDLException e) {
            throw new BPELUtilException("WSDLException WSDLUtil constructor :\n" , e);
        }
    }

    //get a QName for its string representation : {ns}localPart
    public static QName getQNameFromString (String st) {
        int index = st.indexOf("}");
        String ns = st.substring(1,index);
        String localPart = st.substring(index+1);
        return new QName(ns,localPart);
    }

    public Vector getServices () {
        Vector services = new Vector();
        Map servMap = this.definition.getServices();
        Set servSet = servMap.entrySet();
        for (Iterator i = servSet.iterator(); i.hasNext();) {
            Map.Entry entry0 = (Map.Entry) i.next();
            services.addElement(entry0.getValue());
        }
        return services;
    }

    public Service getService (QName serviceQN) {
        return this.definition.getService(serviceQN);
    }

    public PortType getPortType (QName serviceQN,String portName) {
        return getService(serviceQN).getPort(portName).getBinding().getPortType();
    }

    public Vector getPorts (String serviceQNAsString) {
        QName serviceQN = getQNameFromString(serviceQNAsString);
        return getPorts(serviceQN);
    }

    public Vector getPorts (QName serviceQN) {
        Vector ports = new Vector();
        Service service = getService(serviceQN);
        Map portsMap = service.getPorts();
        Set portsSet = portsMap.entrySet();
        for (Iterator i = portsSet.iterator(); i.hasNext();) {
            Map.Entry entry0 = (Map.Entry) i.next();
            ports.addElement(entry0.getValue());
        }
        return ports;
    }

    public Binding getBinding (QName serviceQN,String portName) {
        Port port = getService(serviceQN).getPort(portName);
        return port.getBinding();
    }

    public Vector getOperations (String serviceQNAsString,String portName) {
        QName serviceQN = getQNameFromString(serviceQNAsString);
        return getOperations(serviceQN,portName);
    }

    public Vector getOperations (QName serviceQN,String portName) {
        Vector operations = new Vector();
        Port port = getService(serviceQN).getPort(portName);
        Binding binding = port.getBinding();
        PortType portType = binding.getPortType();

        List operationsList = portType.getOperations();
        for (Iterator j = operationsList.iterator(); j.hasNext();) {
            Operation op = (Operation) j.next();
            operations.addElement(op);
        }
        return operations;
    }

    public Operation getOperation (QName serviceQN,String portName,String operationName) {
        Operation op = null;
        Vector operations = getOperations(serviceQN,portName);
        for (int i=0;i<operations.size() && op==null;i++) {
            Operation tmp = (Operation) operations.get(i);
            if (tmp.getName().equals(operationName)) {
                op = tmp;
            }
        }
        return op;
    }

    public boolean areEquals(QName qn1, QName qn2) {
        boolean b = qn1.getNamespaceURI().equals(qn2.getNamespaceURI());
        b = b && qn1.getLocalPart().equals(qn2.getLocalPart());
        return b;
    }

    public Vector getConcretes(QName ptQN) {
        Vector result = new Vector();
        Vector services = getServices();
        for (int i=0;i<services.size();i++) {
            Service service = (Service) services.get(i);
            QName serviceQN = service.getQName();
            Vector ports = getPorts(serviceQN);
            for (int j=0;j<ports.size();j++) {
                Port port = (Port) ports.get(j);
                String portName = port.getName();
                PortType portType = getPortType(serviceQN,portName);
                QName portTypeQN = portType.getQName();
                if (areEquals(portTypeQN,ptQN)) {
                    result.addElement(serviceQN);
                    result.addElement(portName);
                    result.addElement(port.getBinding().getQName());
                }
            }
        }
        return result;
    }
}


