package org.objectweb.orchestra.tests.restart.restart2;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Vector;

import javax.jms.ConnectionFactory;
import javax.jms.MapMessage;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.InitialContext;



/**
* Copyright (C) 1999 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
**/
public class RestartTest {

    private static String process = null;
    private static Cmd start = null;
    private static Cmd ping = null;
    private static Cmd restart = null;
    private static Cmd initDb = null;
    private static Cmd deployUtil = null;
    private static Cmd deployProcess = null;
    private static Cmd launchProcess = null;

    private static long timeBeforeJonasStart = -1;
    private static boolean printOut = false;
    private static String result = null;

    public static void main(String [] args) {
        try {
            String bpelHome = args[0];
            String jonasRoot = args[1];
            process = args[2];
            long processMaxExecTime = Long.parseLong(args[3]);
            timeBeforeJonasStart = Long.parseLong(args[4]);
            String restartDir = bpelHome + "/tests/restart";
            String engineStopFile = restartDir + "/stopList.txt";

            init(bpelHome,jonasRoot);

            System.setProperty("java.naming.factory.initial","org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory");

            startJonasWithoutWaiting();
            initDb();
            deployUtils();
            deployProcess();

            String stopList = restartDir+"/"+process+"/stopList.txt";
            Vector stops = readStopFile(stopList);
            for (int i=0;i<stops.size();i++) {
                String stop = (String) stops.get(i);
                WriteToEngineStopFile(engineStopFile,stop);

                launchProcess();
                startJonas();
                restartEngine();

                System.out.println("Waiting for message...");
                Context jndi = new InitialContext();
                Queue queue = (Queue) jndi.lookup("BPELRestartQueue");
                javax.jms.ConnectionFactory conFactory = (ConnectionFactory) jndi.lookup("JQCF");
                javax.jms.Connection connection = ((QueueConnectionFactory) conFactory).createQueueConnection();
                javax.jms.Session session = ((QueueConnection) connection).createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
                QueueReceiver rec = ((QueueSession) session).createReceiver(queue);
                connection.start();
                MapMessage message = (MapMessage) rec.receive(processMaxExecTime);
                rec.close();
                session.close();
                connection.close();
                rec = null;
                session = null;
                connection = null;
                if (message != null) {
                    String out = message.getString("message");
                    if (result.equals(out)) {
                        System.out.println("Result received = "+out+", CORRECT ! (stop="+stop+")");
                    } else {
                        System.out.println("Result received = "+out+", WRONG ! (stop="+stop+")");
                        System.out.println("Stopping...");
                        break;
                    }
                    message = null;
                } else {
                    System.out.println("Waiting for message on RestartQueue for "+processMaxExecTime+", no result found.");
                    System.out.println("Stopping...");
                    break;
                }
            }
        } catch (Exception e) {
            System.out.println("RestartTest, exception caught : "+e);
            e.printStackTrace(System.out);
        }
    }

    private static void startJonasWithoutWaiting() throws Exception {
        startJonas(0);
    }

    private static void startJonas() throws Exception {
        waitForJonasStop();
        startJonas(timeBeforeJonasStart);
    }

    private static void startJonas(long t) throws Exception {
        Thread.sleep(t);
        System.out.println("Starting JOnAS");
        start.run();
    }

    private static void initDb() throws Exception {
        System.out.println("Executing initDb");
        initDb.run();
    }

    private static void deployUtils() throws Exception {
        System.out.println("Deploying utils");
        deployUtil.run();
    }

    private static void deployProcess() throws Exception {
        System.out.println("Deploying "+process);
        deployProcess.run();
    }

    private static void launchProcess() throws Exception {
        System.out.println("Launching "+process);
        launchProcess.run();
    }

    private static void restartEngine() throws Exception {
        System.out.println("Restarting Engine");
        restart.run();
    }

    private static Vector readStopFile(String stopFile) {
        Vector result = new Vector();
        try {
            File f = new File(stopFile);
            FileReader fr = new FileReader(f);
            BufferedReader br = new BufferedReader(fr);
            String line = br.readLine();
            String currentLoop = "-1";
            while(line != null) {
                if (line.startsWith("loop=")) {
                    currentLoop = line.substring(5);
                } else if (line.startsWith("nb=")) {
                    String[] nbs = line.substring(3).split(",");
                    for (int j=0;j<nbs.length;j++) {
                        String nb = nbs[j];
                        result.addElement(nb+";"+currentLoop);
                    }
                }
                line = br.readLine();
            }
            fr.close();
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    private static void waitForJonasStop() throws Exception {
        System.out.println("Waiting for JOnAS stop");
        ping.run();
    }

    private static void WriteToEngineStopFile(String engineStopFile, String toWrite) {
        System.out.println("Writing '"+toWrite+"' to stopList");
        try {
            File f = new File(engineStopFile);
            f.getAbsoluteFile().getParentFile().mkdirs();
            FileWriter fw = new FileWriter(f);
            BufferedWriter bw = new BufferedWriter(fw);
            bw.write(toWrite);
            bw.newLine();
            bw.close();
            fw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void init(String bpelHome, String jonasRoot) {
        String orchestra = bpelHome+"/bin/unix/orchestra";
        String jonas = jonasRoot+"/bin/unix/jonas";

        ping = new Cmd(jonas+" admin -n BPEL admin -ping",printOut);
        start = new Cmd(orchestra+" start",printOut);
        restart = new Cmd(orchestra+" restartEngine",printOut);
        initDb = new Cmd(orchestra+" initDb",printOut);
        deployUtil = new Cmd(orchestra+" deployWS -p InvokeUtil -wsdl "+bpelHome+"/tests/org/objectweb/orchestra/tests/util/InvokeUtil/InvokeUtil.wsdl -srcDir "+bpelHome+"/tests/org/objectweb/orchestra/tests/util/InvokeUtil/",printOut);

        String deployRestart1 = orchestra+" deploy -p restart1 -wsdl "+bpelHome+"/tests/org/objectweb/orchestra/tests/restart/restart1/restart1.wsdl -bpel "+bpelHome+"/tests/org/objectweb/orchestra/tests/restart/restart1/restart1.bpel -srcDir "+bpelHome+"/tests/org/objectweb/orchestra/tests/restart/restart1 "+bpelHome+"/tests/org/objectweb/orchestra/tests/util/InvokeUtil/InvokeUtil.wsdl";
        String launchRestart1 = orchestra+" launch -p restart1 -cc org.objectweb.orchestra.tests.restart.restart1.Restart1Client 10";
        String resultRestart1 = "10BCABCABCBC";

        String deployRestart2 = orchestra+" deploy -p restart2 -wsdl "+bpelHome+"/tests/org/objectweb/orchestra/tests/restart/restart2/restart2.wsdl -bpel "+bpelHome+"/tests/org/objectweb/orchestra/tests/restart/restart2/restart2.bpel -srcDir "+bpelHome+"/tests/org/objectweb/orchestra/tests/restart/restart2 "+bpelHome+"/tests/org/objectweb/orchestra/tests/util/InvokeUtil/InvokeUtil.wsdl";
        String launchRestart2 = orchestra+" launch -p restart2 -cc org.objectweb.orchestra.tests.restart.restart2.Restart2Client";
        String resultRestart2 = "restart2_BCDEABCDEABCDEO";

        deployProcess = null;
        launchProcess = null;
        if (process.equals("restart1")) {
            deployProcess = new Cmd(deployRestart1,printOut);
            launchProcess = new Cmd(launchRestart1,printOut);
            result = resultRestart1;
        } else if (process.equals("restart2")) {
            deployProcess = new Cmd(deployRestart2,printOut);
            launchProcess = new Cmd(launchRestart2,printOut);
            result = resultRestart2;
        }
    }
}


