package org.objectweb.orchestra.tests.unit.unitReply;

import javax.xml.rpc.holders.IntHolder;
import javax.xml.rpc.holders.LongHolder;
import javax.xml.rpc.holders.StringHolder;

import org.objectweb.orchestra.util.BPELConstants;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitReply.Adresse;
import org.objectweb.orchestra.tests.unit.unitReply.Num;
import org.objectweb.orchestra.tests.unit.unitReply.Person;
import org.objectweb.orchestra.tests.unit.unitReply.Reply;
import org.objectweb.orchestra.tests.unit.unitReply.Rue;
import org.objectweb.orchestra.tests.unit.unitReply.UnitReplyServiceBP;
import org.objectweb.orchestra.tests.unit.unitReply.UnitReplyServiceBPLocator;
import org.objectweb.orchestra.tests.unit.unitReply.holders.AdresseHolder;
import org.objectweb.orchestra.tests.unit.unitReply.holders.PersonHolder;


public class TestReply extends TestCase {

    private boolean tmp;

    private UnitReplyServiceBP fs = null;
    private Reply fpt = null;

    private Adresse in = null;
    private int integer;
    private String st = null;
    private Person person =  null;

    public TestReply(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestReply.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.tmp = false;
        this.fs = new UnitReplyServiceBPLocator();
        this.fpt = this.fs.getReply();

        this.in = new Adresse();
        this.in.setVille("seyssinet");
        this.in.setCp(38170);
        Rue rue = new Rue();
        rue.setNom("Rousseau");
        Num num = new Num();
        num.setNum1(12);
        num.setNum2(120000);
        rue.setNum(num);
        this.in.setRue(rue);
        this.integer = 2;
        this.st = new String("input string");
        this.person =  new Person();
        this.person.setAdresse(this.in);
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestReply(testtorun));
        }
    }

    public void testReply1() throws Exception {
        //<from variable="in" part="inadr"/> <to variable="outadresse1" part="outadr"/>
        //<!-- Reply with a message containing only one complex part -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply1" variable="outadresse1"/>
        Adresse a1 = this.fpt.reply1(this.in,this.integer,this.st,this.person,"reply1");
        this.tmp = a1.equals(this.in);
        assertTrue("Reply1 error",this.tmp);
    }

    public void testReply2() throws Exception {
        //<from variable="in" part="string"/> <to variable="outstring1" part="st"/>
        //<!-- Reply with a message containing only one simple part -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply2" variable="outstring1"/>
        String s1 = this.fpt.reply2(this.in,this.integer,this.st,this.person,"reply2");
        this.tmp = s1.equals(this.st);
        assertTrue("Reply2 error",this.tmp);
    }

    public void testReply3() throws Exception {
        //<from variable="in" part="number"/> <to variable="outint1" part="int1"/>
        //<!-- Reply with a message containing only one primitive part -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply3" variable="outint1"/>
        int i1 = this.fpt.reply3(this.in,this.integer,this.st,this.person,"reply3");
        this.tmp = (i1 == this.integer);
        assertTrue("Reply3 error",this.tmp);
    }

    public void testReply4() throws Exception {
        //<from variable="in" part="inadr"/> <to variable="outadresse2" part="outadr"/>
        //<from variable="in" part="person"/> <to variable="outadresse2" part="outperson"/>
        //<!-- Reply with a message containing several complex parts -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply4" variable="outadresse2"/>
        AdresseHolder adrh = new AdresseHolder();
        PersonHolder persh = new PersonHolder();
        this.fpt.reply4(this.in,this.integer,this.st,this.person,"reply4",adrh,persh);
        this.tmp = adrh.value.equals(this.in) && persh.value.equals(this.person);
        assertTrue("Reply4 error",this.tmp);
    }

    public void testReply5() throws Exception {
        //<from variable="in" part="string"/> <to variable="outstring2" part="st"/>
        //<from variable="in" part="string"/> <to variable="outstring2" part="st2"/>
        //<!-- Reply with a message containing several simple parts -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply5" variable="outstring2"/>
        StringHolder st1 = new StringHolder();
        StringHolder st2 = new StringHolder();
        this.fpt.reply5(this.in,this.integer,this.st,this.person,"reply5",st1,st2);
        this.tmp = st1.value.equals(this.st) && st2.value.equals(this.st);
        assertTrue("Reply5 error",this.tmp);
    }

    public void testReply6() throws Exception {
        //<from variable="in" part="number"/> <to variable="outint2" part="int1"/>
        //<from variable="in" part="inadr" query="/inadr/rue/num/num2"/> <to variable="outint2" part="long1"/>
        //<!-- Reply with a message containing several primitive parts -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply6" variable="outint2"/>
        IntHolder ih1 = new IntHolder();
        LongHolder lh1 = new LongHolder();
        this.fpt.reply6(this.in,this.integer,this.st,this.person,"reply6",ih1,lh1);
        this.tmp = ih1.value == this.integer && lh1.value == this.in.getRue().getNum().getNum2();
        assertTrue("Reply6 error",this.tmp);
    }

    public void testReply7() throws Exception {
        //<from variable="in" part="inadr"/> <to variable="outadresse3" part="inadr"/>
        //<!-- Reply with a message containing only one part. This one is in inout mode (complex) -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply7" variable="outadresse3"/>
        AdresseHolder adrh = new AdresseHolder(this.in);
        this.fpt.reply7(adrh,this.integer,this.st,this.person,"reply7");
        this.tmp = adrh.value.equals(this.in);
        assertTrue("Reply7 error",this.tmp);
    }

    public void testReply8() throws Exception {
        //<from variable="in" part="string"/> <to variable="outstring3" part="string"/>
        //<!-- Reply with a message containing only one part. This one is in inout mode (simple) -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply8" variable="outstring3"/>
        StringHolder sth = new StringHolder(this.st);
        this.fpt.reply8(this.in,this.integer,sth,this.person,"reply8");
        this.tmp = sth.value.equals(this.st);
        assertTrue("Reply8 error",this.tmp);
    }

    public void testReply9() throws Exception {
        //<from variable="in" part="number"/> <to variable="outint3" part="number"/>
        //<!-- Reply with a message containing only one part. This one is in inout mode (primitive) -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply9" variable="outint3"/>
        IntHolder ih = new IntHolder(this.integer);
        this.fpt.reply9(this.in,ih,this.st,this.person,"reply9");
        this.tmp = ih.value == this.integer;
        assertTrue("Reply9 error",this.tmp);
    }

    public void testReply10() throws Exception {
        //<from variable="in" part="inadr"/> <to variable="outcomplex" part="inadr"/>
        //<from variable="in" part="number"/> <to variable="outcomplex" part="int1"/>
        //<from variable="in" part="string"/> <to variable="outcomplex" part="st"/>
        //<from variable="in" part="person"/> <to variable="outcomplex" part="per"/>
        //<!-- Reply with a message containing several parts. These one are common or not and complex, simple or primitive -->
        //<reply partnerLink="client" portType="tns:Reply" operation="reply10" variable="outcomplex"/>
        AdresseHolder inadrh = new AdresseHolder(this.in);
        IntHolder inth = new IntHolder();
        StringHolder stringh = new StringHolder();
        PersonHolder personh = new PersonHolder();
        this.fpt.reply10(inadrh,this.integer,this.st,this.person,"reply10",inth,stringh,personh);
        this.tmp = inadrh.value.equals(this.in) && inth.value == this.integer && stringh.value.equals(this.st) && personh.value.equals(this.person);
        assertTrue("Reply10 error",this.tmp);
    }
}



