package org.objectweb.orchestra.tests.unit.unitWsdl;

import org.objectweb.orchestra.util.BPELConstants;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitWsdl.WsdlPT;
import org.objectweb.orchestra.tests.unit.unitWsdl.WsdlPerson;
import org.objectweb.orchestra.tests.unit.unitWsdl.WsdlServiceBP;
import org.objectweb.orchestra.tests.unit.unitWsdl.WsdlServiceBPLocator;


public class TestWsdl extends TestCase {

    private boolean tmp;

    private WsdlPT rpcEncodedPT = null;
    private WsdlPT rpcLiteralPT = null;
    private WsdlPT documentLiteralPT = null;
    private WsdlPT documentEncodedPT = null;

    private String in1 = "in1";
    private int out1 = -1;
    private WsdlPerson p = null;
    private WsdlPerson out = null;

    public TestWsdl(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestWsdl.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.tmp = false;
        WsdlServiceBP wsdl = new WsdlServiceBPLocator();
        this.rpcEncodedPT = wsdl.getrpcEncodedPT();
        this.rpcLiteralPT = wsdl.getrpcLiteralPT();
        this.documentLiteralPT = wsdl.getdocumentLiteralPT();
        this.documentEncodedPT = wsdl.getdocumentEncodedPT();

        this.p = new WsdlPerson();
        this.p.setName("personName");
        this.p.setAge(48);
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestWsdl(testtorun));
        }
    }

    public void tesRpcEncoded1() throws Exception {
        this.out1 = this.rpcEncodedPT.oper1(this.in1);
        this.tmp = (this.out1 == 1);
        assertTrue("rpcEncoded1 error, out1 = "+this.out1+" instead of 1",this.tmp);
    }

    public void testRpcLiteral1() throws Exception {
        this.out1 = this.rpcLiteralPT.oper1(this.in1);
        this.tmp = (this.out1 == 1);
        assertTrue("rpcLiteral1 error, out1 = "+this.out1+" instead of 1",this.tmp);
    }

    public void testDocumentLiteral1() throws Exception {
        this.out1 = this.documentLiteralPT.oper1(this.in1);
        this.tmp = (this.out1 == 1);
        assertTrue("documentLiteral1 error, out1 = "+this.out1+" instead of 1",this.tmp);
    }

    public void testDocumentEncoded1() throws Exception {
        this.out1 = this.documentEncodedPT.oper1(this.in1);
        this.tmp = (this.out1 == 1);
        assertTrue("documentEncoded1 error, out1 = "+this.out1+" instead of 1",this.tmp);
    }

    public void testRpcEncoded2() throws Exception {
        this.out = this.rpcEncodedPT.oper2(this.p);
        this.tmp = this.out.getName().equals("personName") && this.out.getAge()==48;
        assertTrue("rpcEncoded2 error, out.getName() = "+this.out.getName()+" instead of personName or out.getAge() = "+this.out.getAge()+" instead of 48",this.tmp);
    }

    public void testRpcLiteral2() throws Exception {
        this.out = this.rpcLiteralPT.oper2(this.p);
        this.tmp = this.out.getName().equals("personName") && this.out.getAge()==48;
        assertTrue("rpcLiteral2 error, out.getName() = "+this.out.getName()+" instead of personName or out.getAge() = "+this.out.getAge()+" instead of 48",this.tmp);
    }

    public void _testDocumentLiteral2() throws Exception {
        //impossible to use document style with complex types that are not elements enclosed
        /*
         out = documentLiteralPT.oper2(p);
         tmp = out.getName().equals("personName") && out.getAge()==48;
         assertTrue("documentLiteral2 error, out.getName() = "+out.getName()+" instead of personName or out.getAge() = "+out.getAge()+" instead of 48",tmp);
         */
    }

    public void _testDocumentEncoded2() throws Exception {
        //impossible to use document style with complex types that are not elements enclosed
        /*
         out = documentEncodedPT.oper2(p);
         tmp = out.getName().equals("personName") && out.getAge()==48;
         assertTrue("documentEncoded2 error, out.getName() = "+out.getName()+" instead of personName or out.getAge() = "+out.getAge()+" instead of 48",tmp);
         */
    }

    public void testRpcEncoded3() throws Exception {
        this.out = this.rpcEncodedPT.oper3(this.p);
        this.tmp = this.out.getName().equals("personName") && this.out.getAge()==48;
        assertTrue("rpcEncoded3 error, out.getName() = "+this.out.getName()+" instead of personName or out.getAge() = "+this.out.getAge()+" instead of 48",this.tmp);
    }

    public void testRpcLiteral3() throws Exception {
        this.out = this.rpcLiteralPT.oper3(this.p);
        this.tmp = this.out.getName().equals("personName") && this.out.getAge()==48;
        assertTrue("rpcLiteral3 error, out.getName() = "+this.out.getName()+" instead of personName or out.getAge() = "+this.out.getAge()+" instead of 48",this.tmp);
    }

    public void testDocumentLiteral3() throws Exception {
        this.out = this.documentLiteralPT.oper3(this.p);
        this.tmp = this.out.getName().equals("personName") && this.out.getAge()==48;
        assertTrue("documentLiteral3 error, out.getName() = "+this.out.getName()+" instead of personName or out.getAge() = "+this.out.getAge()+" instead of 48",this.tmp);
    }

    public void testDocumentEncoded3() throws Exception {
        this.out = this.documentEncodedPT.oper3(this.p);
        this.tmp = this.out.getName().equals("personName") && this.out.getAge()==48;
        assertTrue("documentEncoded3 error, out.getName() = "+this.out.getName()+" instead of personName or out.getAge() = "+this.out.getAge()+" instead of 48",this.tmp);
    }
}



