package org.objectweb.orchestra.tests.unit.unitXPath2;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.xml.rpc.holders.StringHolder;

import org.objectweb.orchestra.util.BPELConstants;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitXPath2.InitiateXPathResponseType;
import org.objectweb.orchestra.tests.unit.unitXPath2.InitiateXPathType;
import org.objectweb.orchestra.tests.unit.unitXPath2.XPathPT2;
import org.objectweb.orchestra.tests.unit.unitXPath2.XPathServiceBP2;
import org.objectweb.orchestra.tests.unit.unitXPath2.XPathServiceBP2Locator;
import org.objectweb.orchestra.tests.unit.unitXPath2.autoloan.CustomerAddress;
import org.objectweb.orchestra.tests.unit.unitXPath2.autoloan.CustomerProfile;
import org.objectweb.orchestra.tests.unit.unitXPath2.autoloan.InvalidApplication;
import org.objectweb.orchestra.tests.unit.unitXPath2.autoloan.LoanApplication;
import org.objectweb.orchestra.tests.unit.unitXPath2.StringList;


public class TestXPath2 extends TestCase {


    private String error = "error";
    private String name = "name";
    private int age = 24;
    private String email = "charles.souillard@bull.net";
    private String ssn = "212-5698";
    private String street1 = "Provence";
    private String state1 = "France";
    private String city1 = "Grenoble";
    private String zipCode1 = "38130";
    private String street2 = "Provence2";
    private String state2 = "France2";
    private String city2 = "Grenoble2";
    private String zipCode2 = "381302";
    private double loanAmount = 100.0;
    private String carModel = "Vectra";
    private String carYear = "2005";
    private int creditRating = 200;
    private String custId = "souillac";
    private Calendar cal = null;
    private InitiateXPathType in = null;
    private boolean result;

    private XPathServiceBP2 fs = null;
    private XPathPT2 fpt = null;

    public TestXPath2(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestXPath2.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.fs = new XPathServiceBP2Locator();
        this.fpt = this.fs.getXPathPort2();

        this.result = true;

        this.cal = new GregorianCalendar();
        this.cal.setTime(new Date());

        this.in = new InitiateXPathType();
        InvalidApplication ia = new InvalidApplication();
        LoanApplication la = new LoanApplication();
        CustomerProfile cp = new CustomerProfile();
        ia.setError(this.error);
        cp.setName(this.name);
        cp.setAge(this.age);
        cp.setBirthDate(this.cal);
        cp.setEmail(this.email);
        cp.setSSN(this.ssn);
        cp.setCustId(this.custId);

        CustomerAddress ca1 = new CustomerAddress();
        ca1.setStreet(this.street1);
        ca1.setState(this.state1);
        ca1.setCity(this.city1);
        ca1.setZipCode(this.zipCode1);

        CustomerAddress ca2 = new CustomerAddress();
        ca2.setStreet(this.street2);
        ca2.setState(this.state2);
        ca2.setCity(this.city2);
        ca2.setZipCode(this.zipCode2);

        cp.setAddresses(new CustomerAddress[]{ca1,ca2});
        la.setCustomer(cp);
        la.setLoanAmount(this.loanAmount);
        la.setCarModel(this.carModel);
        la.setCarYear(this.carYear);
        la.setCreditRating(this.creditRating);
        ia.setApplication(la);

        this.in.setXmlInvalidLoanApp(ia);
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestXPath2(testtorun));
        }
    }

    private void loanApplicationEq(LoanApplication la) {
        this.result = this.result && la.getLoanAmount() == this.loanAmount;
        this.result = this.result && la.getCarModel().equals(this.carModel);
        this.result = this.result && la.getCarYear().equals(this.carYear);
        this.result = this.result && la.getCreditRating() == this.creditRating;
        customerProfileEq(la.getCustomer());
    }

    private void customerProfileEq(CustomerProfile cp) {
        Calendar outCal = cp.getBirthDate();
        int year = Calendar.YEAR;
        int month = Calendar.MONTH;
        int day = Calendar.DAY_OF_MONTH;
        this.result = this.result && cp.getName().equals(this.name);
        this.result = this.result && cp.getAge() == this.age;
        this.result = this.result && outCal.get(year)==this.cal.get(year) && outCal.get(month)==this.cal.get(month) && outCal.get(day)==this.cal.get(day);
        this.result = this.result && cp.getEmail().equals(this.email);
        this.result = this.result && cp.getSSN().equals(this.ssn);
        this.result = this.result && cp.getCustId().equals(this.custId);
        customerAddressEq(cp.getAddresses()[0],0);
        customerAddressEq(cp.getAddresses()[1],1);
    }

    private void customerAddressEq(CustomerAddress ca, int index) {
        if (index == 0) {
            this.result = this.result && ca.getStreet().equals(this.street1);
            this.result = this.result && ca.getState().equals(this.state1);
            this.result = this.result && ca.getCity().equals(this.city1);
            this.result = this.result && ca.getZipCode().equals(this.zipCode1);
        } else if (index == 1) {
            this.result = this.result && ca.getStreet().equals(this.street2);
            this.result = this.result && ca.getState().equals(this.state2);
            this.result = this.result && ca.getCity().equals(this.city2);
            this.result = this.result && ca.getZipCode().equals(this.zipCode2);
        } else {
            this.result = false;
        }
    }

    public void testXPath2_1() throws Exception {
        int out = this.fpt.op1(this.in);
        this.result = this.result && out == this.creditRating;
        assertTrue("op1 error",this.result);
    }

    public void testXPath2_2() throws Exception {
        double out = this.fpt.op2(this.in);
        this.result = this.result && out == this.loanAmount;
        assertTrue("op2 error",this.result);
    }

    public void testXPath2_3() throws Exception {
        LoanApplication out = this.fpt.op3(this.in);
        loanApplicationEq(out);
        assertTrue("op3 error",this.result);
    }

    public void testXPath2_4() throws Exception {
        CustomerProfile out = this.fpt.op4(this.in);
        customerProfileEq(out);
        assertTrue("op4 error",this.result);
    }

    public void testXPath2_5() throws Exception {
        CustomerAddress out = this.fpt.op5(this.in);
        customerAddressEq(out,0);
        assertTrue("op5 error",this.result);
    }

    public void testXPath2_6() throws Exception {
        String out = this.fpt.op6(this.in);
        this.result = this.result && out.equals(this.street2);
        assertTrue("op6 error",this.result);
    }

    public void testXPath2_7() throws Exception {
        String out = this.fpt.op7(this.in);
        this.result = this.result && out.equals(this.custId);
        assertTrue("op7 error",this.result);
    }

    public void testXPath2_8() throws Exception {
        String out = this.fpt.op8(this.in);
        this.result = this.result && out.equals(this.name+" some thing");
        assertTrue("op8 error",this.result);
    }

    public void testXPath2_9() throws Exception {
        String out = this.fpt.op9(this.in);
        this.result = this.result && out.equals(this.custId+" huhuhuhu");
        assertTrue("op9 error",this.result);
    }

    public void testXPath2_10() throws Exception {
        InitiateXPathResponseType out = this.fpt.op10(this.in);
        this.result = this.result && out.getResult().getCustomer().getAddresses()[0].getStreet().equals(this.street2);
        assertTrue("op10 error",this.result);
    }

    public void testXPath2_11() throws Exception {
        InitiateXPathResponseType out = this.fpt.op11(this.in);
        loanApplicationEq(out.getResult());
        assertTrue("op11 error",this.result);
    }

    public void testXPath2_12() throws Exception {
        InitiateXPathResponseType out = this.fpt.op12(this.in);
        this.result = this.result && out.getResult().getLoanAmount() == this.loanAmount;
        assertTrue("op12 error",this.result);
    }

    public void testXPath2_13() throws Exception {
        InitiateXPathResponseType out = this.fpt.op13(this.in);
        this.result = this.result && out.getResult().getCustomer().getName().equals(this.name+" some thing");
        assertTrue("op13 error",this.result);
    }

    public void testXPath2_14() throws Exception {
        InitiateXPathResponseType out = this.fpt.op14(this.in);
        this.result = this.result && out.getResult().getCustomer().getCustId().equals(this.custId);
        assertTrue("op14 error",this.result);
    }

    public void testXPath2_15() throws Exception {
        StringList slIn = new StringList(new String[]{"Bill","Bob"});
        StringHolder item1 = new StringHolder();
        StringHolder item2 = new StringHolder();
        this.fpt.op15(slIn,item1,item2);
        this.result = this.result && item1.value.equals(slIn.getItems()[1]);
        assertTrue("op15 error, item1 != slIn.items[2]",this.result);
        this.result = this.result && item2.value.equals(slIn.getItems()[0]);
        assertTrue("op15 error, item2 != slIn.items[1]",this.result);
    }

    public void testXPath2_16() throws Exception {
        StringList slIn = new StringList(new String[]{"Bill","Bob"});
        StringList slOut = this.fpt.op16(slIn);
        this.result = this.result && slOut.getItems()[2].equals(slIn.getItems()[2]);
        assertTrue("op16 error, slOut.items[1] != slIn.items[1]",this.result);
        this.result = this.result && slOut.getItems()[1].equals(slIn.getItems()[1]);
        assertTrue("op16 error, slOut.items[2] != slIn.items[2]",this.result);
    }

}
