package xmlIndenter;

import java.io.File;
import java.io.FileWriter;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


public class Main {

    private static int spacesStep = -1;
    private static String ret = "\n";
    private static int maxAttrByLine = -1;
    private static HashMap attrs = null;
    private static HashMap beforeChildren = null;
    private static HashMap afterNode = null;

    public static void main(String args[]) {
        int argNeeded = 8;
        if (args.length != argNeeded) {
            System.out.println("Number of arguments needed : "+argNeeded+", args.length found : "+args.length+", exiting...");
            System.exit(0);
        }

        String inFile = args[0];
        String outFile = args[1];
        String eraseOut = args[2];
        spacesStep = Integer.parseInt(args[3]);
        maxAttrByLine = Integer.parseInt(args[4]);
        fillAttrs(args[5]);
        fillBeforeChildren(args[6]);
        fillAfterNode(args[7]);

        System.out.println("Running XmlIndenter, inFile = "+inFile+", outFile = "+outFile+"...");

        try {
            printOut(eraseOut,inFile,outFile);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("ERROR : exiting...");
        }
        System.out.println("Finishing XmlIndenter...");
    }

    private static void fillAttrs (String st) {
        attrs = new HashMap();
        String[] tab = st.split(",");
        for (int i=0;i<tab.length-1;i=i+2) {
            attrs.put(tab[i],new Integer(tab[i+1]));
        }
    }

    private static void fillBeforeChildren (String st) {
        beforeChildren = new HashMap();
        String[] tab = st.split(",");
        for (int i=0;i<tab.length;i++) {
            beforeChildren.put(tab[i],"");
        }
    }

    private static void fillAfterNode (String st) {
        afterNode = new HashMap();
        String[] tab = st.split(",");
        for (int i=0;i<tab.length;i++) {
            afterNode.put(tab[i],"");
        }
    }

    private static void printOut (String eraseOut, String inFile, String outFile) throws Exception {
        File in = new File(inFile);
        if (!in.exists()) {
            throw new Exception("In File : "+inFile+" does not exist");
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = dbf.newDocumentBuilder();
        Document doc = docBuilder.parse(in);

        File out = new File(outFile);
        if (out.exists()) {
            if (eraseOut.equals("true")) {
                System.out.println("Deleting outFile : "+outFile+"...");
                out.delete();
            } else {
                System.out.println("OutFile exists and must not be erased. Nothing done, exiting...");
                System.exit(0);
            }
        }
        out.createNewFile();
        System.out.println("Creating outFile : "+outFile+"...");

        FileWriter fw = new FileWriter(out);
        String spaces = "";
        writeOut(doc,spaces,fw);
        fw.close();
    }

    private static void writeOut(Node n,String spaces, FileWriter fw) throws Exception {
        if (beforeChildren.get(n.getNodeName()) != null) {
            write(fw,"");
        }
        for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {
            if (child.getNodeType() == Node.ELEMENT_NODE) {
                writeElement(fw,(Element)child,spaces);
            } else if (child.getNodeType() == Node.COMMENT_NODE) {
                write(fw,spaces+"<!--"+child.getNodeValue()+"-->");
            }
        }
    }

    private static void write (FileWriter fw, String line) throws Exception {
        fw.write(line+ret);
    }

    private static String getStartElement(String spaces, Element n) {
        String result = "";
        result += spaces+"<"+n.getNodeName();
        NamedNodeMap map = n.getAttributes();
        if (map != null) {
            Node att = null;
            int cpt = 0;
            int maxAttr = maxAttrByLine;
            Object o = attrs.get(n.getNodeName());
            if (o != null) {
                maxAttr = ((Integer)o).intValue();
            }
            for (int i = 0; i < map.getLength(); i++) {
                if (cpt == maxAttr) {
                    result += "\n" + spaces + getSpaces(n.getNodeName().length()+1);
                    cpt = 0;
                }
                att = map.item(i);

                String value = att.getNodeValue();
                value = value.replaceAll("<","&lt;");
                value = value.replaceAll(">","&gt;");

                result += " ";
                result += att.getNodeName();
                result += "=";
                result += "\"";
                result += value;
                result += "\"";
                cpt++;
            }
        }
        result += ">";
        return result;
    }

    private static String getSpaces (int a) {
        String result = "";
        for (int i=0;i<a;i++) {
            result += " ";
        }
        return result;
    }

    private static void writeElement(FileWriter fw, Element n, String spaces) throws Exception {
        int children = getChildrenNb(n);
        if (children == 0) {
            String tmp = getStartElement(spaces,n);
            NodeList textChildren = n.getChildNodes();
            if (textChildren.getLength() == 0) {
                tmp = tmp.substring(0,tmp.length()-1);
                tmp += "/>";
            } else {
                tmp += textChildren.item(0);
                tmp += "</"+n.getNodeName()+">";
            }
            write(fw,tmp);
        } else {
            write(fw,getStartElement(spaces,n));
            writeOut(n,incrSpaces(spaces),fw);
            write(fw,spaces+"</"+n.getNodeName()+">");
        }
        if (afterNode.get(n.getNodeName()) != null) {
            write(fw,"");
        }
    }

    private static int getChildrenNb (Node n) {
        int result = 0;
        for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {
            if (child.getNodeType() == Node.ELEMENT_NODE || child.getNodeType() == Node.COMMENT_NODE) {
                result++;
            }
        }
        return result;
    }

    private static String incrSpaces (String spaces) {
        String newSpaces = spaces;
        for (int i=0;i<spacesStep;i++) {
            newSpaces += " ";
        }
        return newSpaces;
    }

}